@@ -339,8 +339,8 @@ static void hide_screen(
  *              put a fvwm frame on the window
  *
  *  Returned Value:
- *      TRUE    - go ahead and frame the window
- *      FALSE   - don't frame the window
+ *      1   - go ahead and frame the window
+ *      0   - don't frame the window
  *
  *  Inputs:
  *      w       - the window to check
@@ -359,7 +359,7 @@ static int MappedNotOverride(
 	win_opts->initial_state = DontCareState;
 	if ((w==Scr.NoFocusWin)||(!XGetWindowAttributes(dpy, w, &wa)))
 	{
-		return False;
+		return 0;
 	}
 	if (XGetWindowProperty(
 		    dpy,w,_XA_WM_STATE,0L,3L,False,_XA_WM_STATE,
@@ -374,7 +374,9 @@ static int MappedNotOverride(
 	if (wa.override_redirect == True)
 	{
 		XSelectInput(dpy, w, XEVMASK_ORW);
+		XFlush(dpy);
 	}
+
 	return (((win_opts->initial_state == IconicState) ||
 		 (wa.map_state != IsUnmapped)) &&
 		(wa.override_redirect != True));
@@ -474,6 +476,12 @@ static void setup_window_structure(
 		CLEAR_USER_STATES(*pfw, ~0);
 		SET_USER_STATES(*pfw, GET_USER_STATES(savewin));
 	}
+	else
+	{
+		/* make sure that new windows *remember* being shaded with
+		 * title dir last */
+		SET_USED_TITLE_DIR_FOR_SHADING(*pfw,1);
+	}
 
 	(*pfw)->cmap_windows = (Window *)NULL;
 	if (FMiniIconsSupported)
@@ -1455,6 +1463,64 @@ static void setup_key_and_button_grabs(FvwmWindow *fw)
 	return;
 }
 
+static void __add_window_handle_x_resources(FvwmWindow *fw)
+{
+	int client_argc = 0;
+	char **client_argv = NULL;
+	XrmValue rm_value;
+	XrmDatabase db = NULL;
+	static XrmOptionDescRec table [] = {
+		{"-xrn", NULL, XrmoptionResArg, (caddr_t) NULL},
+		{"-xrm", NULL, XrmoptionResArg, (caddr_t) NULL},
+	};
+	/* Get global X resources */
+	MergeXResources(dpy, &db, False);
+	/* Find out if the client requested a specific style on the command
+	 * line.
+	 */
+	if (XGetCommand(dpy, FW_W(fw), &client_argv, &client_argc))
+	{
+		if (client_argc > 0 && client_argv != NULL)
+		{
+			/* command line takes precedence over all */
+			MergeCmdLineResources(
+				&db, table, 2, fw->class.res_name,
+				&client_argc, client_argv, True);
+		}
+	}
+
+
+	/* parse the database values */
+	if (GetResourceString(db, "fvwmstyle", fw->class.res_name, &rm_value)
+	    && rm_value.size != 0)
+	{
+		char *style_name;
+		int name_len;
+		style_name = rm_value.addr;
+		name_len = rm_value.size-1;
+		/* Trim spaces at the start of the name */
+		while (name_len>0 && isspace(*style_name))
+		{
+			style_name++;
+			name_len--;
+		}
+		/* Trim spaces at the end of the name */
+		while (name_len>0 && isspace(*(style_name+name_len-1)))
+		{
+			name_len--;
+		}
+		if (name_len>0) {
+			fw->style_name = (char*)safemalloc(sizeof(char)*
+							   (name_len+1));
+			memcpy(fw->style_name,style_name,name_len);
+			fw->style_name[name_len] = 0;
+		}
+	}
+	XFreeStringList(client_argv);
+	XrmDestroyDatabase(db);
+	return;
+}
+
 /* ---------------------------- interface functions ------------------------ */
 
 void setup_visible_name(FvwmWindow *fw, Bool is_icon)
@@ -2043,6 +2109,7 @@ FvwmWindow *AddWindow(
 	setup_class_and_resource(fw);
 
 	/****** style setup ******/
+	__add_window_handle_x_resources(fw);
 	/* get merged styles */
 	lookup_style(fw, &style);
 	sflags = SGET_FLAGS_POINTER(style);
@@ -2060,6 +2127,10 @@ FvwmWindow *AddWindow(
 				fw->class.res_class);
 		}
 		free_window_names(fw, True, True);
+		if (fw->style_name)
+		{
+			free(fw->style_name);
+		}
 		free(fw);
 		MyXUngrabServer(dpy);
 		return AW_UNMANAGED;
@@ -2108,7 +2179,7 @@ FvwmWindow *AddWindow(
 	SET_MAXIMIZED(fw, 0);
 	/*
 	 * Reparenting generates an UnmapNotify event, followed by a MapNotify.
-	 * Set the map state to FALSE to prevent a transition back to
+	 * Set the map state to 0 to prevent a transition back to
 	 * WithdrawnState in HandleUnmapNotify.  Map state gets set corrected
 	 * again in HandleMapNotify.
 	 */
@@ -2474,7 +2545,7 @@ FvwmWindow *AddWindow(
 void FetchWmProtocols(FvwmWindow *tmp)
 {
 	Atom *protocols = NULL, *ap;
-	long *l_protocols;
+	unsigned long *l_protocols;
 	int i, n;
 	Atom atype;
 	int aformat;
@@ -3065,6 +3136,11 @@ void destroy_window(FvwmWindow *fw)
 
 	free_window_names(fw, True, True);
 
+	if (fw->style_name)
+	{
+		free(fw->style_name);
+		fw->style_name = NULL;
+	}
 	if (fw->class.res_name && fw->class.res_name != NoResource)
 	{
 		XFree ((char *)fw->class.res_name);
