@@ -883,6 +883,7 @@ static void do_button_style(F_CMD_ARGS, Bool do_add)
 	int i;
 	int multi = 0;
 	int button = 0;
+	int do_return;
 	char *text = NULL;
 	char *prev = NULL;
 	char *parm = NULL;
@@ -910,6 +911,7 @@ static void do_button_style(F_CMD_ARGS, Bool do_add)
 
 	Scr.flags.do_need_window_update = 1;
 
+	do_return = 0;
 	if (!isdigit(*parm))
 	{
 		if (StrEquals(parm,"left"))
@@ -939,13 +941,12 @@ static void do_button_style(F_CMD_ARGS, Bool do_add)
 					"Bad button style (2) in line %s",
 					action);
 			}
-			return;
+			multi = 3;
+			do_return = 1;
 		}
 	}
-
 	/* mark button style and decor as changed */
 	decor->flags.has_changed = 1;
-
 	if (multi == 0)
 	{
 		/* a single button was specified */
@@ -963,7 +964,10 @@ static void do_button_style(F_CMD_ARGS, Bool do_add)
 			}
 		}
 	}
-
+	if (do_return == 1)
+	{
+		return;
+	}
 	for (prev = text; (parm = PeekToken(text, &text)); prev = text)
 	{
 		if (!do_add && strcmp(parm,"-") == 0)
@@ -1610,11 +1614,16 @@ Bool ReadDecorFace(char *s, DecorFace *df, int button, int verbose)
 
 			vc->num = num_coords;
 			vc->use_fgbg = 0;
-			vc->x = safemalloc(sizeof(char) * num_coords);
-			vc->y = safemalloc(sizeof(char) * num_coords);
-			vc->xoff = safemalloc(sizeof(char) * num_coords);
-			vc->yoff = safemalloc(sizeof(char) * num_coords);
-			vc->c = safemalloc(sizeof(char) * num_coords);
+			vc->x = (signed char*)safemalloc(sizeof(char) *
+							 num_coords);
+			vc->y = (signed char*)safemalloc(sizeof(char) *
+							 num_coords);
+			vc->xoff = (signed char*)safemalloc(sizeof(char) *
+							    num_coords);
+			vc->yoff = (signed char*)safemalloc(sizeof(char) *
+							    num_coords);
+			vc->c = (signed char*)safemalloc(sizeof(char) *
+							 num_coords);
 
 			/* get the points */
 			for (i = 0; i < vc->num; ++i)
@@ -2250,9 +2259,10 @@ void CMD_CursorMove(F_CMD_ARGS)
 	{
 		y = pan_y;
 	}
+	FWarpPointerUpdateEvpos(
+		exc->x.elast, dpy, None, Scr.Root, 0, 0, Scr.MyDisplayWidth,
+		Scr.MyDisplayHeight, x, y);
 
-	FWarpPointer(dpy, None, Scr.Root, 0, 0, Scr.MyDisplayWidth,
-		     Scr.MyDisplayHeight, x, y);
 	return;
 }
 
@@ -2553,14 +2563,29 @@ void CMD_Wait(F_CMD_ARGS)
 			}
 			else if (e.type == KeyPress)
 			{
-				/* TODO: should I be using <t> or <exc->w.fw>?
+				/* should I be using <t> or <exc->w.fw>?
+				 * DV: t
 				 */
-				int context = GetContext(&t, t, &e, &nonewin);
+				int context;
+				XClassHint *class;
+				char *name;
+
+				context = GetContext(&t, t, &e, &nonewin);
+				if (t != NULL)
+				{
+					class = &(t->class);
+					name = t->name.name;
+				}
+				else
+				{
+					class = NULL;
+					name = NULL;
+				}
 				escape = CheckBinding(
 					Scr.AllBindings, STROKE_ARG(0)
 					e.xkey.keycode, e.xkey.state,
 					GetUnusedModifiers(), context,
-					BIND_KEYPRESS, &t->class, t->name.name);
+					BIND_KEYPRESS, class, name);
 				if (escape != NULL)
 				{
 					if (!strcasecmp(escape,"escapefunc"))
@@ -2749,7 +2774,6 @@ void CMD_ModulePath(F_CMD_ARGS)
 	return;
 }
 
-
 void CMD_ModuleTimeout(F_CMD_ARGS)
 {
 	int timeout;
@@ -2847,34 +2871,6 @@ void CMD_AddTitleStyle(F_CMD_ARGS)
 	return;
 }
 
-void CMD_Colorset(F_CMD_ARGS)
-{
-	int n;
-	char *token;
-
-	if (GetIntegerArguments(action, &token, &n, 1) != 1)
-	{
-		return;
-	}
-	if (n < 0)
-	{
-		return;
-	}
-	if (token == NULL)
-	{
-		return;
-	}
-	parse_colorset(n, token);
-	update_fvwm_colorset(n);
-
-	return;
-}
-
-void CMD_CleanupColorsets(F_CMD_ARGS)
-{
-	cleanup_colorsets();
-}
-
 void CMD_PropertyChange(F_CMD_ARGS)
 {
 	char string[256] = "\0";
@@ -3633,6 +3629,22 @@ void CMD_BugOpts(F_CMD_ARGS)
 				break;
 			}
 		}
+		else if (StrEquals(opt, "DebugCRMotionMethod"))
+		{
+			switch (toggle)
+			{
+			case -1:
+				Scr.bo.do_debug_cr_motion_method ^= 1;
+				break;
+			case 0:
+			case 1:
+				Scr.bo.do_debug_cr_motion_method = toggle;
+				break;
+			default:
+				Scr.bo.do_debug_cr_motion_method = 0;
+				break;
+			}
+		}
 		else
 		{
 			fvwm_msg(ERR, "SetBugOptions",
@@ -4074,7 +4086,7 @@ void CMD_StrokeFunc(F_CMD_ARGS)
 	Bool feed_back = False;
 	int stroke_width = 1;
 	XEvent e;
-	XClassHint tmp, *pClass;
+	XClassHint *class;
 
 
 
@@ -4338,27 +4350,24 @@ void CMD_StrokeFunc(F_CMD_ARGS)
 		fvwm_msg(INFO, "StrokeFunc", "stroke sequence: %s (N%s)",
 			 sequence, num_seq);
 	}
-
 	if (abort)
 	{
 		return;
 	}
-
 	if (exc->w.fw == NULL)
 	{
-		tmp.res_class = tmp.res_name = name = "root";
-		pClass = &tmp;
+		class = NULL;
+		name = NULL;
 	}
 	else
 	{
-		pClass = &exc->w.fw->class;
+		class = &exc->w.fw->class;
 		name = exc->w.fw->name.name;
 	}
-
 	/* check for a binding */
 	stroke_action = CheckBinding(
 		Scr.AllBindings, sequence, 0, modifiers, GetUnusedModifiers(),
-		exc->w.wcontext, BIND_STROKE, pClass, name);
+		exc->w.wcontext, BIND_STROKE, class, name);
 
 	/* execute the action */
 	if (stroke_action != NULL)
@@ -4389,7 +4398,7 @@ void CMD_State(F_CMD_ARGS)
 	int n;
 	FvwmWindow * const fw = exc->w.fw;
 
-	n = GetIntegerArguments(action, &action, &state, 1);
+	n = GetIntegerArguments(action, &action, (int *)&state, 1);
 	if (n <= 0)
 	{
 		return;
