@@ -330,6 +330,7 @@ ewmh_atom *ewmh_GetEwmhAtomByAtom(Atom atom, ewmh_atom_list_name list_name)
 		}
 		i++;
 	}
+
 	return NULL;
 }
 
@@ -350,6 +351,8 @@ void ewmh_ChangeProperty(
 			dpy, w, a->atom, a->atom_type , format,
 			PropModeReplace, data, length);
 	}
+
+	return;
 }
 
 void ewmh_DeleteProperty(
@@ -361,6 +364,20 @@ void ewmh_DeleteProperty(
 	{
 		XDeleteProperty(dpy, w, a->atom);
 	}
+
+	return;
+}
+
+static int atom_size(int format)
+{
+	if (format == 32)
+	{
+		return sizeof(long);
+	}
+	else
+	{
+		return (format >> 3);
+	}
 }
 
 static
@@ -377,22 +394,41 @@ void *atom_get(Window win, Atom to_get, Atom type, unsigned int *size)
 	retval = NULL;
 	length = 0x7fffffff;
 	ok = XGetWindowProperty(
-		dpy, win, to_get, 0, length, False, type, &type_ret,
+		dpy, win, to_get, 0L, length, False, type, &type_ret,
 		&format_ret, &num_ret, &bytes_after, &retval);
 
 	if ((ok == Success) && (retval) && (num_ret > 0) && (format_ret > 0))
 	{
-		data = safemalloc(num_ret * (format_ret >> 3));
-		if (data)
+		int asize;
+
+		asize = atom_size(format_ret);
+		data = safemalloc(num_ret * asize);
+		if (format_ret == 32 && asize * 8 != format_ret)
 		{
-			memcpy(data, retval, num_ret * (format_ret >> 3));
+			int i;
+
+			for (i = 0; i < num_ret; i++)
+			{
+				((CARD32 *)data)[i] = ((long *)retval)[i];
+			}
+		}
+		else
+		{
+			if (data)
+			{
+				memcpy(data, retval, num_ret * asize);
+			}
 		}
 		XFree(retval);
 		*size = num_ret * (format_ret >> 3);
+
 		return data;
 	}
 	if (retval)
+	{
 		XFree(retval);
+	}
+
 	return NULL;
 }
 
@@ -418,21 +454,22 @@ static
 int check_desk(void)
 {
 	int d = -1;
-	FvwmWindow *t;
+	FvwmWindow *fw;
 
-	for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
+	for (fw = Scr.FvwmRoot.next; fw != NULL; fw = fw->next)
 	{
-		if (!IS_STICKY_ACROSS_DESKS(t))
+		if (!IS_STICKY_ACROSS_DESKS(fw))
 		{
-			d = max(d,t->Desk);
+			d = max(d, fw->Desk);
 		}
 	}
+
 	return d;
 }
 
 void EWMH_SetCurrentDesktop(void)
 {
-	CARD32 val;
+	long val;
 
 	val = Scr.CurrentDesk;
 
@@ -451,11 +488,13 @@ void EWMH_SetCurrentDesktop(void)
 	ewmh_ChangeProperty(Scr.Root,"_NET_CURRENT_DESKTOP",
 			    EWMH_ATOM_LIST_CLIENT_ROOT,
 			    (unsigned char *)&val, 1);
+
+	return;
 }
 
 void EWMH_SetNumberOfDesktops(void)
 {
-	CARD32 val;
+	long val;
 
 	if (ewmhc.CurrentNumberOfDesktops < ewmhc.NumberOfDesktops)
 	{
@@ -480,16 +519,18 @@ void EWMH_SetNumberOfDesktops(void)
 		ewmhc.CurrentNumberOfDesktops = Scr.CurrentDesk + 1;
 	}
 
-	val = (CARD32)ewmhc.CurrentNumberOfDesktops;
+	val = (long)ewmhc.CurrentNumberOfDesktops;
 	ewmh_ChangeProperty(Scr.Root, "_NET_NUMBER_OF_DESKTOPS",
 			    EWMH_ATOM_LIST_CLIENT_ROOT,
 			    (unsigned char *)&val, 1);
 	ewmh_SetWorkArea();
+
+	return;
 }
 
 void EWMH_SetDesktopViewPort(void)
 {
-	CARD32 val[256][2]; /* no more than 256 desktops */
+	long val[256][2]; /* no more than 256 desktops */
 	int i = 0;
 
 	while(i < ewmhc.NumberOfDesktops && i < 256)
@@ -501,17 +542,21 @@ void EWMH_SetDesktopViewPort(void)
 	ewmh_ChangeProperty(
 		Scr.Root, "_NET_DESKTOP_VIEWPORT", EWMH_ATOM_LIST_CLIENT_ROOT,
 		(unsigned char *)&val, i*2);
+
+	return;
 }
 
 void EWMH_SetDesktopGeometry(void)
 {
-	CARD32 val[2];
+	long val[2];
 
 	val[0] = Scr.VxMax + Scr.MyDisplayWidth;
 	val[1] = Scr.VyMax + Scr.MyDisplayHeight;
 	ewmh_ChangeProperty(
 		Scr.Root,"_NET_DESKTOP_GEOMETRY", EWMH_ATOM_LIST_CLIENT_ROOT,
 		(unsigned char *)&val, 2);
+
+	return;
 }
 
 /*
@@ -522,15 +567,17 @@ void EWMH_SetActiveWindow(Window w)
 	ewmh_ChangeProperty(
 		Scr.Root, "_NET_ACTIVE_WINDOW", EWMH_ATOM_LIST_CLIENT_WIN,
 		(unsigned char *)&w, 1);
+
+	return;
 }
 
-void EWMH_SetWMDesktop(FvwmWindow *fwin)
+void EWMH_SetWMDesktop(FvwmWindow *fw)
 {
-	CARD32 desk = fwin->Desk;
+	long desk = fw->Desk;
 
-	if (IS_STICKY_ACROSS_DESKS(fwin))
+	if (IS_STICKY_ACROSS_DESKS(fw))
 	{
-		desk = 0xFFFFFFFF;
+		desk = (unsigned long)-1;
 	}
 	else if (desk >= ewmhc.CurrentNumberOfDesktops)
 	{
@@ -538,15 +585,17 @@ void EWMH_SetWMDesktop(FvwmWindow *fwin)
 		EWMH_SetNumberOfDesktops();
 	}
 	ewmh_ChangeProperty(
-		FW_W(fwin), "_NET_WM_DESKTOP", EWMH_ATOM_LIST_CLIENT_WIN,
+		FW_W(fw), "_NET_WM_DESKTOP", EWMH_ATOM_LIST_CLIENT_WIN,
 		(unsigned char *)&desk, 1);
+
+	return;
 }
 
 /*
  *  fvwm must maintain the _NET_WM_STATE
  */
 
-void EWMH_SetWMState(FvwmWindow *fwin, Bool do_restore)
+void EWMH_SetWMState(FvwmWindow *fw, Bool do_restore)
 {
 	Atom wm_state[EWMH_NUMBER_OF_STATE];
 	int i = 0;
@@ -554,7 +603,7 @@ void EWMH_SetWMState(FvwmWindow *fwin, Bool do_restore)
 
 	while(list->name != NULL)
 	{
-		if (list->action(fwin, NULL, NULL, do_restore))
+		if (list->action(fw, NULL, NULL, do_restore))
 		{
 			wm_state[i++] = list->atom;
 		}
@@ -564,15 +613,17 @@ void EWMH_SetWMState(FvwmWindow *fwin, Bool do_restore)
 	if (i > 0)
 	{
 		ewmh_ChangeProperty(
-			FW_W(fwin), "_NET_WM_STATE", EWMH_ATOM_LIST_CLIENT_WIN,
+			FW_W(fw), "_NET_WM_STATE", EWMH_ATOM_LIST_CLIENT_WIN,
 			(unsigned char *)wm_state, i);
 	}
 	else
 	{
 		ewmh_DeleteProperty(
-			FW_W(fwin), "_NET_WM_STATE",
+			FW_W(fw), "_NET_WM_STATE",
 			EWMH_ATOM_LIST_CLIENT_WIN);
 	}
+
+	return;
 }
 
 /*
@@ -594,10 +645,11 @@ void add_kst_item(Window w)
 		prev = &(t->next);
 		t = t->next;
 	}
-
 	*prev = (KstItem *)safemalloc(sizeof(KstItem));
 	(*prev)->w = w;
 	(*prev)->next = NULL;
+
+	return;
 }
 
 static
@@ -621,6 +673,8 @@ void delete_kst_item(Window w)
 		*prev = t->next;
 	}
 	free(t);
+
+	return;
 }
 
 static
@@ -665,16 +719,18 @@ void set_kde_sys_tray(void)
 	{
 		free(wins);
 	}
+
+	return;
 }
 
-void ewmh_AddToKdeSysTray(FvwmWindow *fwin)
+void ewmh_AddToKdeSysTray(FvwmWindow *fw)
 {
 	unsigned int size = 0;
 	Atom *val;
 	KstItem *t;
 
 	val = ewmh_AtomGetByName(
-		FW_W(fwin), "_KDE_NET_WM_SYSTEM_TRAY_WINDOW_FOR",
+		FW_W(fw), "_KDE_NET_WM_SYSTEM_TRAY_WINDOW_FOR",
 		EWMH_ATOM_LIST_FIXED_PROPERTY, &size);
 
 	if (val == NULL)
@@ -684,7 +740,7 @@ void ewmh_AddToKdeSysTray(FvwmWindow *fwin)
 	free(val);
 
 	t = ewmh_KstWinList;
-	while(t != NULL && t->w != FW_W(fwin))
+	while(t != NULL && t->w != FW_W(fw))
 		t = t->next;
 
 	if (t != NULL)
@@ -692,8 +748,10 @@ void ewmh_AddToKdeSysTray(FvwmWindow *fwin)
 		return; /* already in the list */
 	}
 
-	add_kst_item(FW_W(fwin));
+	add_kst_item(FW_W(fw));
 	set_kde_sys_tray();
+
+	return;
 }
 
 #if 0
@@ -710,6 +768,8 @@ void ewmh_FreeKdeSysTray(void)
 		t = ewmh_KstWinList;
 	}
 	set_kde_sys_tray();
+
+	return;
 }
 #endif
 
@@ -719,8 +779,9 @@ int EWMH_IsKdeSysTrayWindow(Window w)
 
 	t = ewmh_KstWinList;
 	while(t != NULL && t->w != w)
+	{
 		t = t->next;
-
+	}
 	if (t == NULL)
 	{
 		return 0;
@@ -728,6 +789,7 @@ int EWMH_IsKdeSysTrayWindow(Window w)
 #ifdef DEBUG_KST
 	fprintf(stderr,"IsKdeSysTrayWindow: 0x%lx\n", w);
 #endif
+
 	return 1;
 }
 
@@ -775,6 +837,8 @@ void EWMH_ManageKdeSysTray(Window w, int type)
 #endif
 		break;
 	}
+
+	return;
 }
 
 /**** Client lists ****/
@@ -782,20 +846,20 @@ void EWMH_ManageKdeSysTray(Window w, int type)
 void EWMH_SetClientList()
 {
 	Window *wl = NULL;
-	FvwmWindow *t;
+	FvwmWindow *fw;
 	int nbr = 0;
 	int i = 0;
 
-	for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
+	for (fw = Scr.FvwmRoot.next; fw != NULL; fw = fw->next)
 	{
 		nbr++;
 	}
 	if (nbr != 0)
 	{
-		wl = (Window *)safemalloc(sizeof (Window) * nbr);
-		for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
+		wl = (Window *)safemalloc(sizeof(Window) * nbr);
+		for (fw = Scr.FvwmRoot.next; fw != NULL; fw = fw->next)
 		{
-			wl[i++] = FW_W(t);
+			wl[i++] = FW_W(fw);
 		}
 	}
 	ewmh_ChangeProperty(
@@ -805,28 +869,32 @@ void EWMH_SetClientList()
 	{
 		free (wl);
 	}
+
+	return;
 }
 
 void EWMH_SetClientListStacking()
 {
 	Window *wl = NULL;
-	FvwmWindow *t;
+	FvwmWindow *fw;
 	int nbr = 0;
 	int i = 0;
 
-	for (t = Scr.FvwmRoot.stack_next; t != &Scr.FvwmRoot;
-	     t = t->stack_next)
+	for (
+		fw = Scr.FvwmRoot.stack_next; fw != &Scr.FvwmRoot;
+		fw = fw->stack_next)
 	{
 		nbr++;
 	}
 	i = nbr-1;
 	if (nbr != 0)
 	{
-		wl = (Window *)safemalloc(sizeof (Window) * nbr);
-		for (t = Scr.FvwmRoot.stack_next; t != &Scr.FvwmRoot;
-		     t = t->stack_next)
+		wl = (Window *)safemalloc(sizeof(Window) * nbr);
+		for (
+			fw = Scr.FvwmRoot.stack_next; fw != &Scr.FvwmRoot;
+			fw = fw->stack_next)
 		{
-			wl[i--] = FW_W(t);
+			wl[i--] = FW_W(fw);
 		}
 	}
 	ewmh_ChangeProperty(
@@ -836,6 +904,8 @@ void EWMH_SetClientListStacking()
 	{
 		free (wl);
 	}
+
+	return;
 }
 
 /**** Working Area stuff ****/
@@ -843,7 +913,7 @@ void EWMH_SetClientListStacking()
 
 void ewmh_SetWorkArea(void)
 {
-	CARD32 val[256][4]; /* no more than 256 desktops */
+	long val[256][4]; /* no more than 256 desktops */
 	int i = 0;
 
 	while(i < ewmhc.NumberOfDesktops && i < 256)
@@ -857,6 +927,8 @@ void ewmh_SetWorkArea(void)
 	ewmh_ChangeProperty(
 		Scr.Root, "_NET_WORKAREA", EWMH_ATOM_LIST_FVWM_ROOT,
 		(unsigned char *)&val, i*4);
+
+	return;
 }
 
 void ewmh_ComputeAndSetWorkArea(void)
@@ -866,19 +938,20 @@ void ewmh_ComputeAndSetWorkArea(void)
 	int top = ewmhc.BaseStrut.top;
 	int bottom = ewmhc.BaseStrut.bottom;
 	int x,y,width,height;
-	FvwmWindow *t;
+	FvwmWindow *fw;
 
-	for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
+	for (fw = Scr.FvwmRoot.next; fw != NULL; fw = fw->next)
 	{
-		if (DO_EWMH_IGNORE_STRUT_HINTS(t) ||
-		    !IS_STICKY_ACROSS_PAGES(t))
+		if (
+			DO_EWMH_IGNORE_STRUT_HINTS(fw) ||
+			!IS_STICKY_ACROSS_PAGES(fw))
 		{
 			continue;
 		}
-		left = max(left, t->strut.left);
-		right = max(right, t->strut.right);
-		top = max(top, t->strut.top);
-		bottom = max(bottom, t->strut.bottom);
+		left = max(left, fw->strut.left);
+		right = max(right, fw->strut.right);
+		top = max(top, fw->strut.top);
+		bottom = max(bottom, fw->strut.bottom);
 	}
 
 	x = left;
@@ -886,10 +959,11 @@ void ewmh_ComputeAndSetWorkArea(void)
 	width = Scr.MyDisplayWidth - (left + right);
 	height = Scr.MyDisplayHeight - (top + bottom);
 
-	if (Scr.Desktops->ewmh_working_area.x != x ||
-	    Scr.Desktops->ewmh_working_area.y != y ||
-	    Scr.Desktops->ewmh_working_area.width != width ||
-	    Scr.Desktops->ewmh_working_area.height != height)
+	if (
+		Scr.Desktops->ewmh_working_area.x != x ||
+		Scr.Desktops->ewmh_working_area.y != y ||
+		Scr.Desktops->ewmh_working_area.width != width ||
+		Scr.Desktops->ewmh_working_area.height != height)
 	{
 		Scr.Desktops->ewmh_working_area.x = x;
 		Scr.Desktops->ewmh_working_area.y = y;
@@ -898,6 +972,7 @@ void ewmh_ComputeAndSetWorkArea(void)
 		ewmh_SetWorkArea();
 	}
 
+	return;
 }
 
 void ewmh_HandleDynamicWorkArea(void)
@@ -907,19 +982,20 @@ void ewmh_HandleDynamicWorkArea(void)
 	int dyn_top = ewmhc.BaseStrut.top;
 	int dyn_bottom = ewmhc.BaseStrut.bottom;
 	int x,y,width,height;
-	FvwmWindow *t;
+	FvwmWindow *fw;
 
-	for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
+	for (fw = Scr.FvwmRoot.next; fw != NULL; fw = fw->next)
 	{
-		if (DO_EWMH_IGNORE_STRUT_HINTS(t) ||
-		    !IS_STICKY_ACROSS_PAGES(t))
+		if (
+			DO_EWMH_IGNORE_STRUT_HINTS(fw) ||
+			!IS_STICKY_ACROSS_PAGES(fw))
 		{
 			continue;
 		}
-		dyn_left = max(dyn_left, t->dyn_strut.left);
-		dyn_right = max(dyn_right, t->dyn_strut.right);
-		dyn_top = max(dyn_top, t->dyn_strut.top);
-		dyn_bottom = max(dyn_bottom, t->dyn_strut.bottom);
+		dyn_left = max(dyn_left, fw->dyn_strut.left);
+		dyn_right = max(dyn_right, fw->dyn_strut.right);
+		dyn_top = max(dyn_top, fw->dyn_strut.top);
+		dyn_bottom = max(dyn_bottom, fw->dyn_strut.bottom);
 	}
 
 	x = dyn_left;
@@ -927,10 +1003,11 @@ void ewmh_HandleDynamicWorkArea(void)
 	width = Scr.MyDisplayWidth - (dyn_left + dyn_right);
 	height = Scr.MyDisplayHeight - (dyn_top + dyn_bottom);
 
-	if (Scr.Desktops->ewmh_dyn_working_area.x != x ||
-	    Scr.Desktops->ewmh_dyn_working_area.y != y ||
-	    Scr.Desktops->ewmh_dyn_working_area.width != width ||
-	    Scr.Desktops->ewmh_dyn_working_area.height != height)
+	if (
+		Scr.Desktops->ewmh_dyn_working_area.x != x ||
+		Scr.Desktops->ewmh_dyn_working_area.y != y ||
+		Scr.Desktops->ewmh_dyn_working_area.width != width ||
+		Scr.Desktops->ewmh_dyn_working_area.height != height)
 	{
 		Scr.Desktops->ewmh_dyn_working_area.x = x;
 		Scr.Desktops->ewmh_dyn_working_area.y = y;
@@ -938,22 +1015,27 @@ void ewmh_HandleDynamicWorkArea(void)
 		Scr.Desktops->ewmh_dyn_working_area.height = height;
 		/* here we may update the maximized window ...etc */
 	}
+
+	return;
 }
 
 void EWMH_UpdateWorkArea(void)
 {
 	ewmh_ComputeAndSetWorkArea();
 	ewmh_HandleDynamicWorkArea();
+
+	return;
 }
 
 void EWMH_GetWorkAreaIntersection(
-	FvwmWindow *fwin, int *x, int *y, int *w, int *h, int type)
+	FvwmWindow *fw, int *x, int *y, unsigned int *w, unsigned int *h,
+	int type)
 {
 	int nx,ny,nw,nh;
 	int area_x = Scr.Desktops->ewmh_working_area.x;
 	int area_y = Scr.Desktops->ewmh_working_area.y;
-	int area_w = Scr.Desktops->ewmh_working_area.width;
-	int area_h = Scr.Desktops->ewmh_working_area.height;
+	unsigned int area_w = Scr.Desktops->ewmh_working_area.width;
+	unsigned int area_h = Scr.Desktops->ewmh_working_area.height;
 	Bool is_dynamic = False;
 
 	switch(type)
@@ -983,6 +1065,8 @@ void EWMH_GetWorkAreaIntersection(
 	*y = ny;
 	*w = nw;
 	*h = nh;
+
+	return;
 }
 
 static
@@ -1007,6 +1091,7 @@ float get_intersection(
 		ret = 100 * max(ret / ((x22 - x21) * (y22 - y21)),
 				ret / ((x12 - x11) * (y12 - y11)));
 	}
+
 	return ret;
 }
 
@@ -1055,8 +1140,9 @@ float EWMH_GetBaseStrutIntersection(
 	int x11, int y11, int x12, int y12, Bool use_percent)
 {
 	return ewmh_GetStrutIntersection(
-		x11, y11, x12, y12, ewmhc.BaseStrut.left, ewmhc.BaseStrut.right,
-		ewmhc.BaseStrut.top, ewmhc.BaseStrut.bottom, use_percent);
+		x11, y11, x12, y12, ewmhc.BaseStrut.left,
+		ewmhc.BaseStrut.right, ewmhc.BaseStrut.top,
+		ewmhc.BaseStrut.bottom, use_percent);
 }
 
 float EWMH_GetStrutIntersection(
@@ -1072,6 +1158,7 @@ float EWMH_GetStrutIntersection(
 	bottom = Scr.MyDisplayHeight -
 		(Scr.Desktops->ewmh_working_area.y
 		 + Scr.Desktops->ewmh_working_area.height);
+
 	return ewmh_GetStrutIntersection(
 		x11, y11, x12, y12, left, right, top, bottom, use_percent);
 }
@@ -1079,17 +1166,17 @@ float EWMH_GetStrutIntersection(
 /*
  *  fvwm_win
  */
-void EWMH_SetFrameStrut(FvwmWindow *fwin)
+void EWMH_SetFrameStrut(FvwmWindow *fw)
 {
-	CARD32 val[4];
+	long val[4];
 	size_borders b;
 
-	if (EWMH_IsKdeSysTrayWindow(FW_W(fwin)))
+	if (EWMH_IsKdeSysTrayWindow(FW_W(fw)))
 	{
 		/* Fixed position of tray window in kicker */
 		return;
 	}
-	get_window_borders(fwin, &b);
+	get_window_borders(fw, &b);
 
 	/* left */
 	val[0] = b.top_left.width;
@@ -1101,11 +1188,13 @@ void EWMH_SetFrameStrut(FvwmWindow *fwin)
 	val[3] = b.bottom_right.height;
 
 	ewmh_ChangeProperty(
-		FW_W(fwin), "_KDE_NET_WM_FRAME_STRUT", EWMH_ATOM_LIST_FVWM_WIN,
+		FW_W(fw), "_KDE_NET_WM_FRAME_STRUT", EWMH_ATOM_LIST_FVWM_WIN,
 		(unsigned char *)&val, 4);
 	ewmh_ChangeProperty(
-		FW_W(fwin), "_NET_FRAME_EXTENTS", EWMH_ATOM_LIST_FVWM_WIN,
+		FW_W(fw), "_NET_FRAME_EXTENTS", EWMH_ATOM_LIST_FVWM_WIN,
 		(unsigned char *)&val, 4);
+
+	return;
 }
 
 /*
@@ -1118,41 +1207,42 @@ Bool ewmh_AllowsYes(EWMH_CMD_ARGS)
 
 Bool ewmh_AllowsClose(EWMH_CMD_ARGS)
 {
-	return is_function_allowed(F_CLOSE, NULL, fwin, True, False);
+	return is_function_allowed(F_CLOSE, NULL, fw, True, False);
 }
 
 Bool ewmh_AllowsFullScreen(EWMH_CMD_ARGS)
 {
-	if (!is_function_allowed(F_MAXIMIZE, NULL, fwin, True, False) ||
-	    !is_function_allowed(F_MOVE, NULL, fwin, True, False) ||
-	    !is_function_allowed(F_RESIZE, NULL, fwin, True, True))
+	if (!is_function_allowed(F_MAXIMIZE, NULL, fw, True, False) ||
+	    !is_function_allowed(F_MOVE, NULL, fw, True, False) ||
+	    !is_function_allowed(F_RESIZE, NULL, fw, True, True))
 	{
 		return False;
 	}
+
 	return True;
 }
 
 Bool ewmh_AllowsMinimize(EWMH_CMD_ARGS)
 {
-	return is_function_allowed(F_ICONIFY, NULL, fwin, True, False);
+	return is_function_allowed(F_ICONIFY, NULL, fw, True, False);
 }
 
 Bool ewmh_AllowsMaximize(EWMH_CMD_ARGS)
 {
-	return is_function_allowed(F_MAXIMIZE, NULL, fwin, True, False);
+	return is_function_allowed(F_MAXIMIZE, NULL, fw, True, False);
 }
 
 Bool ewmh_AllowsMove(EWMH_CMD_ARGS)
 {
-	return is_function_allowed(F_MOVE, NULL, fwin, True, False);
+	return is_function_allowed(F_MOVE, NULL, fw, True, False);
 }
 
 Bool ewmh_AllowsResize(EWMH_CMD_ARGS)
 {
-	return is_function_allowed(F_RESIZE, NULL, fwin, True, False);
+	return is_function_allowed(F_RESIZE, NULL, fw, True, False);
 }
 
-void EWMH_SetAllowedActions(FvwmWindow *fwin)
+void EWMH_SetAllowedActions(FvwmWindow *fw)
 {
 	Atom wm_actions[EWMH_NUMBER_OF_ALLOWED_ACTIONS];
 	int i = 0;
@@ -1160,7 +1250,7 @@ void EWMH_SetAllowedActions(FvwmWindow *fwin)
 
 	while(list->name != NULL)
 	{
-		if (list->action(fwin, NULL, NULL, 0))
+		if (list->action(fw, NULL, NULL, 0))
 			wm_actions[i++] = list->atom;
 		list++;
 	}
@@ -1168,16 +1258,18 @@ void EWMH_SetAllowedActions(FvwmWindow *fwin)
 	if (i > 0)
 	{
 		ewmh_ChangeProperty(
-			FW_W(fwin), "_NET_WM_ALLOWED_ACTIONS",
+			FW_W(fw), "_NET_WM_ALLOWED_ACTIONS",
 			EWMH_ATOM_LIST_FVWM_WIN, (unsigned char *)wm_actions,
 			i);
 	}
 	else
 	{
 		ewmh_DeleteProperty(
-			FW_W(fwin), "_NET_WM_ALLOWED_ACTIONS",
+			FW_W(fw), "_NET_WM_ALLOWED_ACTIONS",
 			EWMH_ATOM_LIST_FVWM_WIN);
 	}
+
+	return;
 }
 
 /*
@@ -1186,7 +1278,7 @@ void EWMH_SetAllowedActions(FvwmWindow *fwin)
 
 int ewmh_HandleDesktop(EWMH_CMD_ARGS)
 {
-	if (Scr.EwmhDesktop != NULL && FW_W(Scr.EwmhDesktop) != FW_W(fwin))
+	if (Scr.EwmhDesktop != NULL && FW_W(Scr.EwmhDesktop) != FW_W(fw))
 	{
 		fvwm_msg(
 			WARN,"ewmh_HandleDesktop",
@@ -1195,8 +1287,8 @@ int ewmh_HandleDesktop(EWMH_CMD_ARGS)
 		/* what to do ? */
 	}
 
-	fwin->ewmh_window_type = EWMH_WINDOW_TYPE_DESKTOP_ID;
-	Scr.EwmhDesktop = fwin;
+	fw->ewmh_window_type = EWMH_WINDOW_TYPE_DESKTOP_ID;
+	Scr.EwmhDesktop = fw;
 
 	SSET_LAYER(*style, 0);
 	style->flags.use_layer = 1;
@@ -1255,10 +1347,10 @@ int ewmh_HandleDesktop(EWMH_CMD_ARGS)
 	FPS_GRAB_FOCUS(S_FOCUS_POLICY(SCC(*style)), 1);
 
 	/* ClickToFocusPassesClick */
-	FPS_PASS_FOCUS_CLICK(S_FOCUS_POLICY(SCF(*style)), 0);
+	FPS_PASS_FOCUS_CLICK(S_FOCUS_POLICY(SCF(*style)), 1);
 	FPS_PASS_FOCUS_CLICK(S_FOCUS_POLICY(SCM(*style)), 1);
 	FPS_PASS_FOCUS_CLICK(S_FOCUS_POLICY(SCC(*style)), 1);
-	FPS_PASS_RAISE_CLICK(S_FOCUS_POLICY(SCF(*style)), 0);
+	FPS_PASS_RAISE_CLICK(S_FOCUS_POLICY(SCF(*style)), 1);
 	FPS_PASS_RAISE_CLICK(S_FOCUS_POLICY(SCM(*style)), 1);
 	FPS_PASS_RAISE_CLICK(S_FOCUS_POLICY(SCC(*style)), 1);
 
@@ -1281,13 +1373,14 @@ int ewmh_HandleDesktop(EWMH_CMD_ARGS)
 
 int ewmh_HandleDialog(EWMH_CMD_ARGS)
 {
-	fwin->ewmh_window_type = EWMH_WINDOW_TYPE_DIALOG_ID;
+	fw->ewmh_window_type = EWMH_WINDOW_TYPE_DIALOG_ID;
+
 	return 0;
 }
 
 int ewmh_HandleDock(EWMH_CMD_ARGS)
 {
-	fwin->ewmh_window_type = EWMH_WINDOW_TYPE_DOCK_ID;
+	fw->ewmh_window_type = EWMH_WINDOW_TYPE_DOCK_ID;
 
 	S_SET_IS_STICKY_ACROSS_PAGES(SCF(*style), 1);
 	S_SET_IS_STICKY_ACROSS_PAGES(SCM(*style), 1);
@@ -1312,9 +1405,9 @@ int ewmh_HandleDock(EWMH_CMD_ARGS)
 	S_SET_IS_UNMAXIMIZABLE(SCM(*style), 1);
 	S_SET_IS_UNMAXIMIZABLE(SCC(*style), 1);
 
-	if (fwin->ewmh_hint_layer == -1)
+	if (fw->ewmh_hint_layer == -1)
 	{
-		fwin->ewmh_hint_layer = Scr.TopLayer;
+		fw->ewmh_hint_layer = Scr.TopLayer;
 		if (DO_EWMH_USE_STACKING_HINTS(style))
 		{
 			SSET_LAYER(*style, Scr.TopLayer);
@@ -1337,7 +1430,7 @@ int ewmh_HandleDock(EWMH_CMD_ARGS)
 
 int ewmh_HandleMenu(EWMH_CMD_ARGS)
 {
-	fwin->ewmh_window_type = EWMH_WINDOW_TYPE_MENU_ID;
+	fw->ewmh_window_type = EWMH_WINDOW_TYPE_MENU_ID;
 
 	/* tear off menu */
 
@@ -1379,13 +1472,14 @@ int ewmh_HandleMenu(EWMH_CMD_ARGS)
 
 int ewmh_HandleNormal(EWMH_CMD_ARGS)
 {
-	fwin->ewmh_window_type = EWMH_WINDOW_TYPE_NORMAL_ID;
+	fw->ewmh_window_type = EWMH_WINDOW_TYPE_NORMAL_ID;
+
 	return 0;
 }
 
 int ewmh_HandleToolBar(EWMH_CMD_ARGS)
 {
-	fwin->ewmh_window_type = EWMH_WINDOW_TYPE_TOOLBAR_ID;
+	fw->ewmh_window_type = EWMH_WINDOW_TYPE_TOOLBAR_ID;
 
 	/* this ok for KDE 2 (and 3??) but I do not think that a toolbar
 	   should be sticky */
@@ -1409,7 +1503,7 @@ int ewmh_HandleToolBar(EWMH_CMD_ARGS)
 	return 1;
 }
 
-void ewmh_HandleWindowType(FvwmWindow *fwin, window_style *style)
+void ewmh_HandleWindowType(FvwmWindow *fw, window_style *style)
 {
 	Atom *val;
 	ewmh_atom *list = ewmh_atom_window_type;
@@ -1417,16 +1511,18 @@ void ewmh_HandleWindowType(FvwmWindow *fwin, window_style *style)
 	int i = 0;
 	Bool found = False;
 
-	fwin->ewmh_window_type = 0;
+	fw->ewmh_window_type = 0;
+	if (DO_EWMH_IGNORE_WINDOW_TYPE(style))
+	{
+		return;
+	}
 	val = ewmh_AtomGetByName(
-		FW_W(fwin), "_NET_WM_WINDOW_TYPE",
+		FW_W(fw), "_NET_WM_WINDOW_TYPE",
 		EWMH_ATOM_LIST_FIXED_PROPERTY, &size);
-
 	if (val == NULL)
 	{
 		return;
 	}
-
 	/* we support only one window type: the first that we support */
 	while(i < size && !found)
 	{
@@ -1435,7 +1531,7 @@ void ewmh_HandleWindowType(FvwmWindow *fwin, window_style *style)
 		{
 			if (list->atom == val[i])
 			{
-				list->action(fwin, NULL, style, 0);
+				list->action(fw, NULL, style, 0);
 				found = True;
 			}
 			list++;
@@ -1443,24 +1539,27 @@ void ewmh_HandleWindowType(FvwmWindow *fwin, window_style *style)
 		i++;
 	}
 	free(val);
+
+	return;
 }
 
 /*
  * a workaround for ksmserver exit windows
  */
 static
-int ksmserver_workarround(FvwmWindow *fwin)
+int ksmserver_workarround(FvwmWindow *fw)
 {
 
-	if (fwin->name.name != NULL && fwin->class.res_name != NULL &&
-	    fwin->icon_name.name != NULL && fwin->class.res_class != NULL &&
-	    strcmp(fwin->name.name, "ksmserver") == 0 &&
-	    strcmp(fwin->class.res_class, "ksmserver") == 0 &&
-	    strcmp(fwin->icon_name.name, "ksmserver") == 0 &&
-	    strcmp(fwin->class.res_name, "unnamed") == 0)
+	if (fw->name.name != NULL && fw->class.res_name != NULL &&
+	    fw->icon_name.name != NULL && fw->class.res_class != NULL &&
+	    strcmp(fw->name.name, "ksmserver") == 0 &&
+	    strcmp(fw->class.res_class, "ksmserver") == 0 &&
+	    strcmp(fw->icon_name.name, "ksmserver") == 0 &&
+	    strcmp(fw->class.res_name, "unnamed") == 0)
 	{
 		int layer = 0;
-		if (IS_TRANSIENT(fwin))
+
+		if (IS_TRANSIENT(fw))
 		{
 			layer = Scr.TopLayer + 2;
 		}
@@ -1468,10 +1567,11 @@ int ksmserver_workarround(FvwmWindow *fwin)
 		{
 			layer = Scr.TopLayer + 1;
 		}
+		new_layer(fw, layer);
 
-		new_layer(fwin, layer);
 		return 1;
 	}
+
 	return 0;
 }
 
@@ -1479,20 +1579,22 @@ int ksmserver_workarround(FvwmWindow *fwin)
  * Window Initialisation / Destroy
  */
 
-void EWMH_GetStyle(FvwmWindow *fwin, window_style *style)
+void EWMH_GetStyle(FvwmWindow *fw, window_style *style)
 {
 	if (style->change_mask.use_layer)
 	{
-		fwin->ewmh_normal_layer = SGET_LAYER(*style);
+		fw->ewmh_normal_layer = SGET_LAYER(*style);
 	}
-	else if (fwin->ewmh_normal_layer == 0)
+	else if (fw->ewmh_normal_layer == 0)
 	{
-		fwin->ewmh_normal_layer = Scr.DefaultLayer;
+		fw->ewmh_normal_layer = Scr.DefaultLayer;
 	}
-	ewmh_WMState(fwin, NULL, style, 0);
-	ewmh_WMDesktop(fwin, NULL, style, 0);
+	ewmh_WMState(fw, NULL, style, 0);
+	ewmh_WMDesktop(fw, NULL, style, 0);
 	/* the window type override the state hint */
-	ewmh_HandleWindowType(fwin, style);
+	ewmh_HandleWindowType(fw, style);
+
+	return;
 }
 
 static void ewmh_check_wm_pid(FvwmWindow *fw)
@@ -1513,65 +1615,75 @@ static void ewmh_check_wm_pid(FvwmWindow *fw)
 			SET_CR_MOTION_METHOD_DETECTED(fw, 1);
 		}
 	}
+
+	return;
 }
 
 /* see also EWMH_WMName and EWMH_WMIconName in add_window */
-void EWMH_WindowInit(FvwmWindow *fwin)
+void EWMH_WindowInit(FvwmWindow *fw)
 {
-	/*EWMH_DLOG("Init window 0x%lx",FW_W(fwin));*/
-	EWMH_SetWMState(fwin, False);
-	EWMH_SetWMDesktop(fwin);
-	EWMH_SetAllowedActions(fwin);
-	ewmh_WMStrut(fwin, NULL, NULL, 0);
-	ewmh_WMIconGeometry(fwin, NULL, NULL, 0);
-	ewmh_AddToKdeSysTray(fwin);
-	EWMH_SetFrameStrut(fwin);
-	if (IS_EWMH_DESKTOP(FW_W(fwin)))
+	/*EWMH_DLOG("Init window 0x%lx",FW_W(fw));*/
+	EWMH_SetWMState(fw, False);
+	EWMH_SetWMDesktop(fw);
+	EWMH_SetAllowedActions(fw);
+	ewmh_WMStrut(fw, NULL, NULL, 0);
+	ewmh_WMIconGeometry(fw, NULL, NULL, 0);
+	ewmh_AddToKdeSysTray(fw);
+	EWMH_SetFrameStrut(fw);
+	if (IS_EWMH_DESKTOP(FW_W(fw)))
 	{
 		return;
 	}
-	if (ksmserver_workarround(fwin))
+	if (ksmserver_workarround(fw))
 	{
 		return;
 	}
-	ewmh_WMIcon(fwin, NULL, NULL, 0);
-	ewmh_check_wm_pid(fwin);
-	/*EWMH_DLOG("window 0x%lx initialised",FW_W(fwin));*/
+	ewmh_WMIcon(fw, NULL, NULL, 0);
+	ewmh_check_wm_pid(fw);
+	/*EWMH_DLOG("window 0x%lx initialised",FW_W(fw));*/
+
+	return;
 }
 
 /* unmap or reparent: restore state */
-void EWMH_RestoreInitialStates(FvwmWindow *fwin, int event_type)
+void EWMH_RestoreInitialStates(FvwmWindow *fw, int event_type)
 {
-	EWMH_SetWMState(fwin, True);
-	if (HAS_EWMH_INIT_WM_DESKTOP(fwin) == EWMH_STATE_HAS_HINT)
+	EWMH_SetWMState(fw, True);
+	if (HAS_EWMH_INIT_WM_DESKTOP(fw) == EWMH_STATE_HAS_HINT)
 	{
 		ewmh_ChangeProperty(
-			FW_W(fwin), "_NET_WM_DESKTOP",
+			FW_W(fw), "_NET_WM_DESKTOP",
 			EWMH_ATOM_LIST_CLIENT_WIN,
-			(unsigned char *)&(fwin->ewmh_hint_desktop), 1);
+			(unsigned char *)&(fw->ewmh_hint_desktop), 1);
 	}
 	else
 	{
 		ewmh_DeleteProperty(
-			FW_W(fwin), "_NET_WM_DESKTOP",
+			FW_W(fw), "_NET_WM_DESKTOP",
 			EWMH_ATOM_LIST_CLIENT_WIN);
 	}
-	if (HAS_EWMH_WM_ICON_HINT(fwin) == EWMH_FVWM_ICON)
-		EWMH_DeleteWmIcon(fwin, True, True);
+	if (HAS_EWMH_WM_ICON_HINT(fw) == EWMH_FVWM_ICON)
+	{
+		EWMH_DeleteWmIcon(fw, True, True);
+	}
+
+	return;
 }
 
 /* a window are going to be destroyed (in the add_window.c destroy_window
  * sens) */
-void EWMH_DestroyWindow(FvwmWindow *fwin)
+void EWMH_DestroyWindow(FvwmWindow *fw)
 {
-	if (IS_EWMH_DESKTOP(FW_W(fwin)))
+	if (IS_EWMH_DESKTOP(FW_W(fw)))
 	{
 		Scr.EwmhDesktop = NULL;
 	}
-	if (fwin->Desk >= ewmhc.NumberOfDesktops)
+	if (fw->Desk >= ewmhc.NumberOfDesktops)
 	{
 		ewmhc.NeedsToCheckDesk = True;
 	}
+
+	return;
 }
 
 /* a window has been destroyed (unmap/reparent/destroy) */
@@ -1585,6 +1697,8 @@ void EWMH_WindowDestroyed(void)
 	}
 	ewmh_ComputeAndSetWorkArea();
 	ewmh_HandleDynamicWorkArea();
+
+	return;
 }
 
 /*
@@ -1605,6 +1719,7 @@ int set_all_atom_in_list(ewmh_atom *list)
 		l++;
 		list++;
 	}
+
 	return l;
 }
 
@@ -1629,6 +1744,8 @@ void set_net_supported(int l)
 		Scr.Root, "_NET_SUPPORTED", EWMH_ATOM_LIST_FVWM_ROOT,
 		(unsigned char *)supported, k);
 	free(supported);
+
+	return;
 }
 
 static
@@ -1637,13 +1754,15 @@ void clean_up(void)
 	ewmh_ChangeProperty(
 		Scr.Root,"_KDE_NET_SYSTEM_TRAY_WINDOWS",
 		EWMH_ATOM_LIST_FVWM_ROOT, NULL, 0);
+
+	return;
 }
 
 void EWMH_Init(void)
 {
 	int i;
 	int supported_count = 0;
-	CARD32 val;
+	long val;
 	XTextProperty text;
 	unsigned char utf_name[4];
 	char *names[1];
@@ -1699,6 +1818,8 @@ void EWMH_Init(void)
 	EWMH_SetClientList();
 	EWMH_SetClientListStacking();
 	ewmh_ComputeAndSetWorkArea();
+
+	return;
 }
 
 /*
@@ -1706,13 +1827,14 @@ void EWMH_Init(void)
  */
 void EWMH_ExitStuff(void)
 {
-	FvwmWindow *t;
+	FvwmWindow *fw;
 
-	for (t = Scr.FvwmRoot.next; t != NULL; t = t->next)
+	for (fw = Scr.FvwmRoot.next; fw != NULL; fw = fw->next)
 	{
-		EWMH_RestoreInitialStates(t, 0);
+		EWMH_RestoreInitialStates(fw, 0);
 	}
 
+	return;
 }
 
 #ifdef EWMH_DEBUG
@@ -1748,10 +1870,12 @@ void EWMH_DLOG(char *msg, ...)
 	va_end(args);
 	fprintf(stderr, "\n");
 	fprintf(stderr, "            [time]: %s\n",buffer);
+
+	return;
 }
 #endif
 
-void EWMH_fullscreen(FvwmWindow *fwin)
+void EWMH_fullscreen(FvwmWindow *fw)
 {
 	fscreen_scr_arg fscr;
 	rectangle scr_g;
@@ -1762,64 +1886,63 @@ void EWMH_fullscreen(FvwmWindow *fwin)
 
 	/* maximize with ResizeMoveMaximize */
 	if (!is_function_allowed(
-		    F_MAXIMIZE, NULL, fwin, True, False) ||
+		    F_MAXIMIZE, NULL, fw, True, False) ||
 	    !is_function_allowed(
-		    F_MOVE, NULL, fwin, True, False) ||
+		    F_MOVE, NULL, fw, True, False) ||
 	    !is_function_allowed(
-		    F_RESIZE, NULL, fwin, True, True))
+		    F_RESIZE, NULL, fw, True, True))
 	{
 		return;
 	}
-	if (IS_ICONIFIED(fwin))
+	if (IS_ICONIFIED(fw))
 	{
 		execute_function_override_window(
-			NULL, NULL, "Iconify off", 0, fwin);
+			NULL, NULL, "Iconify off", 0, fw);
 	}
-	if (IS_SHADED(fwin))
+	if (IS_SHADED(fw))
 	{
-		int sas = fwin->shade_anim_steps;
+		int sas = fw->shade_anim_steps;
 
-		fwin->shade_anim_steps = 0;
+		fw->shade_anim_steps = 0;
 		execute_function_override_window(
-			NULL, NULL, "WindowShade off", 0, fwin);
-		fwin->shade_anim_steps = sas;
+			NULL, NULL, "WindowShade off", 0, fw);
+		fw->shade_anim_steps = sas;
 	}
-	SET_EWMH_FULLSCREEN(fwin,True);
-	apply_decor_change(fwin);
+	SET_EWMH_FULLSCREEN(fw,True);
+	apply_decor_change(fw);
 	fscr.xypos.x =
-		fwin->frame_g.x + fwin->frame_g.width / 2;
+		fw->frame_g.x + fw->frame_g.width / 2;
 	fscr.xypos.y =
-		fwin->frame_g.y + fwin->frame_g.height / 2;
+		fw->frame_g.y + fw->frame_g.height / 2;
 	FScreenGetScrRect(
 		&fscr, FSCREEN_XYPOS, &scr_g.x, &scr_g.y,
 		&scr_g.width, &scr_g.height);
-	get_window_borders(fwin, &b);
-	get_page_offset_check_visible(&page_x, &page_y, fwin);
+	get_window_borders(fw, &b);
+	get_page_offset_check_visible(&page_x, &page_y, fw);
 	sprintf(
 		cmd, "ResizeMoveMaximize %dp %dp +%dp +%dp",
 		scr_g.width, scr_g.height,
 		scr_g.x - b.top_left.width + page_x,
 		scr_g.y - b.top_left.height + page_y);
-	if (DO_EWMH_USE_STACKING_HINTS(fwin))
+	if (DO_EWMH_USE_STACKING_HINTS(fw))
 	{
-		int sl = fwin->ewmh_normal_layer;
+		int sl = fw->ewmh_normal_layer;
 
-		new_layer(fwin, Scr.TopLayer);
+		new_layer(fw, Scr.TopLayer);
 		if (sl == 0)
 		{
-			fwin->ewmh_normal_layer =
-				Scr.DefaultLayer;
+			fw->ewmh_normal_layer = Scr.DefaultLayer;
 		}
 		else
 		{
-			fwin->ewmh_normal_layer = sl;
+			fw->ewmh_normal_layer = sl;
 		}
 	}
 	if (cmd[0] != 0)
 	{
-		SET_DISABLE_CONSTRAIN_SIZE_FULLSCREEN(fwin, 1);
-		execute_function_override_window(NULL, NULL, cmd, 0, fwin);
-		SET_DISABLE_CONSTRAIN_SIZE_FULLSCREEN(fwin, 0);
+		SET_DISABLE_CONSTRAIN_SIZE_FULLSCREEN(fw, 1);
+		execute_function_override_window(NULL, NULL, cmd, 0, fw);
+		SET_DISABLE_CONSTRAIN_SIZE_FULLSCREEN(fw, 0);
 	}
 
 	return;
