@@ -13,20 +13,18 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
-/* ---------------------------- included header files ----------------------- */
+/* ---------------------------- included header files ---------------------- */
 
-#include <config.h>
+#include "config.h"
 
 #include <stdio.h>
 
 #include "fvwm.h"
-#if 0
-#include "commands.h"
-#endif
 #include "externs.h"
 #include "cursor.h"
 #include "functions.h"
 #include "misc.h"
+#include "move_resize.h"
 #include "screen.h"
 #include "geometry.h"
 #include "read.h"
@@ -34,155 +32,283 @@
 #include "colorset.h"
 #include "schedule.h"
 #include "libs/FGettext.h"
+#include "libs/charmap.h"
+#include "libs/wcontext.h"
 
-/* ---------------------------- local definitions --------------------------- */
+/* ---------------------------- local definitions -------------------------- */
 
-/* ---------------------------- local macros -------------------------------- */
+/* ---------------------------- local macros ------------------------------- */
 
-/* ---------------------------- imports ------------------------------------- */
+/* ---------------------------- imports ------------------------------------ */
 
 extern char const * const Fvwm_VersionInfo;
 
-/* ---------------------------- included code files ------------------------- */
+/* ---------------------------- included code files ------------------------ */
 
-/* ---------------------------- local types --------------------------------- */
+/* ---------------------------- local types -------------------------------- */
 
-/* ---------------------------- forward declarations ------------------------ */
+/* ---------------------------- forward declarations ----------------------- */
 
-/* ---------------------------- local variables ----------------------------- */
+/* ---------------------------- local variables ---------------------------- */
+
+static char *partial_function_vars[] =
+{
+	"bg.cs",
+	"desk.name",
+	"fg.cs",
+	"fgsh.cs",
+	"gt.",
+	"hilight.cs",
+	"shadow.cs",
+	NULL
+};
 
 static char *function_vars[] =
 {
-	"fg.cs",
-	"bg.cs",
-	"hilight.cs",
-	"shadow.cs",
-	"fgsh.cs",
-	"gt.",
-	"desk.name",
-	"desk.n",
-	"desk.width",
-	"desk.height",
-	"desk.pagesx",
-	"desk.pagesy",
-	"vp.x",
-	"vp.y",
-	"vp.width",
-	"vp.height",
-	"page.nx",
-	"page.ny",
-	"w.id",
-	"w.name",
-	"w.iconname",
-	"w.class",
-	"w.resource",
-	"w.x",
-	"w.y",
-	"w.width",
-	"w.height",
+	"cond.rc",
+	"cw.height",
+	"cw.width",
 	"cw.x",
 	"cw.y",
-	"cw.width",
-	"cw.height",
-	"it.x",
-	"it.y",
-	"it.width",
-	"it.height",
-	"ip.x",
-	"ip.y",
-	"ip.width",
-	"ip.height",
+	"desk.height",
+	"desk.n",
+	"desk.pagesx",
+	"desk.pagesy",
+	"desk.width",
+	"func.context",
+	"i.height",
+	"i.width",
 	"i.x",
 	"i.y",
-	"i.width",
-	"i.height",
-	"screen",
-	"schedule.last",
-	"schedule.next",
-	"cond.rc",
-	"pointer.x",
-	"pointer.y",
-	"pointer.wx",
-	"pointer.wy",
+	"ip.height",
+	"ip.width",
+	"ip.x",
+	"ip.y",
+	"it.height",
+	"it.width",
+	"it.x",
+	"it.y",
+	"page.nx",
+	"page.ny",
 	"pointer.cx",
 	"pointer.cy",
-	"version.num",
+	"pointer.wx",
+	"pointer.wy",
+	"pointer.x",
+	"pointer.y",
+	"schedule.last",
+	"schedule.next",
+	"screen",
 	"version.info",
 	"version.line",
+	"version.num",
+	"vp.height",
+	"vp.width",
+	"vp.x",
+	"vp.y",
+	"w.class",
+	"w.height",
+	"w.iconname",
+	"w.iconfile",
+	"w.miniiconfile",
+	"w.id",
+	"w.name",
+	"w.resource",
+	"w.width",
+	"w.x",
+	"w.y",
+	"w.desk",
+	"w.layer",
 	NULL
 };
 
 enum
 {
-	VAR_FG_CS,
 	VAR_BG_CS,
-	VAR_HILIGHT_CS,
-	VAR_SHADOW_CS,
-	VAR_FGSH_CS,
-	VAR_GT,
 	VAR_DESK_NAME,
-	VAR_DESK_N,
-	VAR_DESK_WIDTH,
-	VAR_DESK_HEIGHT,
-	VAR_DESK_PAGESX,
-	VAR_DESK_PAGESY,
-	VAR_VP_X,
-	VAR_VP_Y,
-	VAR_VP_WIDTH,
-	VAR_VP_HEIGHT,
-	VAR_PAGE_NX,
-	VAR_PAGE_NY,
-	VAR_W_ID,
-	VAR_W_NAME,
-	VAR_W_ICONNAME,
-	VAR_W_CLASS,
-	VAR_W_RESOURCE,
-	VAR_W_X,
-	VAR_W_Y,
-	VAR_W_WIDTH,
-	VAR_W_HEIGHT,
+	VAR_FG_CS,
+	VAR_FGSH_CS,
+	VAR_GT_,
+	VAR_HILIGHT_CS,
+	VAR_SHADOW_CS
+} partial_extended_vars;
+
+enum
+{
+	VAR_COND_RC,
+	VAR_CW_HEIGHT,
+	VAR_CW_WIDTH,
 	VAR_CW_X,
 	VAR_CW_Y,
-	VAR_CW_WIDTH,
-	VAR_CW_HEIGHT,
-	VAR_IT_X,
-	VAR_IT_Y,
-	VAR_IT_WIDTH,
-	VAR_IT_HEIGHT,
-	VAR_IP_X,
-	VAR_IP_Y,
-	VAR_IP_WIDTH,
-	VAR_IP_HEIGHT,
+	VAR_DESK_HEIGHT,
+	VAR_DESK_N,
+	VAR_DESK_PAGESX,
+	VAR_DESK_PAGESY,
+	VAR_DESK_WIDTH,
+	VAR_FUNC_CONTEXT,
+	VAR_I_HEIGHT,
+	VAR_I_WIDTH,
 	VAR_I_X,
 	VAR_I_Y,
-	VAR_I_WIDTH,
-	VAR_I_HEIGHT,
-	VAR_SCREEN,
-	VAR_SCHEDULE_LAST,
-	VAR_SCHEDULE_NEXT,
-	VAR_COND_RC,
-	VAR_POINTER_X,
-	VAR_POINTER_Y,
-	VAR_POINTER_WX,
-	VAR_POINTER_WY,
+	VAR_IP_HEIGHT,
+	VAR_IP_WIDTH,
+	VAR_IP_X,
+	VAR_IP_Y,
+	VAR_IT_HEIGHT,
+	VAR_IT_WIDTH,
+	VAR_IT_X,
+	VAR_IT_Y,
+	VAR_PAGE_NX,
+	VAR_PAGE_NY,
 	VAR_POINTER_CX,
 	VAR_POINTER_CY,
-	VAR_VERSION_NUM,
+	VAR_POINTER_WX,
+	VAR_POINTER_WY,
+	VAR_POINTER_X,
+	VAR_POINTER_Y,
+	VAR_SCHEDULE_LAST,
+	VAR_SCHEDULE_NEXT,
+	VAR_SCREEN,
 	VAR_VERSION_INFO,
 	VAR_VERSION_LINE,
+	VAR_VERSION_NUM,
+	VAR_VP_HEIGHT,
+	VAR_VP_WIDTH,
+	VAR_VP_X,
+	VAR_VP_Y,
+	VAR_W_CLASS,
+	VAR_W_HEIGHT,
+	VAR_W_ICONNAME,
+	VAR_W_ICONFILE,
+	VAR_W_MINIICONFILE,
+	VAR_W_ID,
+	VAR_W_NAME,
+	VAR_W_RESOURCE,
+	VAR_W_WIDTH,
+	VAR_W_X,
+	VAR_W_Y,
+	VAR_W_DESK,
+	VAR_W_LAYER
 } extended_vars;
 
-/* ---------------------------- exported variables (globals) ---------------- */
+/* ---------------------------- exported variables (globals) --------------- */
 
-/* ---------------------------- local functions ----------------------------- */
+/* ---------------------------- local functions ---------------------------- */
 
-static int expand_vars_extended(
-	char *var_name, char *output, FvwmWindow *fw,
-	fvwm_cond_func_rc *cond_rc)
+int __eae_parse_range(char *input, unsigned int *lower, unsigned int *upper)
+{
+	int rc;
+	int n;
+
+	*lower = 0;
+	*upper = (unsigned int)-1;
+	if (*input == '*')
+	{
+		return 0;
+	}
+	if (!isdigit(*input))
+	{
+		return -1;
+	}
+	rc = sscanf(input, "%u-%u%n", lower, upper, &n);
+	if (rc < 2)
+	{
+		rc = sscanf(input, "%u%n", lower, &n);
+		if (rc < 1)
+		{
+			/* not a positional argument */
+			return -1;
+		}
+		if (input[n] == '-')
+		{
+			/* $[n- */
+			n++;
+		}
+		else
+		{
+			/* $[n */
+			*upper = *lower;
+		}
+	}
+	input += n;
+	if (*input != 0)
+	{
+		/* trailing characters - not good */
+		return -1;
+	}
+	if (*upper < *lower)
+	{
+		/* the range is reverse - not good */
+		return -1;
+	}
+
+	return 0;
+}
+
+static signed int expand_args_extended(
+	char *input, char *argument_string, char *output)
+{
+	int rc;
+	unsigned int lower;
+	unsigned int upper;
+	unsigned int i;
+	size_t len;
+
+	rc = __eae_parse_range(input, &lower, &upper);
+	if (rc == -1)
+	{
+		return -1;
+	}
+	/* Skip to the start of the requested argument range */
+	if (lower > 0)
+	{
+		argument_string = SkipNTokens(argument_string, lower);
+	}
+	if (!argument_string)
+	{
+		/* replace with empty string */
+		return 0;
+	}
+	/* TODO: optimise handling of $[0] to $[9] which have already been
+	 * parsed */
+	for (i = lower, len = 0; i <= upper; i++)
+	{
+		char *token;
+		size_t tlen;
+
+		token = PeekToken(argument_string, &argument_string);
+		if (token == NULL)
+		{
+			break;
+		}
+		/* copy the token */
+		if (i > lower)
+		{
+			if (output != NULL)
+			{
+				*output = ' ';
+				output++;
+			}
+			len++;
+		}
+		tlen = strlen(token);
+		if (output != NULL && tlen > 0)
+		{
+			memcpy(output, token, tlen);
+			output += tlen;
+		}
+		len += tlen;
+	}
+
+	return (int)len;
+}
+
+static signed int expand_vars_extended(
+	char *var_name, char *output, cond_rc_t *cond_rc,
+	const exec_context_t *exc)
 {
-	char *s;
 	char *rest;
-	char dummy[64];
+	char dummy[64] = "\0";
 	char *target = (output) ? output : dummy;
 	int cs = -1;
 	int n;
@@ -193,12 +319,18 @@ static int expand_vars_extended(
 	Pixel pixel = 0;
 	int val = -12345678;
 	const char *string = NULL;
+	char *allocated_string = NULL;
+	char *quoted_string = NULL;
+	Bool should_quote = False;
 	Bool is_numeric = False;
+	Bool is_target = False;
 	Bool is_x;
 	Window context_w = Scr.Root;
+	FvwmWindow *fw = exc->w.fw;
+	signed int len = -1;
 
-	/* allow partial matches for *.cs variables */
-	switch ((i = GetTokenIndex(var_name, function_vars, -1, &rest)))
+	/* allow partial matches for *.cs, gt, ... etc. variables */
+	switch ((i = GetTokenIndex(var_name, partial_function_vars, -1, &rest)))
 	{
 	case VAR_FG_CS:
 	case VAR_BG_CS:
@@ -208,20 +340,20 @@ static int expand_vars_extended(
 		if (!isdigit(*rest) || (*rest == '0' && *(rest + 1) != 0))
 		{
 			/* not a non-negative integer without leading zeros */
-			return 0;
+			return -1;
 		}
 		if (sscanf(rest, "%d%n", &cs, &n) < 1)
 		{
-			return 0;
+			return -1;
 		}
 		if (*(rest + n) != 0)
 		{
 			/* trailing characters */
-			return 0;
+			return -1;
 		}
 		if (cs < 0)
 		{
-			return 0;
+			return -1;
 		}
 		alloc_colorset(cs);
 		switch (i)
@@ -242,48 +374,35 @@ static int expand_vars_extended(
 			pixel = Colorset[cs].fgsh;
 			break;
 		}
-		return pixel_to_color_string(dpy, Pcmap, pixel, target, False);
-	case VAR_GT:
-	{
+		is_target = True;
+		len = pixel_to_color_string(dpy, Pcmap, pixel, target, False);
+		goto GOT_STRING;
+	case VAR_GT_:
 		if (rest == NULL)
 		{
-			return 0;
+			return -1;
 		}
 		string = _(rest);
-		break;
-	}
+		goto GOT_STRING;
 	case VAR_DESK_NAME:
 		if (sscanf(rest, "%d%n", &cs, &n) < 1)
 		{
-			return 0;
+			return -1;
 		}
 		if (*(rest + n) != 0)
 		{
 			/* trailing characters */
-			return 0;
+			return -1;
 		}
-		s = GetDesktopName(cs);
-		if (s == NULL)
+		string = GetDesktopName(cs);
+		if (string == NULL)
 		{
 			const char *ddn = _("Desk");
-			s = (char *)safemalloc(19 + strlen(ddn));
-			sprintf(s, "%s %i", ddn, cs);
-			l = strlen(s);
-			if (output)
-			{
-				strcpy(output, s);
-			}
-			free(s);
+			allocated_string = (char *)safemalloc(19 + strlen(ddn));
+			sprintf(allocated_string, "%s %i", ddn, cs);
+			string = allocated_string;
 		}
-		else
-		{
-			l = strlen(s);
-			if (output)
-			{
-				strcpy(output, s);
-			}
-		}
-		return l;
+		goto GOT_STRING;
 	default:
 		break;
 	}
@@ -338,32 +457,52 @@ static int expand_vars_extended(
 	case VAR_W_ID:
 		if (fw && !IS_EWMH_DESKTOP(FW_W(fw)))
 		{
+			is_target = True;
 			sprintf(target, "0x%x", (unsigned int)FW_W(fw));
-			string = target;
 		}
 		break;
 	case VAR_W_NAME:
 		if (fw && !IS_EWMH_DESKTOP(FW_W(fw)))
 		{
 			string = fw->name.name;
+			should_quote = True;
 		}
 		break;
 	case VAR_W_ICONNAME:
 		if (fw && !IS_EWMH_DESKTOP(FW_W(fw)))
 		{
 			string = fw->icon_name.name;
+			should_quote = True;
+		}
+		break;
+	case VAR_W_ICONFILE:
+	case VAR_W_MINIICONFILE:
+		if (fw && !IS_EWMH_DESKTOP(FW_W(fw)))
+		{
+			char *t;
+
+			t = (i == VAR_W_ICONFILE) ?
+				fw->icon_bitmap_file : fw->mini_pixmap_file;
+			/* expand the path if possible */
+			string = PictureFindImageFile(t, NULL, R_OK);
+			if (!string)
+			{
+				string = t;
+			}
 		}
 		break;
 	case VAR_W_CLASS:
 		if (fw && !IS_EWMH_DESKTOP(FW_W(fw)))
 		{
 			string = fw->class.res_class;
+			should_quote = True;
 		}
 		break;
 	case VAR_W_RESOURCE:
 		if (fw && !IS_EWMH_DESKTOP(FW_W(fw)))
 		{
 			string = fw->class.res_name;
+			should_quote = True;
 		}
 		break;
 	case VAR_W_X:
@@ -372,7 +511,7 @@ static int expand_vars_extended(
 	case VAR_W_HEIGHT:
 		if (!fw || IS_ICONIFIED(fw) || IS_EWMH_DESKTOP(FW_W(fw)))
 		{
-			return 0;
+			return -1;
 		}
 		else
 		{
@@ -395,7 +534,7 @@ static int expand_vars_extended(
 				val = g.height;
 				break;
 			default:
-				return 0;
+				return -1;
 			}
 		}
 		break;
@@ -405,7 +544,7 @@ static int expand_vars_extended(
 	case VAR_CW_HEIGHT:
 		if (!fw || IS_ICONIFIED(fw) || IS_EWMH_DESKTOP(FW_W(fw)))
 		{
-			return 0;
+			return -1;
 		}
 		else
 		{
@@ -428,7 +567,7 @@ static int expand_vars_extended(
 				val = g.height;
 				break;
 			default:
-				return 0;
+				return -1;
 			}
 		}
 		break;
@@ -438,7 +577,7 @@ static int expand_vars_extended(
 	case VAR_IT_HEIGHT:
 		if (!fw || IS_EWMH_DESKTOP(FW_W(fw)))
 		{
-			return 0;
+			return -1;
 		}
 		else
 		{
@@ -446,7 +585,7 @@ static int expand_vars_extended(
 
 			if (get_visible_icon_title_geometry(fw, &g) == False)
 			{
-				return 0;
+				return -1;
 			}
 			is_numeric = True;
 			switch (i)
@@ -464,7 +603,7 @@ static int expand_vars_extended(
 				val = g.height;
 				break;
 			default:
-				return 0;
+				return -1;
 			}
 		}
 		break;
@@ -474,7 +613,7 @@ static int expand_vars_extended(
 	case VAR_IP_HEIGHT:
 		if (!fw || IS_EWMH_DESKTOP(FW_W(fw)))
 		{
-			return 0;
+			return -1;
 		}
 		else
 		{
@@ -482,7 +621,7 @@ static int expand_vars_extended(
 
 			if (get_visible_icon_picture_geometry(fw, &g) == False)
 			{
-				return 0;
+				return -1;
 			}
 			is_numeric = True;
 			switch (i)
@@ -500,7 +639,7 @@ static int expand_vars_extended(
 				val = g.height;
 				break;
 			default:
-				return 0;
+				return -1;
 			}
 		}
 		break;
@@ -510,7 +649,7 @@ static int expand_vars_extended(
 	case VAR_I_HEIGHT:
 		if (!fw || IS_EWMH_DESKTOP(FW_W(fw)))
 		{
-			return 0;
+			return -1;
 		}
 		else
 		{
@@ -518,7 +657,7 @@ static int expand_vars_extended(
 
 			if (get_visible_icon_geometry(fw, &g) == False)
 			{
-				return 0;
+				return -1;
 			}
 			is_numeric = True;
 			switch (i)
@@ -536,10 +675,33 @@ static int expand_vars_extended(
 				val = g.height;
 				break;
 			default:
-				return 0;
+				return -1;
 			}
 		}
 		break;
+	case VAR_W_DESK:
+		if (!fw || IS_EWMH_DESKTOP(FW_W(fw)))
+		{
+			return -1;
+		}
+		is_numeric = True;
+		if (is_window_sticky_across_desks(fw))
+		{
+			val = Scr.CurrentDesk;
+		}
+		else
+		{
+			val = fw->Desk;
+		}
+		break;
+	case VAR_W_LAYER:
+		if (!fw || IS_EWMH_DESKTOP(FW_W(fw)))
+		{
+			return -1;
+		}
+		is_numeric = True;
+		val = fw->layer;
+		break;
 	case VAR_SCREEN:
 		is_numeric = True;
 		val = Scr.screen;
@@ -555,17 +717,18 @@ static int expand_vars_extended(
 	case VAR_COND_RC:
 		if (cond_rc == NULL)
 		{
-			return 0;
+			return -1;
 		}
-		switch (*cond_rc)
+		switch (cond_rc->rc)
 		{
 		case COND_RC_OK:
 		case COND_RC_NO_MATCH:
 		case COND_RC_ERROR:
-			val = (int)(*cond_rc);
+		case COND_RC_BREAK:
+			val = (int)(cond_rc->rc);
 			break;
 		default:
-			return 0;
+			return -1;
 		}
 		is_numeric = True;
 		break;
@@ -584,7 +747,7 @@ static int expand_vars_extended(
 			if (!fw || IS_ICONIFIED(fw)
 			    || IS_EWMH_DESKTOP(FW_W(fw)))
 			{
-				return 0;
+				return -1;
 			}
 			is_numeric = True;
 			context_w = FW_W_FRAME(fw);
@@ -597,7 +760,7 @@ static int expand_vars_extended(
 			if (!fw || IS_ICONIFIED(fw) || IS_SHADED(fw)
 			    || IS_EWMH_DESKTOP(FW_W(fw)))
 			{
-				return 0;
+				return -1;
 			}
 			is_numeric = True;
 			context_w = FW_W(fw);
@@ -614,7 +777,7 @@ static int expand_vars_extended(
 				  &JunkX, &JunkY, &x, &y, &JunkMask) == False)
 		{
 			/* pointer is on a different screen, don't expand */
-			return 0;
+			return -1;
 		}
 		val = (is_x) ? x : y;
 		break;
@@ -627,49 +790,86 @@ static int expand_vars_extended(
 	case VAR_VERSION_LINE:
 		string = Fvwm_VersionInfo;
 		break;
+	case VAR_FUNC_CONTEXT:
+		is_target = True;
+		target[0] = wcontext_wcontext_to_char(exc->w.wcontext);
+		target[1] = '\0';
+		break;
 	default:
 		/* unknown variable - try to find it in the environment */
-		s = getenv(var_name);
-		if (s)
+		string = getenv(var_name);
+		if (!string)
 		{
+			/* Replace it with unexpanded variable. This is needed
+			 * since var_name might have been expanded */
+			l = strlen(var_name) + 3;
 			if (output)
 			{
-				strcpy(output, s);
+				strcpy(output, "$[");
+				strcpy(output + 2, var_name);
+				output[l - 1] = ']';
+				output[l] = 0;
 			}
-			return strlen(s);
-		}
-		else
-		{
-			return 0;
+			return l;
 		}
 	}
+
+GOT_STRING:
 	if (is_numeric)
 	{
+		is_target = True;
 		sprintf(target, "%d", val);
-		return strlen(target);
+	}
+	if (is_target)
+	{
+		string = target;
 	}
 	else
 	{
-		if (output && string)
+		if (!string)
+		{
+			return -1;
+		}
+		if (output)
 		{
 			strcpy(output, string);
 		}
-		return string ? strlen(string) : 0;
 	}
+	if (len < 0)
+	{
+		len = strlen(string);
+	}
+	if (should_quote)
+	{
+		quoted_string = (char *)safemalloc(len * 2 + 3);
+		len = QuoteString(quoted_string, string) - quoted_string;
+		if (output)
+		{
+			strcpy(output, quoted_string);
+		}
+		free(quoted_string);
+	}
+	if (allocated_string)
+	{
+		free(allocated_string);
+	}
+	return len;
 }
 
-/* ---------------------------- interface functions ------------------------- */
+/* ---------------------------- interface functions ------------------------ */
 
 char *expand_vars(
-	char *input, char *arguments[], FvwmWindow *fw, Bool addto, Bool ismod,
-	fvwm_cond_func_rc *cond_rc)
+	char *input, char *arguments[], Bool addto, Bool ismod,
+	cond_rc_t *cond_rc, const exec_context_t *exc)
 {
 	int l, i, l2, n, k, j, m;
-	int xlen;
+	int xlen, xlevel;
+	Bool name_has_dollar;
 	char *out;
 	char *var;
 	const char *string = NULL;
 	Bool is_string = False;
+	FvwmWindow *fw = exc->w.fw;
 
 	l = strlen(input);
 	l2 = l;
@@ -683,9 +883,9 @@ char *expand_vars(
 	i = 0;
 	while (i < l)
 	{
-		if (input[i] == '$' && (!ismod || !isalpha(input[i+1])))
+		if (input[i] == '$' && (!ismod || !isalpha(input[i + 1])))
 		{
-			switch (input[i+1])
+			switch (input[i + 1])
 			{
 			case '$':
 				/* skip the second $, it is not a part of
@@ -694,25 +894,63 @@ char *expand_vars(
 				break;
 			case '[':
 				/* extended variables */
-				var = &input[i+2];
 				m = i + 2;
-				while (m < l && input[m] != ']' && input[m])
+				var = &input[m];
+				xlevel = 1;
+				name_has_dollar = False;
+				while (m < l && xlevel && input[m])
 				{
-					m++;
+					/* handle nested variables */
+					if (input[m] == ']')
+					{
+						xlevel--;
+					}
+					else if (input[m] == '[')
+					{
+						xlevel++;
+					}
+					else if (input[m] == '$')
+					{
+						name_has_dollar = True;
+					}
+					if (xlevel)
+					{
+						m++;
+					}
 				}
 				if (input[m] == ']')
 				{
 					input[m] = 0;
 					/* handle variable name */
 					k = strlen(var);
-					if (!addto)
+					if (addto)
+					{
+						i += k + 2;
+						input[m] = ']';
+						break;
+					}
+					if (name_has_dollar)
+					{
+						var = expand_vars(
+							var, arguments, addto,
+							ismod, cond_rc, exc);
+					}
+					xlen = expand_args_extended(
+						var, arguments ? arguments[0] :
+						NULL, NULL);
+					if (xlen < 0)
 					{
 						xlen = expand_vars_extended(
-							var, NULL, fw, cond_rc);
-						if (xlen > 0)
-						{
-							l2 += xlen - (k + 2);
-						}
+							var, NULL, cond_rc,
+							exc);
+					}
+					if (name_has_dollar)
+					{
+						free(var);
+					}
+					if (xlen >= 0)
+					{
+						l2 += xlen - (k + 2);
 					}
 					i += k + 2;
 					input[m] = ']';
@@ -729,17 +967,17 @@ char *expand_vars(
 			case '8':
 			case '9':
 			case '*':
-				if (input[i+1] == '*')
+				if (input[i + 1] == '*')
 				{
 					n = 0;
 				}
 				else
 				{
-					n = input[i+1] - '0' + 1;
+					n = input[i + 1] - '0' + 1;
 				}
 				if (arguments[n] != NULL)
 				{
-					l2 += strlen(arguments[n])-2;
+					l2 += strlen(arguments[n]) - 2;
 					i++;
 				}
 				break;
@@ -758,7 +996,7 @@ char *expand_vars(
 			case 'n':
 				if (fw && !IS_EWMH_DESKTOP(FW_W(fw)))
 				{
-					switch(input[i+1])
+					switch(input[i + 1])
 					{
 					case 'c':
 						if (fw->class.res_class &&
@@ -810,14 +1048,14 @@ char *expand_vars(
 	}
 
 	/* Actually create expanded string */
-	i=0;
-	out = safemalloc(l2+1);
-	j=0;
-	while (i<l)
+	i = 0;
+	out = safemalloc(l2 + 1);
+	j = 0;
+	while (i < l)
 	{
-		if (input[i] == '$' && (!ismod || !isalpha(input[i+1])))
+		if (input[i] == '$' && (!ismod || !isalpha(input[i + 1])))
 		{
-			switch (input[i+1])
+			switch (input[i + 1])
 			{
 			case '[':
 				/* extended variables */
@@ -828,21 +1066,57 @@ char *expand_vars(
 					out[j++] = input[i];
 					break;
 				}
-				var = &input[i+2];
 				m = i + 2;
-				while (m < l && input[m] != ']' && input[m])
+				var = &input[m];
+				xlevel = 1;
+				name_has_dollar = False;
+				while (m < l && xlevel && input[m])
 				{
-					m++;
+					/* handle nested variables */
+					if (input[m] == ']')
+					{
+						xlevel--;
+					}
+					else if (input[m] == '[')
+					{
+						xlevel++;
+					}
+					else if (input[m] == '$')
+					{
+						name_has_dollar = True;
+					}
+					if (xlevel)
+					{
+						m++;
+					}
 				}
 				if (input[m] == ']')
 				{
 					input[m] = 0;
 					/* handle variable name */
 					k = strlen(var);
-					xlen = expand_vars_extended(
-						var, &out[j], fw, cond_rc);
+					if (name_has_dollar)
+					{
+						var = expand_vars(
+							var, arguments,	addto,
+							ismod, cond_rc,	exc);
+					}
+					xlen = expand_args_extended(
+						var, arguments ?
+						arguments[0] : NULL,
+						&out[j]);
+					if (xlen < 0)
+					{
+						xlen = expand_vars_extended(
+							var, &out[j], cond_rc,
+							exc);
+					}
+					if (name_has_dollar)
+					{
+						free(var);
+					}
 					input[m] = ']';
-					if (xlen > 0)
+					if (xlen >= 0)
 					{
 						j += xlen;
 						i += k + 2;
@@ -874,17 +1148,17 @@ char *expand_vars(
 			case '8':
 			case '9':
 			case '*':
-				if (input[i+1] == '*')
+				if (input[i + 1] == '*')
 				{
 					n = 0;
 				}
 				else
 				{
-					n = input[i+1] - '0' + 1;
+					n = input[i + 1] - '0' + 1;
 				}
 				if (arguments[n] != NULL)
 				{
-					for (k=0;arguments[n][k];k++)
+					for (k = 0; arguments[n][k]; k++)
 					{
 						out[j++] = arguments[n][k];
 					}
@@ -897,15 +1171,6 @@ char *expand_vars(
 				else
 				{
 					i++;
-#if 0
-					/* DV: Whoa! Better live with the extra
-					 * whitespace. */
-					if (isspace(input[i+1]))
-					{
-						/*eliminates extra white space*/
-						i++;
-					}
-#endif
 				}
 				break;
 			case '.':
@@ -915,43 +1180,35 @@ char *expand_vars(
 			case 'w':
 				if (fw && !IS_EWMH_DESKTOP(FW_W(fw)))
 				{
-#if 0
 					fvwm_msg(OLD, "expand_vars",
 						"Use $[w.id] instead of $w");
-#endif
 					sprintf(&out[j], "0x%x",
 						(unsigned int)FW_W(fw));
 				}
 				else
 				{
-					sprintf(&out[j],"$w");
+					sprintf(&out[j], "$w");
 				}
 				j += strlen(&out[j]);
 				i++;
 				break;
 			case 'd':
-#if 0
 				fvwm_msg(OLD, "expand_vars",
 					"Use $[desk.n] instead of $d");
-#endif
 				sprintf(&out[j], "%d", Scr.CurrentDesk);
 				j += strlen(&out[j]);
 				i++;
 				break;
 			case 'x':
-#if 0
 				fvwm_msg(OLD, "expand_vars",
 					"Use $[vp.x] instead of $x");
-#endif
 				sprintf(&out[j], "%d", Scr.Vx);
 				j += strlen(&out[j]);
 				i++;
 				break;
 			case 'y':
-#if 0
 				fvwm_msg(OLD, "expand_vars",
 					"Use $[vp.y] instead of $y");
-#endif
 				sprintf(&out[j], "%d", Scr.Vy);
 				j += strlen(&out[j]);
 				i++;
@@ -962,14 +1219,12 @@ char *expand_vars(
 			case 'n':
 				if (fw && !IS_EWMH_DESKTOP(FW_W(fw)))
 				{
-					switch(input[i+1])
+					switch(input[i + 1])
 					{
 					case 'c':
-#if 0
 						fvwm_msg(OLD, "expand_vars",
 							"Use $[w.class] "
 							"instead of $c");
-#endif
 						if (fw->class.res_class &&
 						    fw->class.res_class[0])
 						{
@@ -978,11 +1233,9 @@ char *expand_vars(
 						}
 						break;
 					case 'r':
-#if 0
 						fvwm_msg(OLD, "expand_vars",
 							"Use $[w.resource] "
 							"instead of $r");
-#endif
 						if (fw->class.res_name &&
 						    fw->class.res_name[0])
 						{
@@ -991,11 +1244,9 @@ char *expand_vars(
 						}
 						break;
 					case 'n':
-#if 0
 						fvwm_msg(OLD, "expand_vars",
 							"Use $[w.name] "
 							"instead of $n");
-#endif
 						if (fw->name.name &&
 						    fw->name.name[0])
 						{
@@ -1024,18 +1275,6 @@ char *expand_vars(
 			} /* switch */
 			if (is_string && string)
 			{
-#if 0
-				out[j++] = '\'';
-				for (k = 0; string[k]; k++)
-				{
-					if (string[k] == '\'')
-					{
-						out[j++] = '\\';
-					}
-					out[j++] = string[k];
-				}
-				out[j++] = '\'';
-#endif
 				j = QuoteString(&out[j], string) - out;
 				string = NULL;
 				is_string = False;
