@@ -14,21 +14,23 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307	 USA
  */
 
-/****************************************************************************
+/*
  * This module is all original code
  * by Rob Nation
  * Copyright 1993, Robert Nation
  *     You may use this code for any purpose, as long as the original
  *     copyright remains in the source code and all documentation
- ****************************************************************************/
+ */
 
-/* ---------------------------- included header files ----------------------- */
+/* ---------------------------- included header files ---------------------- */
 
-#include <config.h>
+#include "config.h"
 
 #include <stdio.h>
 
 #include "libs/fvwmlib.h"
+#include "libs/charmap.h"
+#include "libs/wcontext.h"
 #include "fvwm.h"
 #include "externs.h"
 #include "execcontext.h"
@@ -45,29 +47,29 @@
 #include "colormaps.h"
 #include "add_window.h"
 
-/* ---------------------------- local definitions --------------------------- */
+/* ---------------------------- local definitions -------------------------- */
 
-/* ---------------------------- local macros -------------------------------- */
+/* ---------------------------- local macros ------------------------------- */
 
-/* ---------------------------- imports ------------------------------------- */
+/* ---------------------------- imports ------------------------------------ */
 
-/* ---------------------------- included code files ------------------------- */
+/* ---------------------------- included code files ------------------------ */
 
-/* ---------------------------- local types --------------------------------- */
+/* ---------------------------- local types -------------------------------- */
 
 typedef struct
 {
 	unsigned do_allow_force_broadcast : 1;
 	unsigned do_forbid_warp : 1;
 	unsigned do_force : 1;
-	unsigned is_focus_by_focus_cmd : 1;
+	unsigned is_focus_by_flip_focus_cmd : 1;
 	unsigned client_entered : 1;
 	fpol_set_focus_by_t set_by;
 } sftfwin_args_t;
 
-/* ---------------------------- forward declarations ------------------------ */
+/* ---------------------------- forward declarations ----------------------- */
 
-/* ---------------------------- local variables ----------------------------- */
+/* ---------------------------- local variables ---------------------------- */
 
 static Bool lastFocusType;
 /* Last window which Fvwm gave the focus to NOT the window that really has the
@@ -76,9 +78,9 @@ static FvwmWindow *ScreenFocus = NULL;
 /* Window which had focus before the pointer moved to a different screen. */
 static FvwmWindow *LastScreenFocus = NULL;
 
-/* ---------------------------- exported variables (globals) ---------------- */
+/* ---------------------------- exported variables (globals) --------------- */
 
-/* ---------------------------- local functions ----------------------------- */
+/* ---------------------------- local functions ---------------------------- */
 
 static Bool focus_get_fpol_context_flag(
 	fpol_context_t *fpol_context, int context)
@@ -88,6 +90,7 @@ static Bool focus_get_fpol_context_flag(
 	switch (context)
 	{
 	case C_WINDOW:
+	case C_EWMH_DESKTOP:
 		flag = fpol_context->client;
 		break;
 	case C_ICON:
@@ -101,9 +104,9 @@ static Bool focus_get_fpol_context_flag(
 	return (flag) ? True : False;
 }
 
-/********************************************************************
+/*
  * Helper functions for setting the focus
- ********************************************************************/
+ */
 static void __try_program_focus(Window w, const FvwmWindow *fw)
 {
 	if (fw && WM_TAKES_FOCUS(fw) &&
@@ -166,41 +169,34 @@ static Bool __check_allow_focus(
 }
 
 static void __update_windowlist(
-	FvwmWindow *fw, fpol_set_focus_by_t set_by, int is_focus_by_focus_cmd)
+	FvwmWindow *fw, fpol_set_focus_by_t set_by,
+	int is_focus_by_flip_focus_cmd)
 {
-	lastFocusType = !is_focus_by_focus_cmd;
-	if (fw == NULL || focus_is_focused(fw) || fw == &Scr.FvwmRoot)
+	lastFocusType = (is_focus_by_flip_focus_cmd) ? True : False;
+	if (fw == NULL || focus_is_focused(fw) || fw == &Scr.FvwmRoot ||
+	    IS_SCHEDULED_FOR_DESTROY(fw))
 	{
 		return;
 	}
-	/* ClickToFocus focus queue manipulation - only performed for
-	 * Focus-by-mouse type focus events */
 	/* Watch out: fw may not be on the windowlist and the windowlist may be
 	 * empty */
-
-	/* pluck window from list and deposit at top */
-
-	if (is_focus_by_focus_cmd ||
-	    FP_DO_SORT_WINDOWLIST_BY(FW_FOCUS_POLICY(fw)) ==
-	    FPOL_SORT_WL_BY_OPEN)
+	if (!is_focus_by_flip_focus_cmd &&
+	    (FP_DO_SORT_WINDOWLIST_BY(FW_FOCUS_POLICY(fw)) ==
+	     FPOL_SORT_WL_BY_OPEN ||
+	     set_by == FOCUS_SET_BY_FUNCTION))
 	{
-		/* move the windowlist around so that fw is at the top
-		 */
+		/* move the windowlist around so that fw is at the top */
 		FvwmWindow *fw2;
 
 		/* find the window on the windowlist */
-		fw2 = &Scr.FvwmRoot;
-		while (fw2 && fw2 != fw)
+		for (fw2 = &Scr.FvwmRoot; fw2 && fw2 != fw; fw2 = fw2->next)
 		{
-			fw2 = fw2->next;
+			/* nothing */
 		}
-
 		if (fw2)
 		{
-			/* the window is on the (non-zero length)
-			 * windowlist */
-			/* make fw2 point to the last window on the
-			 * list */
+			/* the window is on the (non-zero length) windowlist */
+			/* make fw2 point to the last window on the list */
 			while (fw2->next)
 			{
 				fw2 = fw2->next;
@@ -219,6 +215,7 @@ static void __update_windowlist(
 	}
 	else
 	{
+		/* pluck window from list and deposit at top */
 		/* remove fw from list */
 		if (fw->prev)
 		{
@@ -228,7 +225,6 @@ static void __update_windowlist(
 		{
 			fw->next->prev = fw->prev;
 		}
-
 		/* insert fw at start */
 		fw->next = Scr.FvwmRoot.next;
 		if (Scr.FvwmRoot.next)
@@ -263,9 +259,9 @@ static Bool __try_other_screen_focus(const FvwmWindow *fw)
 	return False;
 }
 
-/********************************************************************
+/*
  * Sets the input focus to the indicated window.
- ********************************************************************/
+ */
 static void __set_focus_to_fwin(
 	Window w, FvwmWindow *fw, sftfwin_args_t *args)
 {
@@ -280,7 +276,7 @@ static void __set_focus_to_fwin(
 	{
 		return;
 	}
-	__update_windowlist(fw, args->set_by, args->is_focus_by_focus_cmd);
+	__update_windowlist(fw, args->set_by, args->is_focus_by_flip_focus_cmd);
 	if (__try_other_screen_focus(fw) == True)
 	{
 		return;
@@ -312,6 +308,7 @@ static void __set_focus_to_fwin(
 	{
 		FOCUS_SET(Scr.NoFocusWin);
 		set_focus_window(NULL);
+		Scr.UnknownWinFocused = None;
 		XFlush(dpy);
 		return;
 	}
@@ -346,6 +343,14 @@ static void __set_focus_to_fwin(
 	else if (focus_does_accept_input_focus(fw))
 	{
 		/* Window will accept input focus */
+		if (Scr.StolenFocusWin == w && Scr.UnknownWinFocused != None)
+		{
+			/* Without this FocusIn is not generated on the
+			 * window if it was focuesed when the unmanaged
+			 * window took focus. */
+			FOCUS_SET(Scr.NoFocusWin);
+
+		}
 		FOCUS_SET(w);
 		set_focus_window(fw);
 		if (fw)
@@ -386,13 +391,13 @@ static void set_focus_to_fwin(
 	/* Make sure the button grabs on the new and the old focused windows
 	 * are up to date. */
         if (args->client_entered)
-                {
-                        focus_grab_buttons_client_entered(fw);
-                }
-                else
-                {
-                        focus_grab_buttons(fw);
-                }
+	{
+		focus_grab_buttons_client_entered(fw);
+	}
+	else
+	{
+		focus_grab_buttons(fw);
+	}
         /* RBW -- don't call this twice for the same window!  */
         if (fw != get_focus_window())
         {
@@ -409,11 +414,11 @@ static void set_focus_to_fwin(
 	return;
 }
 
-/**************************************************************************
+/*
  *
  * Moves pointer to specified window
  *
- *************************************************************************/
+ */
 static void warp_to_fvwm_window(
 	const exec_context_t *exc, int warp_x, int x_unit, int warp_y,
 	int y_unit)
@@ -428,12 +433,10 @@ static void warp_to_fvwm_window(
 	{
 		return;
 	}
-
 	if (t->Desk != Scr.CurrentDesk)
 	{
 		goto_desk(t->Desk);
 	}
-
 	if (IS_ICONIFIED(t))
 	{
 		rectangle g;
@@ -452,12 +455,12 @@ static void warp_to_fvwm_window(
 		cx = t->frame_g.x + t->frame_g.width/2;
 		cy = t->frame_g.y + t->frame_g.height/2;
 	}
-
 	dx = (cx + Scr.Vx) / Scr.MyDisplayWidth * Scr.MyDisplayWidth;
 	dy = (cy + Scr.Vy) / Scr.MyDisplayHeight * Scr.MyDisplayHeight;
-
-	MoveViewport(dx,dy,True);
-
+	if (dx != Scr.Vx || dy != Scr.Vy)
+	{
+		MoveViewport(dx, dy, True);
+	}
 	if (IS_ICONIFIED(t))
 	{
 		rectangle g;
@@ -511,9 +514,9 @@ static void warp_to_fvwm_window(
 				(t->frame_g.height - 1) * (100 + warp_y) / 100;
 		}
 	}
-	FWarpPointer(dpy, None, Scr.Root, 0, 0, 0, 0, x, y);
+	FWarpPointerUpdateEvpos(
+		exc->x.elast, dpy, None, Scr.Root, 0, 0, 0, 0, x, y);
 	RaiseWindow(t);
-
 	/* If the window is still not visible, make it visible! */
 	if (t->frame_g.x + t->frame_g.width  < 0 ||
 	    t->frame_g.y + t->frame_g.height < 0 ||
@@ -522,7 +525,8 @@ static void warp_to_fvwm_window(
 	{
 		frame_setup_window(
 			t, 0, 0, t->frame_g.width, t->frame_g.height, False);
-		FWarpPointer(dpy, None, Scr.Root, 0, 0, 0, 0, 2, 2);
+		FWarpPointerUpdateEvpos(
+			exc->x.elast, dpy, None, Scr.Root, 0, 0, 0, 0, 2, 2);
 	}
 
 	return;
@@ -581,7 +585,8 @@ static FvwmWindow *__restore_focus_after_unmap(
 			if (!FP_DO_OVERRIDE_RELEASE_FOCUS(
 				    FW_FOCUS_POLICY(t)) &&
 			    t->Desk == fw->Desk && !DO_SKIP_CIRCULATE(t) &&
-			    !(DO_SKIP_ICON_CIRCULATE(t) && IS_ICONIFIED(t)) &&
+			    !(IS_ICONIFIED(t) && (DO_SKIP_ICON_CIRCULATE(t) ||
+			      IS_ICON_SUPPRESSED(t))) &&
 			    (!do_skip_marked_transients ||
 			     !IS_IN_TRANSIENT_SUBTREE(t)))
 			{
@@ -605,13 +610,13 @@ static FvwmWindow *__restore_focus_after_unmap(
 	return set_focus_to;
 }
 
-/**************************************************************************
+/*
  *
  * Moves focus to specified window; only to be called bay Focus and FlipFocus
  *
- *************************************************************************/
+ */
 static void __activate_window_by_command(
-	F_CMD_ARGS, Bool is_focus_by_focus_cmd)
+	F_CMD_ARGS, int is_focus_by_flip_focus_cmd)
 {
 	int dx;
 	int dy;
@@ -621,8 +626,9 @@ static void __activate_window_by_command(
 	sftfwin_args_t sf_args;
 	FvwmWindow * const fw = exc->w.fw;
 
+	memset(&sf_args, 0, sizeof(sf_args));
 	sf_args.do_allow_force_broadcast = 1;
-	sf_args.is_focus_by_focus_cmd = !!is_focus_by_focus_cmd;
+	sf_args.is_focus_by_flip_focus_cmd = is_focus_by_flip_focus_cmd;
 	sf_args.set_by = FOCUS_SET_BY_FUNCTION;
         sf_args.client_entered = 0;
 	if (fw == NULL || !FP_DO_FOCUS_BY_FUNCTION(FW_FOCUS_POLICY(fw)))
@@ -678,8 +684,9 @@ static void __activate_window_by_command(
 			if (FP_DO_WARP_POINTER_ON_FOCUS_FUNC(
 				    FW_FOCUS_POLICY(fw)))
 			{
-				FWarpPointer(
-					dpy, None, Scr.Root, 0, 0, 0, 0, 2, 2);
+				FWarpPointerUpdateEvpos(
+					exc->x.elast, dpy, None, Scr.Root, 0,
+					0, 0, 0, 2, 2);
 			}
 		}
 	}
@@ -733,7 +740,7 @@ static void __focus_grab_one_button(
 	return;
 }
 
-/* ---------------------------- interface functions ------------------------- */
+/* ---------------------------- interface functions ------------------------ */
 
 Bool focus_does_accept_input_focus(const FvwmWindow *fw)
 {
@@ -799,10 +806,6 @@ Bool focus_query_open_grab_focus(FvwmWindow *fw, FvwmWindow *focus_win)
 			 * focus. */
 			return True;
 		}
-		else
-		{
-			return False;
-		}
 	}
 	else
 	{
@@ -812,10 +815,6 @@ Bool focus_query_open_grab_focus(FvwmWindow *fw, FvwmWindow *focus_win)
 		{
 			return True;
 		}
-		else
-		{
-			return False;
-		}
 	}
 
 	return False;
@@ -852,7 +851,7 @@ static void __focus_grab_buttons(FvwmWindow *fw, Bool client_entered)
 
 	if (fw == NULL || IS_SCHEDULED_FOR_DESTROY(fw) || !IS_MAPPED(fw))
 	{
-		/* It is pointless to grab buttons on dying windows.  Buttons
+		/* It is pointless to grab buttons on dieing windows.  Buttons
 		 * can not be grabbed when the window is unmapped. */
 		return;
 	}
@@ -877,20 +876,22 @@ static void __focus_grab_buttons(FvwmWindow *fw, Bool client_entered)
 
 void focus_grab_buttons(FvwmWindow *fw)
 {
-        return __focus_grab_buttons(fw, False);
+        __focus_grab_buttons(fw, False);
 }
 
 void focus_grab_buttons_client_entered(FvwmWindow *fw)
 {
-        return __focus_grab_buttons(fw, True);
+        __focus_grab_buttons(fw, True);
 }
 
 void focus_grab_buttons_on_layer(int layer)
 {
 	FvwmWindow *fw;
 
-	for (fw = Scr.FvwmRoot.stack_next; fw->layer >= layer;
-	     fw = fw->stack_next)
+	for (
+		fw = Scr.FvwmRoot.stack_next;
+		fw != &Scr.FvwmRoot && fw->layer >= layer;
+		fw = fw->stack_next)
 	{
 		if (fw->layer == layer)
 		{
@@ -919,8 +920,9 @@ void _SetFocusWindow(
 {
 	sftfwin_args_t sf_args;
 
+	memset(&sf_args, 0, sizeof(sf_args));
 	sf_args.do_allow_force_broadcast = !!do_allow_force_broadcast;
-	sf_args.is_focus_by_focus_cmd = 0;
+	sf_args.is_focus_by_flip_focus_cmd = 0;
 	sf_args.do_forbid_warp = 0;
 	sf_args.do_force = 1;
 	sf_args.set_by = set_by;
@@ -941,8 +943,9 @@ void _ReturnFocusWindow(FvwmWindow *fw)
 {
 	sftfwin_args_t sf_args;
 
+	memset(&sf_args, 0, sizeof(sf_args));
 	sf_args.do_allow_force_broadcast = 1;
-	sf_args.is_focus_by_focus_cmd = 0;
+	sf_args.is_focus_by_flip_focus_cmd = 0;
 	sf_args.do_forbid_warp = 1;
         sf_args.client_entered = 0;
 	sf_args.do_force = 0;
@@ -956,8 +959,9 @@ void _DeleteFocus(Bool do_allow_force_broadcast)
 {
 	sftfwin_args_t sf_args;
 
+	memset(&sf_args, 0, sizeof(sf_args));
 	sf_args.do_allow_force_broadcast = !!do_allow_force_broadcast;
-	sf_args.is_focus_by_focus_cmd = 0;
+	sf_args.is_focus_by_flip_focus_cmd = 0;
 	sf_args.do_forbid_warp = 0;
         sf_args.client_entered = 0;
 	sf_args.do_force = 0;
@@ -971,8 +975,9 @@ void _ForceDeleteFocus(void)
 {
 	sftfwin_args_t sf_args;
 
+	memset(&sf_args, 0, sizeof(sf_args));
 	sf_args.do_allow_force_broadcast = 1;
-	sf_args.is_focus_by_focus_cmd = 0;
+	sf_args.is_focus_by_flip_focus_cmd = 0;
 	sf_args.do_forbid_warp = 0;
         sf_args.client_entered = 0;
 	sf_args.do_force = 1;
@@ -1154,19 +1159,19 @@ void refresh_focus(const FvwmWindow *fw)
 	return;
 }
 
-/* ---------------------------- builtin commands ---------------------------- */
+/* ---------------------------- builtin commands --------------------------- */
 
 void CMD_FlipFocus(F_CMD_ARGS)
 {
 	/* Reorder the window list */
-	__activate_window_by_command(F_PASS_ARGS, False);
+	__activate_window_by_command(F_PASS_ARGS, 1);
 
 	return;
 }
 
 void CMD_Focus(F_CMD_ARGS)
 {
-	__activate_window_by_command(F_PASS_ARGS, True);
+	__activate_window_by_command(F_PASS_ARGS, 0);
 
 	return;
 }
@@ -1232,7 +1237,8 @@ void CMD_WarpToWindow(F_CMD_ARGS)
 				y += wh;
 			}
 		}
-		FWarpPointer(dpy, None, exc->w.w, 0, 0, 0, 0, x, y);
+		FWarpPointerUpdateEvpos(
+			exc->x.elast, dpy, None, exc->w.w, 0, 0, 0, 0, x, y);
 	}
 
 	return;
