@@ -69,7 +69,7 @@ typedef struct
 	rectangle curr_sidebar_g;
 	rectangle start_g;
 	rectangle end_g;
-	rectangle delta_g;
+	signed_rectangle delta_g;
 	rectangle current_g;
 	rectangle client_g;
 	int anim_steps;
@@ -96,7 +96,7 @@ typedef struct
 	int next_titlebar_compression;
 	rectangle next_sidebar_g;
 	rectangle next_g;
-	rectangle dstep_g;
+	signed_rectangle dstep_g;
 	size_rect parent_s;
 	int minimal_w_offset;
 	int minimal_h_offset;
@@ -150,7 +150,7 @@ static void combine_gravities(
 	ret_grav->decor_grav = gravity_combine_xy_grav(
 		grav_x->decor_grav, grav_y->decor_grav);
 	ret_grav->title_grav = gravity_combine_xy_grav(
-			grav_x->title_grav, grav_y->title_grav);
+		grav_x->title_grav, grav_y->title_grav);
 	ret_grav->lbutton_grav = gravity_combine_xy_grav(
 		grav_x->lbutton_grav, grav_y->lbutton_grav);
 	ret_grav->rbutton_grav = gravity_combine_xy_grav(
@@ -204,6 +204,7 @@ static void get_resize_decor_gravities_one_axis(
 	case FRAME_MR_SETUP:
 	case FRAME_MR_SETUP_BY_APP:
 		ret_grav->client_grav = neg_grav;
+		break;
 	case FRAME_MR_DONT_DRAW:
 		/* can not happen, just a dummy to keep -Wall happy */
 		break;
@@ -223,16 +224,6 @@ static void frame_get_titlebar_dimensions_only(
 	}
 	switch (GET_TITLE_DIR(fw))
 	{
-	case DIR_N:
-	case DIR_S:
-		ret_titlebar_g->y = (GET_TITLE_DIR(fw) == DIR_N) ?
-			bs->top_left.height :
-			frame_g->height - bs->bottom_right.height -
-			fw->title_thickness;
-		ret_titlebar_g->x = bs->top_left.width;
-		ret_titlebar_g->width = frame_g->width - bs->total_size.width;
-		ret_titlebar_g->height = fw->title_thickness;
-		break;
 	case DIR_W:
 	case DIR_E:
 		ret_titlebar_g->x = (GET_TITLE_DIR(fw) == DIR_W) ?
@@ -244,6 +235,17 @@ static void frame_get_titlebar_dimensions_only(
 		ret_titlebar_g->height =
 			frame_g->height - bs->total_size.height;
 		break;
+	case DIR_N:
+	case DIR_S:
+	default: /* default makes gcc4 happy */
+		ret_titlebar_g->y = (GET_TITLE_DIR(fw) == DIR_N) ?
+			bs->top_left.height :
+			frame_g->height - bs->bottom_right.height -
+			fw->title_thickness;
+		ret_titlebar_g->x = bs->top_left.width;
+		ret_titlebar_g->width = frame_g->width - bs->total_size.width;
+		ret_titlebar_g->height = fw->title_thickness;
+		break;
 	}
 
 	return;
@@ -251,7 +253,7 @@ static void frame_get_titlebar_dimensions_only(
 
 static void frame_setup_border(
 	FvwmWindow *fw, rectangle *frame_g, window_parts setup_parts,
-	rectangle *diff_g)
+	signed_rectangle *diff_g)
 {
 	XWindowChanges xwc;
 	Window w;
@@ -309,7 +311,7 @@ static void frame_setup_border(
 
 static void frame_setup_titlebar(
 	FvwmWindow *fw, rectangle *frame_g, window_parts setup_parts,
-	rectangle *diff_g)
+	signed_rectangle *diff_g)
 {
 	frame_title_layout_t title_layout;
 	int i;
@@ -519,7 +521,7 @@ static int frame_get_titlebar_compression(
  */
 static void frame_get_resize_decor_gravities(
 	frame_decor_gravities_type *ret_grav, direction_t title_dir,
-	frame_move_resize_mode rmode, rectangle *delta_g)
+	frame_move_resize_mode rmode, signed_rectangle *delta_g)
 {
 	frame_decor_gravities_type grav_x;
 	frame_decor_gravities_type grav_y;
@@ -817,7 +819,8 @@ static void frame_mrs_hide_changing_parts(
 	int t_add;
 	int r_add;
 	int b_add;
-	int i;
+	int w;
+	int h;
 
 	t_add = 0;
 	l_add = 0;
@@ -863,26 +866,41 @@ static void frame_mrs_hide_changing_parts(
 		b_add = (mra->dstep_g.height < 0) ? -mra->dstep_g.height : 0;
 		b_add -= mra->minimal_h_offset;
 	}
-	/* cover top/left borders */
-	XMoveResizeWindow(
-		dpy, hide_wins.w[0], 0, 0, mra->current_g.width,
-		mra->b_g.top_left.height + t_add);
-	XMoveResizeWindow(
-		dpy, hide_wins.w[1], 0, 0, mra->b_g.top_left.width + l_add,
-		mra->current_g.height);
-	/* cover bottom/right borders and possibly part of the client */
-	XMoveResizeWindow(
-		dpy, hide_wins.w[2],
-		0,
-		mra->current_g.height - mra->b_g.bottom_right.height - b_add,
-		mra->current_g.width, mra->b_g.bottom_right.height + b_add);
-	XMoveResizeWindow(
-		dpy, hide_wins.w[3],
-		mra->current_g.width - mra->b_g.bottom_right.width - r_add, 0,
-		mra->b_g.bottom_right.width + r_add, mra->current_g.height);
-	for (i = 0; i < 4; i++)
+	/* cover top border */
+	w = mra->current_g.width;
+	h = mra->b_g.top_left.height + t_add;
+	if (w > 0 && h > 0)
 	{
-		XMapWindow(dpy, hide_wins.w[i]);
+		XMoveResizeWindow(dpy, hide_wins.w[0], 0, 0, w, h);
+		XMapWindow(dpy, hide_wins.w[0]);
+	}
+	/* cover left border */
+	w = mra->b_g.top_left.width + l_add;
+	h = mra->current_g.height;
+	if (w > 0 && h > 0)
+	{
+		XMoveResizeWindow(dpy, hide_wins.w[1], 0, 0, w, h);
+		XMapWindow(dpy, hide_wins.w[1]);
+	}
+	/* cover bottom border and possibly part of the client */
+	w = mra->current_g.width;
+	h = mra->b_g.bottom_right.height + b_add;
+	if (w > 0 && h > 0)
+	{
+		XMoveResizeWindow(
+			dpy, hide_wins.w[2], 0, mra->current_g.height -
+			mra->b_g.bottom_right.height - b_add, w, h);
+		XMapWindow(dpy, hide_wins.w[2]);
+	}
+	/* cover right border and possibly part of the client */
+	w = mra->b_g.bottom_right.width + r_add;
+	h = mra->current_g.height;
+	if (w > 0 && h > 0)
+	{
+		XMoveResizeWindow(
+			dpy, hide_wins.w[3], mra->current_g.width -
+			mra->b_g.bottom_right.width - r_add, 0, w, h);
+		XMapWindow(dpy, hide_wins.w[3]);
 	}
 
 	return;
@@ -1183,21 +1201,14 @@ static int frame_get_shading_laziness(
 	}
 }
 
-static void frame_reshape_border(FvwmWindow *fw)
+void frame_reshape_border(FvwmWindow *fw)
 {
-	int dw;
-	int dh;
 	int grav;
 	int off_x = 0;
 	int off_y = 0;
 	rectangle naked_g;
 	rectangle *new_g;
-	size_borders b_old;
-	size_borders b_new;
 
-	get_window_borders(fw, &b_old);
-	dw = b_new.total_size.width - b_old.total_size.width;
-	dh = b_new.total_size.height - b_old.total_size.height;
 	/* calculate the new offsets */
 	if (!IS_MAXIMIZED(fw))
 	{
@@ -1217,7 +1228,6 @@ static void frame_reshape_border(FvwmWindow *fw)
 	gravity_translate_to_northwest_geometry_no_bw(
 		grav, fw, &naked_g, &naked_g);
 	set_window_border_size(fw, fw->unshaped_boundary_width);
-	get_window_borders(fw, &b_new);
 	gravity_add_decoration(grav, fw, new_g, &naked_g);
 	if (IS_MAXIMIZED(fw))
 	{
@@ -1234,13 +1244,17 @@ static void frame_reshape_border(FvwmWindow *fw)
 			SET_SHADED_DIR(fw, GET_TITLE_DIR(fw));
 		}
 		get_shaded_geometry(fw, &fw->frame_g, new_g);
+		frame_force_setup_window(
+			fw, fw->frame_g.x, fw->frame_g.y, fw->frame_g.width,
+			fw->frame_g.height, False);
 	}
 	else
 	{
-		get_relative_geometry(&fw->frame_g, new_g);
+		get_relative_geometry(new_g, new_g);
+		frame_force_setup_window(
+			fw, new_g->x, new_g->y, new_g->width, new_g->height,
+			True);
 	}
-	frame_force_setup_window(
-		fw, new_g->x, new_g->y, new_g->width, new_g->height, True);
 
 	return;
 }
@@ -1313,7 +1327,7 @@ void frame_destroyed_frame(
 }
 
 void frame_get_titlebar_dimensions(
-	FvwmWindow *fw, rectangle *frame_g, rectangle *diff_g,
+	FvwmWindow *fw, rectangle *frame_g, signed_rectangle *diff_g,
 	frame_title_layout_t *title_layout)
 {
 	size_borders b;
@@ -1751,20 +1765,31 @@ frame_move_resize_args frame_create_move_resize_args(
 	mra->w_with_focus = (fw == get_focus_window()) ? FW_W(fw) : None;
 
 	/* calculate various geometries */
-	rc = XGetGeometry(
-		dpy, FW_W(fw), &JunkRoot, &mra->client_g.x, &mra->client_g.y,
-		&mra->client_g.width, &mra->client_g.height, &JunkBW,
-		&JunkDepth);
-	if (rc == True)
+	if (!IS_SHADED(fw))
 	{
-		rc = XTranslateCoordinates(
-			dpy, FW_W_PARENT(fw), Scr.Root, mra->client_g.x,
-			mra->client_g.y, &mra->client_g.x, &mra->client_g.y,
-			&JunkChild);
+		rc = XGetGeometry(
+			dpy, FW_W(fw), &JunkRoot, &mra->client_g.x,
+			&mra->client_g.y, &mra->client_g.width,
+			&mra->client_g.height, &JunkBW,	&JunkDepth);
+		if (rc == True)
+		{
+			rc = XTranslateCoordinates(
+				dpy, FW_W_PARENT(fw), Scr.Root,
+				mra->client_g.x, mra->client_g.y,
+				&mra->client_g.x, &mra->client_g.y,
+				&JunkChild);
+		}
+		if (rc == False)
+		{
+			/* Can only happen if the window died */
+			get_client_geometry(fw, &mra->client_g);
+		}
 	}
-	if (rc == False)
+	else
 	{
-		/* Can only happen if the window died */
+		/* If the window was reisez while shaded the client window
+		 * will not have been resized. Use the frame to get the
+		 * geometry */
 		get_client_geometry(fw, &mra->client_g);
 	}
 	get_window_borders(fw, &mra->b_g);
@@ -1906,14 +1931,14 @@ void frame_free_move_resize_args(
 		border_draw_decorations(
 			fw, PART_ALL,
 			(mra->w_with_focus != None) ? True : False, True,
-			CLEAR_ALL, &mra->current_g, &mra->next_g);
+			CLEAR_ALL, &mra->current_g, &mra->end_g);
 	}
 	else if (mra->trans_parts != PART_NONE)
 	{
 		border_draw_decorations(
 			fw, mra->trans_parts,
 			(mra->w_with_focus != None) ? True : False, True,
-			CLEAR_ALL, &mra->current_g, &mra->next_g);
+			CLEAR_ALL, &mra->current_g, &mra->end_g);
 	}
 	update_absolute_geometry(fw);
 	frame_reparent_hide_windows(Scr.NoFocusWin);
