@@ -14,21 +14,20 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-/****************************************************************************
+/*
  * This module is all original code
  * by Rob Nation
  * Copyright 1993, Robert Nation
  *     You may use this code for any purpose, as long as the original
  *     copyright remains in the source code and all documentation
- ****************************************************************************/
+ */
 
-/* ---------------------------- included header files ----------------------- */
+/* ---------------------------- included header files ---------------------- */
 
-#include <config.h>
+#include "config.h"
 
 #include <stdio.h>
 #include <signal.h>
-#include <sys/wait.h>
 #include <sys/stat.h>
 #ifdef HAVE_FCNTL_H
 #include <fcntl.h>
@@ -49,6 +48,8 @@
 #include "libs/PictureBase.h"
 #include "libs/PictureUtils.h"
 #include "libs/FRenderInit.h"
+#include "libs/charmap.h"
+#include "libs/wcontext.h"
 #include "fvwm.h"
 #include "externs.h"
 #include "cursor.h"
@@ -77,7 +78,7 @@
 #include "menus.h"
 #include "libs/FGettext.h"
 
-/* ---------------------------- local definitions --------------------------- */
+/* ---------------------------- local definitions -------------------------- */
 
 #define MAXHOSTNAME 255
 #define MAX_CFG_CMDS 10
@@ -90,15 +91,15 @@
 #define s_g_width 4
 #define s_g_height 4
 
-/* ---------------------------- local macros -------------------------------- */
+/* ---------------------------- local macros ------------------------------- */
 
-/* ---------------------------- imports ------------------------------------- */
+/* ---------------------------- imports ------------------------------------ */
 
 extern int last_event_type;
 
-/* ---------------------------- included code files ------------------------- */
+/* ---------------------------- included code files ------------------------ */
 
-/* ---------------------------- local types --------------------------------- */
+/* ---------------------------- local types -------------------------------- */
 
 typedef enum
 {
@@ -107,9 +108,9 @@ typedef enum
 	FVWM_RESTART
 } fvwm_run_state_t;
 
-/* ---------------------------- forward declarations ------------------------ */
+/* ---------------------------- forward declarations ----------------------- */
 
-/* ---------------------------- local variables ----------------------------- */
+/* ---------------------------- local variables ---------------------------- */
 
 static char *config_commands[MAX_CFG_CMDS];
 static int num_config_commands=0;
@@ -134,7 +135,7 @@ static const char *initFunctionNames[4] =
 	"Nop"
 };
 
-/* ---------------------------- exported variables (globals) ---------------- */
+/* ---------------------------- exported variables (globals) --------------- */
 
 int master_pid;                 /* process number of 1st fvwm process */
 
@@ -194,15 +195,16 @@ Atom _XA_OL_DECOR_HEADER;
 Atom _XA_OL_DECOR_ICON_NAME;
 
 Atom _XA_WM_WINDOW_ROLE;
+Atom _XA_WINDOW_ROLE;
 Atom _XA_WM_CLIENT_LEADER;
 Atom _XA_SM_CLIENT_ID;
 
 Atom _XA_XROOTPMAP_ID;
 Atom _XA_XSETROOT_ID;
 
-/* ---------------------------- local functions ----------------------------- */
+/* ---------------------------- local functions ---------------------------- */
 
-static void SaveDesktopState()
+static void SaveDesktopState(void)
 {
 	FvwmWindow *t;
 	unsigned long data[1];
@@ -225,9 +227,7 @@ static void SaveDesktopState()
 
 static void InternUsefulAtoms (void)
 {
-	/*
-	 * Create priority colors if necessary.
-	 */
+	/* Create priority colors if necessary. */
 	_XA_MIT_PRIORITY_COLORS = XInternAtom(
 		dpy, "_MIT_PRIORITY_COLORS", False);
 	_XA_WM_CHANGE_STATE = XInternAtom(dpy, "WM_CHANGE_STATE", False);
@@ -240,7 +240,6 @@ static void InternUsefulAtoms (void)
 	_XA_WM_DESKTOP = XInternAtom(dpy, "WM_DESKTOP", False);
 	_XA_MwmAtom=XInternAtom(dpy, "_MOTIF_WM_HINTS",False);
 	_XA_MOTIF_WM=XInternAtom(dpy, "_MOTIF_WM_INFO",False);
-
 	_XA_OL_WIN_ATTR=XInternAtom(dpy, "_OL_WIN_ATTR",False);
 	_XA_OL_WT_BASE=XInternAtom(dpy, "_OL_WT_BASE",False);
 	_XA_OL_WT_CMD=XInternAtom(dpy, "_OL_WT_CMD",False);
@@ -253,11 +252,10 @@ static void InternUsefulAtoms (void)
 	_XA_OL_DECOR_RESIZE=XInternAtom(dpy, "_OL_DECOR_RESIZE",False);
 	_XA_OL_DECOR_HEADER=XInternAtom(dpy, "_OL_DECOR_HEADER",False);
 	_XA_OL_DECOR_ICON_NAME=XInternAtom(dpy, "_OL_DECOR_ICON_NAME",False);
-
 	_XA_WM_WINDOW_ROLE=XInternAtom(dpy, "WM_WINDOW_ROLE",False);
+	_XA_WINDOW_ROLE=XInternAtom(dpy, "WINDOW_ROLE",False);
 	_XA_WM_CLIENT_LEADER=XInternAtom(dpy, "WM_CLIENT_LEADER",False);
 	_XA_SM_CLIENT_ID=XInternAtom(dpy, "SM_CLIENT_ID",False);
-
 	_XA_XROOTPMAP_ID=XInternAtom(dpy, "_XROOTPMAP_ID",False);
 	_XA_XSETROOT_ID=XInternAtom(dpy, "_XSETROOT_ID",False);
 
@@ -280,9 +278,9 @@ static void catch_exit(void)
 	return;
 }
 
-/*************************************************************************
+/*
  * Restart on a signal
- ************************************************************************/
+ */
 static RETSIGTYPE
 Restart(int sig)
 {
@@ -293,7 +291,7 @@ Restart(int sig)
 	 * BEFORE we call it ... */
 	fvwmSetTerminate(sig);
 
-	return;
+	SIGNAL_RETURN;
 }
 
 static RETSIGTYPE
@@ -306,7 +304,7 @@ SigDone(int sig)
 	 * BEFORE we call it ... */
 	fvwmSetTerminate(sig);
 
-	return;
+	SIGNAL_RETURN;
 }
 
 /*
@@ -490,21 +488,50 @@ static int parseCommandArgs(
 	return errorCode ? errorCode : argc;
 }
 
-/***********************************************************************
+/*
+ *
+ */
+static
+char *get_display_name(char *display_name, int screen_num)
+{
+	char *msg;
+	char *new_dn;
+	char *cp;
+	char string_screen_num[32];
+
+	CopyString(&msg, display_name);
+	cp = strchr(msg, ':');
+	if (cp != NULL)
+	{
+		cp = strchr(cp, '.');
+		if (cp != NULL)
+		{
+			/* truncate at display part */
+			*cp = '\0';
+		}
+	}
+	sprintf(string_screen_num, ".%d", screen_num);
+	new_dn = safemalloc(
+		strlen(msg) + strlen(string_screen_num) + 1);
+	new_dn[0] = '\0';
+	strcat(new_dn, msg);
+	strcat(new_dn, string_screen_num);
+	free(msg);
+
+	return new_dn;
+}
+
+/*
  *
  *  Procedure:
  *      Done - tells FVWM to clean up and exit
  *
- ***********************************************************************/
+ */
 /* if restart is true, command must not be NULL... */
 void Done(int restart, char *command)
 {
 	const char *exitFuncName;
 
-	/* XFree freeze hack */
-	XUngrabPointer(dpy, CurrentTime);
-	XUngrabKeyboard(dpy, CurrentTime);
-	XUngrabServer(dpy);
 	if (!restart)
 	{
 		MoveViewport(0,0,False);
@@ -518,13 +545,17 @@ void Done(int restart, char *command)
 
 		char *action = safestrdup(
 			CatString2("Function ", exitFuncName));
-		ecc.type = EXCT_EXIT;
+		ecc.type = restart ? EXCT_TORESTART : EXCT_QUIT;
 		ecc.w.wcontext = C_ROOT;
 		exc = exc_create_context(&ecc, ECC_TYPE | ECC_WCONTEXT);
 		execute_function(NULL, exc, action, 0);
 		exc_destroy_context(exc);
 		free(action);
 	}
+	/* XFree freeze hack */
+	XUngrabPointer(dpy, CurrentTime);
+	XUngrabKeyboard(dpy, CurrentTime);
+	XUngrabServer(dpy);
 	if (!restart)
 	{
 		Reborder();
@@ -680,42 +711,6 @@ void Done(int restart, char *command)
 	exit(0);
 }
 
-/*************************************************************************
- * Reap child processes, preventing them from becoming zombies.
- * We do this asynchronously within the SIGCHLD handler so that
- * "it just happens".
- ************************************************************************/
-static RETSIGTYPE
-ReapChildren(int sig)
-{
-	(void)sig;
-
-	BSD_BLOCK_SIGNALS;
-	/*
-	 * This is a signal handler, AND SO MUST BE REENTRANT!
-	 * Now the wait() functions are safe here, but please don't
-	 * add anything unless you're SURE that the new functions
-	 * (plus EVERYTHING they call) are also reentrant. There
-	 * are very few functions which are truly safe.
-	 */
-#if HAVE_WAITPID
-	while (waitpid(-1, NULL, WNOHANG) > 0)
-	{
-		/* nothing to do here */
-	}
-#elif HAVE_WAIT3
-	while (wait3(NULL, WNOHANG, NULL) > 0)
-	{
-		/* nothing to do here */
-	}
-#else
-# error One of waitpid or wait3 is needed.
-#endif
-	BSD_UNBLOCK_SIGNALS;
-
-	return;
-}
-
 /***********************************************************************
  *
  *  Procedure:
@@ -758,30 +753,25 @@ InstallSignals(void)
 	sigaction(SIGQUIT, &sigact, NULL);
 	sigaction(SIGTERM, &sigact, NULL);
 
-	/*
-	 * Reap all zombies automatically! This signal handler will
-	 * only be called if a child process dies, not if someone
-	 * sends a child a STOP signal. Note that none of our "terminate"
-	 * signals can be delivered until the SIGCHLD handler completes,
-	 * and this is a Good Thing because the terminate handlers
-	 * might exit abruptly via "siglongjmp". This could potentially
-	 * leave SIGCHLD handler with unfinished business ...
+	/* Reap all zombies automatically! This signal handler will only be
+	 * called if a child process dies, not if someone sends a child a STOP
+	 * signal. Note that none of our "terminate" signals can be delivered
+	 * until the SIGCHLD handler completes, and this is a Good Thing
+	 * because the terminate handlers might exit abruptly via "siglongjmp".
+	 * This could potentially leave SIGCHLD handler with unfinished
+	 * business ...
 	 *
-	 * NOTE: We could still receive SIGPIPE signals within the
-	 *       SIGCHLD handler, but the SIGPIPE handler has the
-	 *       SA_RESTART flag set and so should not affect our
-	 *       "wait" system call.
-	 */
+	 * NOTE:  We could still receive SIGPIPE signals within the SIGCHLD
+	 * handler, but the SIGPIPE handler has the SA_RESTART flag set and so
+	 * should not affect our "wait" system call. */
 	sigact.sa_flags |= SA_NOCLDSTOP;
-	sigact.sa_handler = ReapChildren;
+	sigact.sa_handler = fvwmReapChildren;
 	sigaction(SIGCHLD, &sigact, NULL);
 #else
 #ifdef USE_BSD_SIGNALS
-	fvwmSetSignalMask( sigmask(SIGUSR1) |
-			   sigmask(SIGINT)  |
-			   sigmask(SIGHUP)  |
-			   sigmask(SIGQUIT) |
-			   sigmask(SIGTERM) );
+	fvwmSetSignalMask(
+		sigmask(SIGUSR1) | sigmask(SIGINT) | sigmask(SIGHUP) |
+		sigmask(SIGQUIT) | sigmask(SIGTERM) );
 #endif
 
 	/*
@@ -810,7 +800,7 @@ InstallSignals(void)
 #ifdef HAVE_SIGINTERRUPT
 	siginterrupt(SIGTERM, 0);
 #endif
-	signal(SIGCHLD, ReapChildren);
+	signal(SIGCHLD, fvwmReapChildren);
 #ifdef HAVE_SIGINTERRUPT
 	siginterrupt(SIGCHLD, 0);
 #endif
@@ -822,7 +812,7 @@ InstallSignals(void)
 	 * table if they exited while the default handler was in place.
 	 * We fix this by invoking the SIGCHLD handler NOW, so that they
 	 * may finally rest in peace. */
-	ReapChildren(0);
+	fvwmReapChildren(0);
 
 	return;
 }
@@ -845,11 +835,11 @@ static void LoadDefaultLeftButton(DecorFace *df, int i)
 	case 0:
 	case 4:
 		v->num = 5;
-		v->x = safemalloc(sizeof(char) * v->num);
-		v->y = safemalloc(sizeof(char) * v->num);
-		v->xoff = safemalloc(sizeof(char) * v->num);
-		v->yoff = safemalloc(sizeof(char) * v->num);
-		v->c = safecalloc(v->num, sizeof(char));
+		v->x = (signed char*)safemalloc(sizeof(char) * v->num);
+		v->y = (signed char*)safemalloc(sizeof(char) * v->num);
+		v->xoff = (signed char*)safemalloc(sizeof(char) * v->num);
+		v->yoff = (signed char*)safemalloc(sizeof(char) * v->num);
+		v->c = (signed char*)safecalloc(v->num, sizeof(char));
 		v->x[0] = 22;
 		v->y[0] = 39;
 		v->c[0] = 1;
@@ -866,11 +856,11 @@ static void LoadDefaultLeftButton(DecorFace *df, int i)
 		break;
 	case 1:
 		v->num = 5;
-		v->x = safemalloc(sizeof(char) * v->num);
-		v->y = safemalloc(sizeof(char) * v->num);
-		v->xoff = safemalloc(sizeof(char) * v->num);
-		v->yoff = safemalloc(sizeof(char) * v->num);
-		v->c = safecalloc(v->num, sizeof(char));
+		v->x = (signed char*)safemalloc(sizeof(char) * v->num);
+		v->y = (signed char*)safemalloc(sizeof(char) * v->num);
+		v->xoff = (signed char*)safemalloc(sizeof(char) * v->num);
+		v->yoff = (signed char*)safemalloc(sizeof(char) * v->num);
+		v->c = (signed char*)safecalloc(v->num, sizeof(char));
 		v->x[0] = 32;
 		v->y[0] = 45;
 		v->x[1] = 68;
@@ -886,11 +876,11 @@ static void LoadDefaultLeftButton(DecorFace *df, int i)
 		break;
 	case 2:
 		v->num = 5;
-		v->x = safemalloc(sizeof(char) * v->num);
-		v->y = safemalloc(sizeof(char) * v->num);
-		v->xoff = safemalloc(sizeof(char) * v->num);
-		v->yoff = safemalloc(sizeof(char) * v->num);
-		v->c = safecalloc(v->num, sizeof(char));
+		v->x = (signed char*)safemalloc(sizeof(char) * v->num);
+		v->y = (signed char*)safemalloc(sizeof(char) * v->num);
+		v->xoff = (signed char*)safemalloc(sizeof(char) * v->num);
+		v->yoff = (signed char*)safemalloc(sizeof(char) * v->num);
+		v->c = (signed char*)safecalloc(v->num, sizeof(char));
 		v->x[0] = 49;
 		v->y[0] = 49;
 		v->c[0] = 1;
@@ -907,11 +897,11 @@ static void LoadDefaultLeftButton(DecorFace *df, int i)
 		break;
 	case 3:
 		v->num = 5;
-		v->x = safemalloc(sizeof(char) * v->num);
-		v->y = safemalloc(sizeof(char) * v->num);
-		v->xoff = safemalloc(sizeof(char) * v->num);
-		v->yoff = safemalloc(sizeof(char) * v->num);
-		v->c = safecalloc(v->num, sizeof(char));
+		v->x = (signed char*)safemalloc(sizeof(char) * v->num);
+		v->y = (signed char*)safemalloc(sizeof(char) * v->num);
+		v->xoff = (signed char*)safemalloc(sizeof(char) * v->num);
+		v->yoff = (signed char*)safemalloc(sizeof(char) * v->num);
+		v->c = (signed char*)safecalloc(v->num, sizeof(char));
 		v->x[0] = 32;
 		v->y[0] = 45;
 		v->c[0] = 1;
@@ -955,11 +945,11 @@ static void LoadDefaultRightButton(DecorFace *df, int i)
 	case 0:
 	case 3:
 		v->num = 5;
-		v->x = safemalloc(sizeof(char) * v->num);
-		v->y = safemalloc(sizeof(char) * v->num);
-		v->xoff = safemalloc(sizeof(char) * v->num);
-		v->yoff = safemalloc(sizeof(char) * v->num);
-		v->c = safecalloc(v->num, sizeof(char));
+		v->x = (signed char*)safemalloc(sizeof(char) * v->num);
+		v->y = (signed char*)safemalloc(sizeof(char) * v->num);
+		v->xoff = (signed char*)safemalloc(sizeof(char) * v->num);
+		v->yoff = (signed char*)safemalloc(sizeof(char) * v->num);
+		v->c = (signed char*)safecalloc(v->num, sizeof(char));
 		v->x[0] = 25;
 		v->y[0] = 25;
 		v->c[0] = 1;
@@ -976,11 +966,11 @@ static void LoadDefaultRightButton(DecorFace *df, int i)
 		break;
 	case 1:
 		v->num = 5;
-		v->x = safemalloc(sizeof(char) * v->num);
-		v->y = safemalloc(sizeof(char) * v->num);
-		v->xoff = safemalloc(sizeof(char) * v->num);
-		v->yoff = safemalloc(sizeof(char) * v->num);
-		v->c = safecalloc(v->num, sizeof(char));
+		v->x = (signed char*)safemalloc(sizeof(char) * v->num);
+		v->y = (signed char*)safemalloc(sizeof(char) * v->num);
+		v->xoff = (signed char*)safemalloc(sizeof(char) * v->num);
+		v->yoff = (signed char*)safemalloc(sizeof(char) * v->num);
+		v->c = (signed char*)safecalloc(v->num, sizeof(char));
 		v->x[0] = 39;
 		v->y[0] = 39;
 		v->c[0] = 1;
@@ -997,11 +987,11 @@ static void LoadDefaultRightButton(DecorFace *df, int i)
 		break;
 	case 2:
 		v->num = 5;
-		v->x = safemalloc(sizeof(char) * v->num);
-		v->y = safemalloc(sizeof(char) * v->num);
-		v->xoff = safemalloc(sizeof(char) * v->num);
-		v->yoff = safemalloc(sizeof(char) * v->num);
-		v->c = safecalloc(v->num, sizeof(char));
+		v->x = (signed char*)safemalloc(sizeof(char) * v->num);
+		v->y = (signed char*)safemalloc(sizeof(char) * v->num);
+		v->xoff = (signed char*)safemalloc(sizeof(char) * v->num);
+		v->yoff = (signed char*)safemalloc(sizeof(char) * v->num);
+		v->c = (signed char*)safecalloc(v->num, sizeof(char));
 		v->x[0] = 49;
 		v->y[0] = 49;
 		v->c[0] = 1;
@@ -1018,11 +1008,11 @@ static void LoadDefaultRightButton(DecorFace *df, int i)
 		break;
 	case 4:
 		v->num = 5;
-		v->x = safemalloc(sizeof(char) * v->num);
-		v->y = safemalloc(sizeof(char) * v->num);
-		v->xoff = safemalloc(sizeof(char) * v->num);
-		v->yoff = safemalloc(sizeof(char) * v->num);
-		v->c = safecalloc(v->num, sizeof(char));
+		v->x = (signed char*)safemalloc(sizeof(char) * v->num);
+		v->y = (signed char*)safemalloc(sizeof(char) * v->num);
+		v->xoff = (signed char*)safemalloc(sizeof(char) * v->num);
+		v->yoff = (signed char*)safemalloc(sizeof(char) * v->num);
+		v->c = (signed char*)safecalloc(v->num, sizeof(char));
 		v->x[0] = 36;
 		v->y[0] = 36;
 		v->c[0] = 1;
@@ -1076,7 +1066,9 @@ static void CreateGCs(void)
 	c.pixel = GetColor(DEFAULT_FORE_COLOR);
 	Scr.ScratchMonoPixmap = XCreatePixmap(dpy, Scr.Root, 1, 1, 1);
 	Scr.MonoGC = fvwmlib_XCreateGC(dpy, Scr.ScratchMonoPixmap, gcm, &gcv);
-
+	Scr.ScratchAlphaPixmap = XCreatePixmap(
+		dpy, Scr.Root, 1, 1, FRenderGetAlphaDepth());
+	Scr.AlphaGC = fvwmlib_XCreateGC(dpy, Scr.ScratchAlphaPixmap, gcm, &gcv);
 	return;
 }
 
@@ -1188,7 +1180,7 @@ static void InitVariables(void)
 	   "WRQ, Inc."
 	   is the ServerVendor string of the Reflection X server under Windows.
 	*/
-	Scr.bo.RaiseHackNeeded =
+	Scr.bo.is_raise_hack_needed =
 		(strcmp (
 			ServerVendor (dpy),
 			"Hummingbird Communications Ltd.") == 0) ||
@@ -1197,14 +1189,14 @@ static void InitVariables(void)
 			"Network Computing Devices Inc.") == 0) ||
 		(strcmp (ServerVendor (dpy), "WRQ, Inc.") == 0);
 
-	Scr.bo.ModalityIsEvil = 0;
-	Scr.bo.DisableConfigureNotify = 0;
-	Scr.bo.InstallRootCmap = 0;
-	Scr.bo.FlickeringQtDialogsWorkaround = 1;
-	Scr.bo.EWMHIconicStateWorkaround = 0;
+	Scr.bo.is_modality_evil = 0;
+	Scr.bo.do_disable_configure_notify = 0;
+	Scr.bo.do_install_root_cmap = 0;
+	Scr.bo.do_enable_flickering_qt_dialogs_workaround = 1;
+	Scr.bo.do_enable_ewmh_iconic_state_workaround = 0;
 
-	Scr.gs.EmulateMWM = DEFAULT_EMULATE_MWM;
-	Scr.gs.EmulateWIN = DEFAULT_EMULATE_WIN;
+	Scr.gs.do_emulate_mwm = DEFAULT_EMULATE_MWM;
+	Scr.gs.do_emulate_win = DEFAULT_EMULATE_WIN;
 	Scr.gs.use_active_down_buttons = DEFAULT_USE_ACTIVE_DOWN_BUTTONS;
 	Scr.gs.use_inactive_buttons = DEFAULT_USE_INACTIVE_BUTTONS;
 	Scr.gs.use_inactive_down_buttons = DEFAULT_USE_INACTIVE_DOWN_BUTTONS;
@@ -1216,6 +1208,11 @@ static void InitVariables(void)
 	Scr.PanFrameBottom.command = NULL;
 	Scr.PanFrameRight.command  = NULL;
 	Scr.PanFrameLeft.command   = NULL;
+	/* EdgeLeaveCommands - no edge leave commands by default */
+	Scr.PanFrameTop.command_leave    = NULL;
+	Scr.PanFrameBottom.command_leave = NULL;
+	Scr.PanFrameRight.command_leave  = NULL;
+	Scr.PanFrameLeft.command_leave   = NULL;
 	Scr.flags.is_pointer_on_this_screen = !!FQueryPointer(
 		dpy, Scr.Root, &JunkRoot, &JunkChild, &JunkX, &JunkY, &JunkX,
 		&JunkY, &JunkMask);
@@ -1233,7 +1230,7 @@ static void usage(int is_verbose)
 		" [-d display]"
 		" [-f cfgfile]"
 		" [-c cmd]"
-		" [-s]"
+		" [-s [screen_num]]"
 		" [-I vis-id | -C vis-class]"
 		" [-l colors"
 		" [-L|A|S|P] ...]"
@@ -1262,7 +1259,7 @@ static void usage(int is_verbose)
 		" -L:           strict color limit\n"
 		" -P:           visual palette\n"
 		" -r:           replace running window manager\n"
-		" -s:           manage a single screen\n"
+		" -s [screen]:  manage a single screen\n"
 		" -S:           static palette\n"
 		" -V:           print version information\n"
 		);
@@ -1307,15 +1304,12 @@ static void setVersionInfo(void)
 #endif
 #ifdef HAVE_PNG
 	strcat(support_str, " PNG,");
-#endif
-#ifdef GNOME
-	strcat(support_str, " GNOME WM hints,");
-#endif
-#ifdef HAVE_EWMH
-	strcat(support_str, " EWMH hints,");
 #endif
 	if (FHaveShapeExtension)
 		strcat(support_str, " Shape,");
+#ifdef HAVE_XSHM
+	strcat(support_str, " XShm,");
+#endif
 #ifdef SESSION
 	strcat(support_str, " SM,");
 #endif
@@ -1325,9 +1319,15 @@ static void setVersionInfo(void)
 #ifdef HAVE_XINERAMA
 	strcat(support_str, " Xinerama,");
 #endif
+#ifdef HAVE_XRENDER
+	strcat(support_str, " XRender,");
+#endif
 #ifdef HAVE_XFT
 	strcat(support_str, " XFT,");
 #endif
+#ifdef HAVE_NLS
+	strcat(support_str, " NLS,");
+#endif
 
 	support_len = strlen(support_str);
 	if (support_len > 0)
@@ -1351,45 +1351,61 @@ static void SetRCDefaults(void)
 	int i;
 	/* set up default colors, fonts, etc */
 	const char *defaults[][3] = {
-		{"XORValue 0","",""},
-		{"DefaultFont","",""},
-		{"DefaultColors black grey","",""},
-		{DEFAULT_MENU_STYLE,"",""},
-		{"TitleStyle Centered -- Raised"},
-		{"Style * Color lightgrey/dimgrey","",""},
-		{"Style * HilightFore black, HilightBack grey","",""},
-		{"AddToMenu MenuFvwmRoot \"",
-		 _("Builtin Menu"), "\" Title"},
-		{"+ \"&1. XTerm\" Exec xterm","",""},
-		{"+ \"&2. ",
-		_("Setup Form"),
-		"\" Module FvwmForm FvwmForm-Setup"},
-		{"+ \"&3. ",
-		 _("Setup 95 Script"), 
-		 "\" Module FvwmScript FvwmScript-Setup95"},
-		{"+ \"&4. ",
-		 _("Issue FVWM commands"),
-		"\" Module FvwmConsole"},
-		{"+ \"&R. ",
-		 _("Restart FVWM"), "\" Restart"},
-		{"+ \"&X. ", gettext("Exit FVWM"), "\" Quit"},
-		{"Mouse 0 R N Menu MenuFvwmRoot","",""},
-		{"Read "FVWM_DATADIR"/ConfigFvwmDefaults","",""},
-		{NULL,NULL,NULL}
+		{ "XORValue 0", "", "" },
+		{ "DefaultFont", "", "" },
+		{ "DefaultColors black grey", "", "" },
+		{ DEFAULT_MENU_STYLE, "", "" },
+		{ "TitleStyle Centered -- Raised", "", "" },
+		{ "Style * Color lightgrey/dimgrey", "", "" },
+		{ "Style * HilightFore black, HilightBack grey", "", "" },
+		{
+			"AddToMenu MenuFvwmRoot \"",
+			_("Builtin Menu"),
+			"\" Title"
+		},
+		{ "+ \"&1. XTerm\" Exec xterm", "", ""},
+		{
+			"+ \"&2. ",
+			_("Setup Form"),
+			"\" Module FvwmForm FvwmForm-Setup"
+		},
+		{
+			"+ \"&3. ",
+			_("Setup 95 Script"),
+			"\" Module FvwmScript FvwmScript-Setup95"
+		},
+		{
+			"+ \"&4. ",
+			_("Issue FVWM commands"),
+			"\" Module FvwmConsole"
+		},
+		{
+			"+ \"&R. ",
+			_("Restart FVWM"),
+			"\" Restart"
+		},
+		{
+			"+ \"&X. ",
+			_("Exit FVWM"),
+			"\" Quit"
+		},
+		{ "Mouse 1 R A Menu MenuFvwmRoot", "", "" },
+		{ "Read "FVWM_DATADIR"/ConfigFvwmDefaults", "", "" },
+		{ NULL, NULL, NULL }
 	};
 
-	for (i = 0; defaults[i][0]; i++)
+	for (i = 0; defaults[i][0] != NULL; i++)
 	{
 		const exec_context_t *exc;
 		exec_context_changes_t ecc;
+		char *cmd;
 
-		ecc.type = EXCT_INIT;
+		ecc.type = Restarting ? EXCT_RESTART : EXCT_INIT;
 		ecc.w.wcontext = C_ROOT;
 		exc = exc_create_context(&ecc, ECC_TYPE | ECC_WCONTEXT);
-		execute_function(
-			NULL, exc,
-			CatString3(defaults[i][0],defaults[i][1],defaults[i][2]),
-			0);
+		cmd = CatString3(
+			defaults[i][0], defaults[i][1], defaults[i][2]);
+		execute_function(NULL, exc, cmd, 0);
 		exc_destroy_context(exc);
 	}
 
@@ -1450,7 +1466,7 @@ static int FvwmErrorHandler(Display *dpy, XErrorEvent *event)
 	return 0;
 }
 
-/* ---------------------------- interface functions ------------------------- */
+/* ---------------------------- interface functions ------------------------ */
 
 /* Does initial window captures and runs init/restart function */
 void StartupStuff(void)
@@ -1460,9 +1476,9 @@ void StartupStuff(void)
 	const exec_context_t *exc;
 	exec_context_changes_t ecc;
 
-	ecc.type = EXCT_INIT;
+	ecc.type = Restarting ? EXCT_RESTART : EXCT_INIT;
 	ecc.w.wcontext = C_ROOT;
-	exc = exc_create_context(&ecc, ECC_TYPE | ECC_W);
+	exc = exc_create_context(&ecc, ECC_TYPE | ECC_WCONTEXT);
 	CaptureAllWindows(exc, False);
 	/* Turn off the SM stuff after the initial capture so that new windows
 	 * will not be matched by accident. */
@@ -1485,10 +1501,17 @@ void StartupStuff(void)
 		Scr.ClickTime = -Scr.ClickTime;
 	}
 
-	/* It is safe to Ungrab here: if not and one of the init functions not
-	 * finish we've got a complete freeze ! */
+# if 0
+	/* It is safe to ungrab here: if not, and one of the init functions
+	 * does not finish, we've got a complete freeze! */
+	/* DV (15-Jul-2004): No, it is not safe to ungrab.  If another
+	 * application grabs the pointer before execute_function gets it, the
+	 * start functions are not executed.  And the pointer is grabbed
+	 * during function execution anyway, so releasing it here buys us
+	 * nothing. */
 	UngrabEm(GRAB_STARTUP);
 	XUngrabPointer(dpy, CurrentTime);
+#endif
 
 	/* migo (04-Sep-1999): execute StartFunction */
 	if (functions_is_complex_function(startFuncName))
@@ -1508,11 +1531,12 @@ void StartupStuff(void)
 		execute_function(NULL, exc, action, 0);
 		free(action);
 	}
+	/* see comment above */
+	UngrabEm(GRAB_STARTUP);
+	XUngrabPointer(dpy, CurrentTime);
 
-	/*
-	  This should be done after the initialization is finished, since
-	  it directly changes the global state.
-	*/
+	/* This should be done after the initialization is finished, since
+	 * it directly changes the global state. */
 	LoadGlobalState(state_filename);
 
 	/*
@@ -1561,7 +1585,8 @@ void DestroyAllButtons(FvwmDecor *decor)
 	int i;
 	int j;
 
-	for (tbp = decor->buttons, i = 0; i < NUMBER_OF_BUTTONS; i++, tbp++)
+	for (tbp = decor->buttons, i = 0; i < NUMBER_OF_TITLE_BUTTONS;
+	     i++, tbp++)
 	{
 		for (j = 0, face = TB_STATE(*tbp); j < BS_MaxButtonState;
 		     j++, face++)
@@ -1580,7 +1605,9 @@ void ResetAllButtons(FvwmDecor *decor)
 	int i;
 	int j;
 
-	for (tbp = decor->buttons, i = 0; i < NUMBER_OF_BUTTONS; i++, tbp++)
+	DestroyAllButtons(decor);
+	for (tbp = decor->buttons, i = 0; i < NUMBER_OF_TITLE_BUTTONS;
+	     i++, tbp++)
 	{
 		memset(&TB_FLAGS(*tbp), 0, sizeof(TB_FLAGS(*tbp)));
 		TB_JUSTIFICATION(*tbp) = JUST_CENTER;
@@ -1604,8 +1631,9 @@ void SetMWM_INFO(Window window)
 {
 	struct mwminfo
 	{
-		long flags;
-		Window win;
+		long props[2];
+		/* prop[0]: flags */
+		/* prop[1]: win */
 	}  motif_wm_info;
 	static char set_yorn='n';
 
@@ -1614,12 +1642,12 @@ void SetMWM_INFO(Window window)
 		return;
 	}
 
-	if (Scr.bo.ModalityIsEvil)
+	if (Scr.bo.is_modality_evil)
 	{
 		/* Set Motif WM_INFO atom to make motif relinquish
 		 * broken handling of modal dialogs */
-		motif_wm_info.flags = 2;
-		motif_wm_info.win = window;
+		motif_wm_info.props[0] = 2;
+		motif_wm_info.props[1] = window;
 		XChangeProperty(
 			dpy,Scr.Root, _XA_MOTIF_WM, _XA_MOTIF_WM,32,
 			PropModeReplace, (unsigned char *)&motif_wm_info, 2);
@@ -1661,8 +1689,8 @@ int main(int argc, char **argv)
 	int i;
 	int len;
 	char *display_string;
-	char message[255];
 	Bool do_force_single_screen = False;
+	int single_screen_num = -1;
 	Bool replace_wm = False;
 	int visualClass = -1;
 	int visualId = -1;
@@ -1688,7 +1716,7 @@ int main(int argc, char **argv)
 	setVersionInfo();
 	/* Put the default module directory into the environment so it can be
 	 * used later by the config file, etc.  */
-	putenv("FVWM_MODULEDIR=" FVWM_MODULEDIR);
+	flib_putenv("FVWM_MODULEDIR", "FVWM_MODULEDIR=" FVWM_MODULEDIR);
 
 	/* Figure out user's home directory */
 	home_dir = getenv("HOME");
@@ -1711,10 +1739,14 @@ int main(int argc, char **argv)
 	fvwm_userdir = getenv("FVWM_USERDIR");
 	if (fvwm_userdir == NULL)
 	{
+		char *s;
+
 		fvwm_userdir = safestrdup(CatString2(home_dir, "/.fvwm"));
 		/* Put the user directory into the environment so it can be used
 		 * later everywhere. */
-		putenv(safestrdup(CatString2("FVWM_USERDIR=", fvwm_userdir)));
+		s = safestrdup(CatString2("FVWM_USERDIR=", fvwm_userdir));
+		flib_putenv("FVWM_USERDIR", s);
+		free(s);
 	}
 
 	/* Create FVWM_USERDIR directory if needed */
@@ -1771,6 +1803,16 @@ int main(int argc, char **argv)
 			 strcmp(argv[i], "--single-screen") == 0)
 		{
 			do_force_single_screen = True;
+			if (i+1 < argc && argv[i+1][0] != '-')
+			{
+				i++;
+				if (sscanf(argv[i], "%d", &single_screen_num) ==
+				    0)
+				{
+					usage(0);
+					exit(1);
+				}
+			}
 		}
 		else if (strcmp(argv[i], "-d") == 0 ||
 			 strcmp(argv[i], "-display") == 0 ||
@@ -1843,8 +1885,8 @@ int main(int argc, char **argv)
 			/* obsolete option */
 			fvwm_msg(
 				WARN, "main",
-				"The -blackout option is obsolete, it may be "
-				"removed in the future.");
+				"The -blackout option is obsolete, it will be "
+				"removed in 3.0.");
 		}
 		else if (strcmp(argv[i], "-r") == 0 ||
 			 strcmp(argv[i], "-replace") == 0 ||
@@ -1970,48 +2012,101 @@ int main(int argc, char **argv)
 	DBUG("main", "Installing signal handlers");
 	InstallSignals();
 
-	if (!(Pdpy = dpy = XOpenDisplay(display_name)))
+	if (single_screen_num >= 0)
 	{
-		fvwm_msg(
-			ERR, "main", "can't open display %s",
-			XDisplayName(display_name));
-		exit (1);
-	}
-	atexit(catch_exit);
-	Scr.screen= DefaultScreen(dpy);
-	Scr.NumberOfScreens = ScreenCount(dpy);
+		char *dn = NULL;
 
+		if (display_name)
+		{
+			dn = display_name;
+		}
+		if (!dn)
+		{
+			dn = getenv("DISPLAY");
+		}
+		if (!dn)
+		{
+			/* should never happen ? */
+			if (!(dpy = XOpenDisplay(dn)))
+			{
+				fvwm_msg(
+					ERR, "main", "can't open display %s"
+					"to get the default display",
+					XDisplayName(dn));
+			}
+			else
+			{
+				dn = XDisplayString(dpy);
+			}
+		}
+		if (dn == NULL)
+		{
+			fvwm_msg(
+				ERR, "main", "couldn't find default display (%s)",
+				XDisplayName(dn));
+		}
+		else
+		{
+			char *new_dn;
+
+			new_dn = get_display_name(dn, single_screen_num);
+			if (dpy && strcmp(new_dn, dn) == 0)
+			{
+				/* allready opened */
+				Pdpy = dpy;
+			}
+			else if (dpy)
+			{
+				XCloseDisplay(dpy);
+				dpy = NULL;
+			}
+			if (!dpy && !(Pdpy = dpy = XOpenDisplay(new_dn)))
+			{
+				fvwm_msg(
+					ERR, "main",
+					"can't open display %s, single screen "
+					"number %d maybe not correct",
+					new_dn, single_screen_num);
+			}
+			Scr.screen = single_screen_num;
+			Scr.NumberOfScreens = ScreenCount(dpy);
+			free(new_dn);
+		}
+	}
+
+	if (!dpy)
+	{
+		if(!(Pdpy = dpy = XOpenDisplay(display_name)))
+		{
+			fvwm_msg(
+				ERR, "main", "can't open display %s",
+				XDisplayName(display_name));
+			exit (1);
+		}
+		Scr.screen= DefaultScreen(dpy);
+		Scr.NumberOfScreens = ScreenCount(dpy);
+	}
+
+	atexit(catch_exit);
 	master_pid = getpid();
 
 	if (!do_force_single_screen)
 	{
 		int myscreen = 0;
-		char *cp;
+		char *new_dn;
+		char *dn;
 
-		strcpy(message, XDisplayString(dpy));
+		dn = XDisplayString(dpy);
 		for (i=0;i<Scr.NumberOfScreens;i++)
 		{
 			if (i != Scr.screen && fork() == 0)
 			{
 				myscreen = i;
-				/* Truncate the string 'whatever:n.n' to
-				 * 'whatever:n', and then append the screen
-				 * number. */
-				cp = strchr(message, ':');
-				if (cp != NULL)
-				{
-					cp = strchr(cp, '.');
-					if (cp != NULL)
-					{
-						/* truncate at display part */
-						*cp = '\0';
-					}
-				}
-				sprintf(message + strlen(message), ".%d",
-					myscreen);
-				Pdpy = dpy = XOpenDisplay(message);
+				new_dn = get_display_name(dn, myscreen);
+				Pdpy = dpy = XOpenDisplay(new_dn);
 				Scr.screen = myscreen;
 				Scr.NumberOfScreens = ScreenCount(dpy);
+				free(new_dn);
 
 				break;
 			}
@@ -2037,13 +2132,11 @@ int main(int argc, char **argv)
 	len = strlen(XDisplayString(dpy));
 	display_string = safemalloc(len+10);
 	sprintf(display_string, "DISPLAY=%s",XDisplayString(dpy));
-	putenv(display_string);
+	flib_putenv("DISPLAY", display_string);
 	/* Add a HOSTDISPLAY environment variable, which is the same as
 	 * DISPLAY, unless display = :0.0 or unix:0.0, in which case the full
 	 * host name will be used for ease in networking.
-	 *
-	 * Note: Can't free the rdisplay_string after putenv, because it
-	 * becomes part of the environment! */
+	 */
 	if (strncmp(display_string, "DISPLAY=:",9)==0)
 	{
 		char client[MAXHOSTNAME], *rdisplay_string;
@@ -2051,24 +2144,28 @@ int main(int argc, char **argv)
 		rdisplay_string = safemalloc(len+14 + strlen(client));
 		sprintf(rdisplay_string, "HOSTDISPLAY=%s:%s", client,
 			&display_string[9]);
-		putenv(rdisplay_string);
+		flib_putenv("HOSTDISPLAY", rdisplay_string);
+		free(rdisplay_string);
 	}
 	else if (strncmp(display_string, "DISPLAY=unix:",13)==0)
 	{
 		char client[MAXHOSTNAME], *rdisplay_string;
 		gethostname(client,MAXHOSTNAME);
 		rdisplay_string = safemalloc(len+14 + strlen(client));
-		sprintf(rdisplay_string, "HOSTDISPLAY=%s:%s",client,
+		sprintf(rdisplay_string, "HOSTDISPLAY=%s:%s", client,
 			&display_string[13]);
-		putenv(rdisplay_string);
+		flib_putenv("HOSTDISPLAY", rdisplay_string);
+		free(rdisplay_string);
 	}
 	else
 	{
 		char *rdisplay_string;
 		rdisplay_string = safemalloc(len+14);
 		sprintf(rdisplay_string, "HOSTDISPLAY=%s",XDisplayString(dpy));
-		putenv(rdisplay_string);
+		flib_putenv("HOSTDISPLAY", rdisplay_string);
+		free(rdisplay_string);
 	}
+	free(display_string);
 
 	Scr.Root = RootWindow(dpy, Scr.screen);
 	if (Scr.Root == None)
@@ -2168,20 +2265,15 @@ int main(int argc, char **argv)
 	/* use default visuals if none found so far */
 	if (visualClass == -1 && visualId == -1)
 	{
-		Colormap *cmap_list = NULL;
-		int num, i;
-
 		Pdepth = 0;
 
-		/* Detection of a card with 2 hardware colormaps (8+24) which use
-		 * depth 8 for the default. We can use our own depth 24 cmap
-		 * without affecting other applications. */
-		if (DefaultDepth(dpy, Scr.screen) <= 8 &&
-		    (cmap_list = XListInstalledColormaps(dpy, Scr.Root, &num))
-		    && num > 1)
+		/* Detection of a card with 2 hardware colormaps (8+24) which
+		 * use depth 8 for the default. We can use our own depth 24
+		 * cmap without affecting other applications. */
+		if (DefaultDepth(dpy, Scr.screen) <= 8)
 		{
-			XVisualInfo template, *vizinfo;
-			int total;
+			XVisualInfo template, *vizinfo = NULL;
+			int total,i;
 
 			template.screen = Scr.screen;
 			template.class = TrueColor;
@@ -2197,7 +2289,10 @@ int main(int argc, char **argv)
 					Pdepth = vizinfo[i].depth;
 				}
 			}
-			XFree(vizinfo);
+			if (vizinfo)
+			{
+				XFree(vizinfo);
+			}
 			if (Pdepth > 0)
 			{
 				Pcmap = XCreateColormap(
@@ -2205,10 +2300,6 @@ int main(int argc, char **argv)
 				Pdefault = False;
 			}
 		}
-		if (cmap_list)
-		{
-			XFree(cmap_list);
-		}
 		if (Pdepth == 0)
 		{
 			Pvisual = DefaultVisual(dpy, Scr.screen);
@@ -2268,13 +2359,21 @@ int main(int argc, char **argv)
 	}
 
 	SetupICCCM2(replace_wm);
-	XSetErrorHandler(CatchRedirectError);
 	XSetIOErrorHandler(CatchFatal);
-	XSelectInput(dpy, Scr.Root, XEVMASK_ROOTW);
-	XFlush(dpy);
-
-	XSetErrorHandler(FvwmErrorHandler);
-
+	{
+		/* We need to catch any errors of XSelectInput on the root
+		 * window here.  The event mask contains
+		 * SubstructureRedirectMask which can be acquired by exactly
+		 * one client (window manager).  Synchronizing is necessary
+		 * here because Neither XSetErrorHandler nor XSelectInput
+		 * generate any protocol requests.
+		 */
+		XSync(dpy, 0);
+		XSetErrorHandler(CatchRedirectError);
+		XSelectInput(dpy, Scr.Root, XEVMASK_ROOTW);
+		XSync(dpy, 0);
+		XSetErrorHandler(FvwmErrorHandler);
+	}
 	{
 		/* do not grab the pointer earlier because if fvwm exits with
 		 * the pointer grabbed while a different display is visible,
@@ -2336,13 +2435,13 @@ int main(int argc, char **argv)
 	simplify_style_list();
 
 	DBUG("main", "Running config_commands...");
-	ecc.type = EXCT_INIT;
+	ecc.type = Restarting ? EXCT_RESTART : EXCT_INIT;
 	ecc.w.wcontext = C_ROOT;
 	exc = exc_create_context(&ecc, ECC_TYPE | ECC_WCONTEXT);
 	if (num_config_commands > 0)
 	{
 		int i;
-		for (i=0;i<num_config_commands;i++)
+		for (i = 0; i < num_config_commands; i++)
 		{
 			DoingCommandLine = True;
 			execute_function(NULL, exc, config_commands[i], 0);
@@ -2352,26 +2451,44 @@ int main(int argc, char **argv)
 	}
 	else
 	{
-		/* Run startup command file in 5 places: FVWM_USERDIR,
-		 * FVWM_DATADIR, and for compatibility: ~/.fvwm2rc,
-		 * $sysconfdir/system.fvwm2rc */
-		if (!run_command_file(
-			    CatString3(fvwm_userdir, "/", FVWMRC), exc) &&
-		    !run_command_file(
-			    CatString3(home_dir, "/", FVWMRC), exc) &&
-		    !run_command_file(
-			    CatString3(FVWM_DATADIR, "/", FVWMRC), exc) &&
-		    !run_command_file(
-			    CatString3(FVWM_DATADIR, "/system", FVWMRC), exc) &&
-		    !run_command_file(
-			    CatString3(FVWM_CONFDIR, "/system", FVWMRC), exc))
+		/* Run startup command file in these places (default prefix):
+		 *   ~/.fvwm/config
+		 *   /usr/local/share/fvwm/config
+		 * and for compatibility:
+		 *   ~/.fvwm/.fvwm2rc
+		 *   /usr/local/share/fvwm/system.fvwm2rc
+		 * and for compatibility to be discontinued:
+		 *   ~/.fvwm2rc,
+		 *   /usr/local/share/fvwm/.fvwm2rc
+		 *   /usr/local/etc/system.fvwm2rc
+		 */
+		if (
+			!run_command_file(CatString3(
+				fvwm_userdir, "/", FVWM_CONFIG), exc) &&
+			!run_command_file(CatString3(
+				FVWM_DATADIR, "/", FVWM_CONFIG), exc) &&
+			!run_command_file(CatString3(
+				fvwm_userdir, "/", FVWM2RC), exc) &&
+			!run_command_file(CatString3(
+				home_dir, "/", FVWM2RC), exc) &&
+			!run_command_file(CatString3(
+				FVWM_DATADIR, "/", FVWM2RC), exc) &&
+			!run_command_file(CatString3(
+				FVWM_DATADIR, "/system", FVWM2RC), exc) &&
+			!run_command_file(CatString3(
+				FVWM_CONFDIR, "/system", FVWM2RC), exc))
 		{
 			fvwm_msg(
-				ERR, "main", "Cannot read startup file, tried:"
-				" \n\t%s/%s\n\t%s/%s\n\t%s/%s\n\t%s/system"
-				"%s\n\t%s/system%s", fvwm_userdir, FVWMRC,
-				home_dir, FVWMRC, FVWM_DATADIR, FVWMRC,
-				FVWM_DATADIR, FVWMRC, FVWM_CONFDIR, FVWMRC);
+				ERR, "main", "Cannot read startup config file,"
+				" tried: \n\t%s/%s\n\t%s/%s\n\t%s/%s\n\t"
+				"%s/%s\n\t%s/%s\n\t%s/system%s\n\t%s/system%s",
+				fvwm_userdir, FVWM_CONFIG,
+				FVWM_DATADIR, FVWM_CONFIG,
+				fvwm_userdir, FVWM2RC,
+				home_dir, FVWM2RC,
+				FVWM_DATADIR, FVWM2RC,
+				FVWM_DATADIR, FVWM2RC,
+				FVWM_CONFDIR, FVWM2RC);
 		}
 	}
 	exc_destroy_context(exc);
