@@ -52,9 +52,8 @@
 #include "events.h"
 #include "geometry.h"
 #include "fvwmsignal.h"
-
-/* Diasable old ConfigureWindow module interface */
-#define DISABLE_MBC
+#include "decorations.h"
+#include "commands.h"
 
 /*
  * Use POSIX behaviour if we can, otherwise use SysV instead
@@ -82,6 +81,8 @@ static msg_masks_t *SyncMask;
 static msg_masks_t *NoGrabMask;
 fqueue *pipeQueue;
 
+static const unsigned long dummy = 0;
+
 extern fd_set init_fdset;
 
 static void DeleteMessageQueueBuff(int module);
@@ -173,10 +174,15 @@ void ClosePipes(void)
 	int i;
 	for (i=0; i<npipes; ++i)
 	{
-		if (writePipes[i] > 0)
+		if (writePipes[i] >= 0)
 		{
 			close(writePipes[i]);
+			writePipes[i] = -1;
+		}
+		if (readPipes[i] >= 0)
+		{
 			close(readPipes[i]);
+			readPipes[i] = -1;
 		}
 		if (pipeName[i] != NULL)
 		{
@@ -197,7 +203,7 @@ void ClosePipes(void)
 	return;
 }
 
-static int do_execute_module(F_CMD_ARGS, Bool desperate)
+static int do_execute_module(F_CMD_ARGS, Bool desperate, Bool do_listen_only)
 {
 	int fvwm_to_app[2], app_to_fvwm[2];
 	int i, val, nargs = 0;
@@ -276,7 +282,7 @@ static int do_execute_module(F_CMD_ARGS, Bool desperate)
 
 	/* Look for an available pipe slot */
 	i = 0;
-	while ((i<npipes) && (writePipes[i] >=0))
+	while (i < npipes && (writePipes[i] >=0 || readPipes[i] >= 0))
 	{
 		i++;
 	}
@@ -293,7 +299,12 @@ static int do_execute_module(F_CMD_ARGS, Bool desperate)
 	/* I want one-ended pipes, so I open two two-ended pipes,
 	 * and close one end of each. I need one ended pipes so that
 	 * I can detect when the module crashes/malfunctions */
-	if (pipe(fvwm_to_app) != 0)
+	if (do_listen_only == True)
+	{
+		fvwm_to_app[0] = -1;
+		fvwm_to_app[1] = -1;
+	}
+	else if (pipe(fvwm_to_app) != 0)
 	{
 		fvwm_msg(ERR, "executeModule", "Failed to open pipe");
 		free(arg1);
@@ -306,6 +317,7 @@ static int do_execute_module(F_CMD_ARGS, Bool desperate)
 		fvwm_msg(ERR, "executeModule", "Failed to open pipe2");
 		free(arg1);
 		free(cptr);
+		/* dont't care that these may be -1 */
 		close(fvwm_to_app[0]);
 		close(fvwm_to_app[1]);
 		free(args);
@@ -354,6 +366,7 @@ static int do_execute_module(F_CMD_ARGS, Bool desperate)
 		/* close appropriate descriptors from each pipe so
 		 * that fvwm will be able to tell when the app dies */
 		close(app_to_fvwm[1]);
+		/* dont't care that this may be -1 */
 		close(fvwm_to_app[0]);
 
 		/* add these pipes to fvwm's active pipe list */
@@ -377,7 +390,10 @@ static int do_execute_module(F_CMD_ARGS, Bool desperate)
 
 		/* make the PositiveWrite pipe non-blocking. Don't want to jam
 		 * up fvwm because of an uncooperative module */
-		fcntl(writePipes[i], F_SETFL, O_NONBLOCK);
+		if (writePipes[i] >= 0)
+		{
+			fcntl(writePipes[i], F_SETFL, O_NONBLOCK);
+		}
 		/* Mark the pipes close-on exec so other programs
 		 * won`t inherit them */
 		if (fcntl(readPipes[i], F_SETFD, 1) == -1)
@@ -386,7 +402,9 @@ static int do_execute_module(F_CMD_ARGS, Bool desperate)
 				ERR, "executeModule",
 				"module close-on-exec failed");
 		}
-		if (fcntl(writePipes[i], F_SETFD, 1) == -1)
+		if (
+			writePipes[i] >= 0 &&
+			fcntl(writePipes[i], F_SETFD, 1) == -1)
 		{
 			fvwm_msg(
 				ERR, "executeModule",
@@ -405,6 +423,7 @@ static int do_execute_module(F_CMD_ARGS, Bool desperate)
 		/* this is the child */
 		/* this fork execs the module */
 #ifdef FORK_CREATES_CHILD
+		/* dont't care that this may be -1 */
 		close(fvwm_to_app[1]);
 		close(app_to_fvwm[0]);
 #endif
@@ -420,6 +439,11 @@ static int do_execute_module(F_CMD_ARGS, Bool desperate)
 			sprintf(colormap, "FVWM_COLORMAP=%lx", Pcmap);
 			flib_putenv("FVWM_COLORMAP", colormap);
 		}
+		else
+		{
+			flib_unsetenv("FVWM_VISUALID");
+			flib_unsetenv("FVWM_COLORMAP");
+		}
 
 		/* Why is this execvp??  We've already searched the module
 		 * path! */
@@ -429,6 +453,7 @@ static int do_execute_module(F_CMD_ARGS, Bool desperate)
 			arg1);
 		perror("");
 		close(app_to_fvwm[1]);
+		/* dont't care that this may be -1 */
 		close(fvwm_to_app[0]);
 #ifdef FORK_CREATES_CHILD
 		exit(1);
@@ -455,12 +480,20 @@ static int do_execute_module(F_CMD_ARGS, Bool desperate)
 
 int executeModuleDesperate(F_CMD_ARGS)
 {
-	return do_execute_module(F_PASS_ARGS, True);
+	return do_execute_module(F_PASS_ARGS, True, False);
 }
 
 void CMD_Module(F_CMD_ARGS)
 {
-	do_execute_module(F_PASS_ARGS, False);
+	do_execute_module(F_PASS_ARGS, False, False);
+
+	return;
+}
+
+void CMD_ModuleListenOnly(F_CMD_ARGS)
+{
+	do_execute_module(F_PASS_ARGS, False, True);
+
 	return;
 }
 
@@ -518,7 +551,7 @@ void CMD_ModuleSynchronous(F_CMD_ARGS)
 		return;
 	}
 
-	pipe_slot = do_execute_module(F_PASS_ARGS, False);
+	pipe_slot = do_execute_module(F_PASS_ARGS, False, False);
 	if (pipe_slot == -1)
 	{
 		/* executing the module failed, just return */
@@ -558,7 +591,7 @@ void CMD_ModuleSynchronous(F_CMD_ARGS)
 			}
 
 			timeout.tv_sec = 0;
-			timeout.tv_usec = 1;  /* use 1 usec timeout in select */
+			timeout.tv_usec = 1;
 			num_fd = fvwmSelect(
 				fd_width, &in_fdset, &out_fdset, 0, &timeout);
 		} while (num_fd < 0 && !isTerminated);
@@ -612,14 +645,27 @@ void CMD_ModuleSynchronous(F_CMD_ARGS)
 		if (FPending(dpy) &&
 		    FCheckMaskEvent(dpy, KeyPressMask, &tmpevent))
 		{
+			int context;
+			XClassHint *class;
+			char *name;
+
+			context = GetContext(
+				NULL, exc->w.fw, &tmpevent, &targetWindow);
+			if (exc->w.fw != NULL)
+			{
+				class = &(exc->w.fw->class);
+				name = exc->w.fw->name.name;
+			}
+			else
+			{
+				class = NULL;
+				name = NULL;
+			}
 			escape = CheckBinding(
 				Scr.AllBindings, STROKE_ARG(0)
 				tmpevent.xkey.keycode, tmpevent.xkey.state,
-				GetUnusedModifiers(),
-				GetContext(
-					NULL, exc->w.fw, &tmpevent,
-					&targetWindow),
-				BIND_KEYPRESS);
+				GetUnusedModifiers(), context, BIND_KEYPRESS,
+				class, name);
 			if (escape != NULL)
 			{
 				if (!strcasecmp(escape,"escapefunc"))
@@ -646,6 +692,7 @@ static void ExecuteModuleCommand(Window w, int module, char *text)
 	exec_context_changes_t ecc;
 	int flags;
 
+	memset(&e, 0, sizeof(e));
 	if (XFindContext(dpy, w, FvwmContext, (caddr_t *)&ecc.w.fw) == XCNOENT)
 	{
 		ecc.w.fw = NULL;
@@ -657,7 +704,7 @@ static void ExecuteModuleCommand(Window w, int module, char *text)
 	 */
 	if (FQueryPointer(
 		    dpy, Scr.Root, &JunkRoot, &JunkChild, &JunkX,&JunkY,
-		    &e.xbutton.x_root, &e.xbutton.y_root, &JunkMask) ==
+		    &e.xbutton.x_root, &e.xbutton.y_root, &e.xbutton.state) ==
 	    False)
 	{
 		/* pointer is not on this screen */
@@ -676,10 +723,12 @@ static void ExecuteModuleCommand(Window w, int module, char *text)
 	if (StrEquals(text, "popup"))
 	{
 		e.xbutton.type = ButtonPress;
+		e.xbutton.state |= Button1Mask;
 	}
 	else
 	{
 		e.xbutton.type = ButtonRelease;
+		e.xbutton.state &= (~(Button1Mask));
 	}
 	e.xbutton.x = 0;
 	e.xbutton.y = 0;
@@ -717,7 +766,7 @@ Bool HandleModuleInput(Window w, int module, char *expect, Bool queue)
 		fvwm_msg(
 			ERR, "HandleModuleInput",
 			"Fail to read (Module: %i, read: %i, size: %i)",
-			module, n, sizeof(size));
+			module, n, (int)sizeof(size));
 		KillModule(module);
 		return False;
 	}
@@ -726,7 +775,7 @@ Bool HandleModuleInput(Window w, int module, char *expect, Bool queue)
 	{
 		fvwm_msg(ERR, "HandleModuleInput",
 			 "Module(%i) command is too big (%ld), limit is %d",
-			 module, size, sizeof(text));
+			 module, size, (int)sizeof(text));
 		/* The rest of the output from this module is going to be
 		 * scrambled so let's kill it rather than risk interpreting
 		 * garbage */
@@ -752,7 +801,7 @@ Bool HandleModuleInput(Window w, int module, char *expect, Bool queue)
 	{
 		fvwm_msg(ERR, "HandleModuleInput",
 			 "Module %i, Size Problems (read: %d, size: %d)",
-			 module, n, sizeof(cont));
+			 module, n, (int)sizeof(cont));
 		KillModule(module);
 		return False;
 	}
@@ -790,11 +839,16 @@ RETSIGTYPE DeadPipe(int sig)
 
 void KillModule(int channel)
 {
-	close(readPipes[channel]);
-	close(writePipes[channel]);
-
-	readPipes[channel] = -1;
-	writePipes[channel] = -1;
+	if (readPipes[channel] >= 0)
+	{
+		close(readPipes[channel]);
+		readPipes[channel] = -1;
+	}
+	if (writePipes[channel] >= 0)
+	{
+		close(writePipes[channel]);
+		writePipes[channel] = -1;
+	}
 	pipeOn[channel] = -1;
 	while (!FQUEUE_IS_EMPTY(&pipeQueue[channel]))
 	{
@@ -895,7 +949,7 @@ make_vpacket(unsigned long *body, unsigned long event_type,
     RBW - 05/01/2000 -
     A length of zero means that an int is being passed which
     must be stored in the packet as an unsigned long. This is
-    a special hack to accommodate the old CONFIGARGSNEW
+    a special hack to accommodate the old CONFIGARGS
     technique of sending the args for the M_CONFIGURE_WINDOW
     packet.
 */
@@ -903,13 +957,12 @@ static unsigned long
 make_new_vpacket(unsigned char *body, unsigned long event_type,
 		 unsigned long num, va_list ap)
 {
-	unsigned long arglen;
+	long arglen;
+	unsigned long addlen;
 	unsigned long bodylen = 0;
 	unsigned long *bp = (unsigned long *)body;
 	unsigned long *bp1 = bp;
 	unsigned long plen = 0;
-	int *tmpint;
-	Bool expandint;
 
 	*(bp++) = START_FLAG;
 	*(bp++) = event_type;
@@ -919,32 +972,66 @@ make_new_vpacket(unsigned char *body, unsigned long event_type,
 
 	for (; num > 0; --num)
 	{
-		arglen = va_arg(ap, unsigned long);
-		if (arglen == 0)
+		arglen = va_arg(ap, long);
+		if (arglen <= 0)
 		{
-			expandint = True;
-			arglen = sizeof(unsigned long);
+			if (arglen == 0)
+			{
+				arglen = -sizeof(int);
+			}
+			addlen = sizeof(unsigned long);
 		}
 		else
 		{
-			expandint = False;
+			addlen = arglen;
 		}
-		bodylen += arglen;
-		if (bodylen < FvwmPacketMaxSize_byte)
+		bodylen += addlen;
+		if (bodylen >= FvwmPacketMaxSize_byte)
 		{
-			if (!expandint)
-			{
-				register char *tmp = (char *)bp;
-				memcpy(tmp, va_arg(ap, char *), arglen);
-				tmp += arglen;
-				bp = (unsigned long *)tmp;
-			}
-			else
-			{
-				tmpint = va_arg(ap, int *);
-				*bp = (unsigned long) *tmpint;
-				bp++;
-			}
+			fvwm_msg(
+				ERR, "make_new_vpacket",
+				"packet too long %ld %ld", (long)bodylen,
+				(long)FvwmPacketMaxSize_byte);
+			break;
+		}
+		if (arglen > 0)
+		{
+			register char *tmp = (char *)bp;
+			memcpy(tmp, va_arg(ap, char *), arglen);
+			tmp += arglen;
+			bp = (unsigned long *)tmp;
+		}
+		else if (arglen == 0 || arglen == -sizeof(int))
+		{
+			int *tmp;
+
+			tmp = va_arg(ap, int *);
+			*bp = (unsigned long) *tmp;
+			bp++;
+		}
+		else if (arglen == -sizeof(long))
+		{
+			unsigned long *tmp;
+
+			tmp = va_arg(ap, unsigned long *);
+			*bp = (unsigned long) *tmp;
+			bp++;
+		}
+		else if (arglen == -sizeof(short))
+		{
+			short *tmp;
+
+			tmp = va_arg(ap, short *);
+			*bp = (unsigned long) *tmp;
+			bp++;
+		}
+		else
+		{
+			fvwm_msg(
+				ERR, "make_new_vpacket",
+				"can not handle arglen %ld, please contact"
+				" fvwm-workers@fvwm.org. aborting...", arglen);
+			abort();
 		}
 	}
 
@@ -970,9 +1057,9 @@ SendPacket(int module, unsigned long event_type, unsigned long num_datum, ...)
 	va_start(ap, num_datum);
 	make_vpacket(body, event_type, num_datum, ap);
 	va_end(ap);
-
 	PositiveWrite(
-		module, body, (num_datum+FvwmPacketHeaderSize)*sizeof(body[0]));
+		module, body,
+		(num_datum+FvwmPacketHeaderSize)*sizeof(body[0]));
 
 	return;
 }
@@ -987,7 +1074,6 @@ BroadcastPacket(unsigned long event_type, unsigned long num_datum, ...)
 	va_start(ap,num_datum);
 	make_vpacket(body, event_type, num_datum, ap);
 	va_end(ap);
-
 	for (i=0; i<npipes; i++)
 	{
 		PositiveWrite(
@@ -1012,7 +1098,6 @@ static void SendNewPacket(int module, unsigned long event_type,
 	va_start(ap,num_datum);
 	plen = make_new_vpacket(body, event_type, num_datum, ap);
 	va_end(ap);
-
 	PositiveWrite(module, (void *) &body, plen);
 
 	return;
@@ -1029,7 +1114,6 @@ static void BroadcastNewPacket(unsigned long event_type,
 	va_start(ap,num_datum);
 	plen = make_new_vpacket(body, event_type, num_datum, ap);
 	va_end(ap);
-
 	for (i=0; i<npipes; i++)
 	{
 		PositiveWrite(i, (void *) &body, plen);
@@ -1038,106 +1122,24 @@ static void BroadcastNewPacket(unsigned long event_type,
 	return;
 }
 
+action_flags *__get_allowed_actions(const FvwmWindow *fw)
+{
+	static action_flags act;
+	act.is_movable = is_function_allowed(F_MOVE, NULL, fw, True, False);
+	act.is_deletable = is_function_allowed(F_DELETE, NULL, fw, True,
+					       False);
+	act.is_destroyable = is_function_allowed(F_DESTROY, NULL, fw, True,
+						 False);
+	act.is_closable = is_function_allowed(F_CLOSE, NULL, fw, True, False);
+	act.is_maximizable = is_function_allowed(F_MAXIMIZE, NULL, fw, True,
+						 False);
+	act.is_resizable = is_function_allowed(F_RESIZE, NULL, fw, True,
+					       False);
+	act.is_iconifiable = is_function_allowed(F_ICONIFY, NULL, fw, True,
+						 False);
 
-/* this is broken, the flags may not fit in a word */
-#define CONFIGARGS(_t) 27,\
-	    FW_W(_t),\
-	    FW_W_FRAME(_t),\
-	    (unsigned long)(_t),\
-	    (_t)->frame_g.x,\
-	    (_t)->frame_g.y,\
-	    (_t)->frame_g.width,\
-	    (_t)->frame_g.height,\
-	    (_t)->Desk,\
-	    (_t)->flags,\
-	    (_t)->title_thickness,\
-	    (_t)->boundary_width,\
-	    (_t)->hints.base_width,\
-	    (_t)->hints.base_height,\
-	    (_t)->hints.width_inc,\
-	    (_t)->hints.height_inc,\
-	    (_t)->hints.min_width,\
-	    (_t)->hints.min_height,\
-	    (_t)->hints.max_width,\
-	    (_t)->hints.max_height,\
-	    FW_W_ICON_TITLE(_t),\
-	    FW_W_ICON_PIXMAP(_t),\
-	    (_t)->hints.win_gravity,\
-	    (_t)->colors.fore,\
-	    (_t)->colors.back,\
-	    (_t)->ewmh_hint_layer,\
-	    (_t)->ewmh_hint_desktop,\
-	    (_t)->ewmh_window_type
-
-#ifndef DISABLE_MBC
-#define OLDCONFIGARGS(_t) 27,\
-	    FW_W(_t),\
-	    FW_W_FRAME(_t),\
-	    (unsigned long)(_t),\
-	    (_t)->frame_g.x,\
-	    (_t)->frame_g.y,\
-	    (_t)->frame_g.width,\
-	    (_t)->frame_g.height,\
-	    (_t)->Desk,\
-	    old_flags,\
-	    (_t)->title_thickness,\
-	    (_t)->boundary_width,\
-	    (_t)->hints.base_width,\
-	    (_t)->hints.base_height,\
-	    (_t)->hints.width_inc,\
-	    (_t)->hints.height_inc,\
-	    (_t)->hints.min_width,\
-	    (_t)->hints.min_height,\
-	    (_t)->hints.max_width,\
-	    (_t)->hints.max_height,\
-	    FW_W_ICON_TITLE(_t),\
-	    FW_W_ICON_PIXMAP(_t),\
-	    (_t)->hints.win_gravity,\
-	    (_t)->colors.fore,\
-	    (_t)->colors.back,\
-	    (_t)->ewmh_hint_layer,\
-	    (_t)->ewmh_hint_desktop,\
-	    (_t)->ewmh_window_type
-
-#define SETOLDFLAGS \
-{ \
-	int i = 1; \
-	old_flags |= 0 /* was start_iconic */         ? i : 0; i<<=1; \
-	old_flags |= False /* OnTop */                ? i : 0; i<<=1; \
-	old_flags |= IS_STICKY(t)                     ? i : 0; i<<=1; \
-	old_flags |= DO_SKIP_WINDOW_LIST(t)           ? i : 0; i<<=1; \
-	old_flags |= IS_ICON_SUPPRESSED(t)            ? i : 0; i<<=1; \
-	old_flags |= HAS_NO_ICON_TITLE(t)             ? i : 0; i<<=1; \
-	old_flags |= FP_IS_LENIENT(FW_FOCUS_POLICY(t)) ? i : 0; i<<=1; \
-	old_flags |= IS_ICON_STICKY(t)                ? i : 0; i<<=1; \
-	old_flags |= DO_SKIP_ICON_CIRCULATE(t)        ? i : 0; i<<=1; \
-	old_flags |= DO_SKIP_CIRCULATE(t)             ? i : 0; i<<=1; \
-	old_flags |= HASx_CLICK_FOCUS(t)               ? i : 0; i<<=1; \
-	old_flags |= HASx_SLOPPY_FOCUS(t)              ? i : 0; i<<=1; \
-	old_flags |= !DO_NOT_SHOW_ON_MAP(t)           ? i : 0; i<<=1; \
-	old_flags |= !HAS_NO_BORDER(t)                ? i : 0; i<<=1; \
-	old_flags |= HAS_TITLE(t)                     ? i : 0; i<<=1; \
-	old_flags |= IS_MAPPED(t)                     ? i : 0; i<<=1; \
-	old_flags |= IS_ICONIFIED(t)                  ? i : 0; i<<=1; \
-	old_flags |= IS_TRANSIENT(t)                  ? i : 0; i<<=1; \
-	old_flags |= False /* Raised */               ? i : 0; i<<=1; \
-	old_flags |= IS_FULLY_VISIBLE(t)              ? i : 0; i<<=1; \
-	old_flags |= IS_ICON_OURS(t)                  ? i : 0; i<<=1; \
-	old_flags |= IS_PIXMAP_OURS(t)                ? i : 0; i<<=1; \
-	old_flags |= IS_ICON_SHAPED(t)                ? i : 0; i<<=1; \
-	old_flags |= IS_MAXIMIZED(t)                  ? i : 0; i<<=1; \
-	old_flags |= WM_TAKES_FOCUS(t)                ? i : 0; i<<=1; \
-	old_flags |= WM_DELETES_WINDOW(t)             ? i : 0; i<<=1; \
-	old_flags |= IS_ICON_MOVED(t)                 ? i : 0; i<<=1; \
-	old_flags |= IS_ICON_UNMAPPED(t)              ? i : 0; i<<=1; \
-	old_flags |= IS_MAP_PENDING(t)                ? i : 0; i<<=1; \
-	old_flags |= HAS_MWM_OVERRIDE_HINTS(t)        ? i : 0; i<<=1; \
-	old_flags |= HAS_MWM_BUTTONS(t)               ? i : 0; i<<=1; \
-	old_flags |= HAS_MWM_BORDER(t)                ? i : 0; \
+	return &act;
 }
-#endif /* DISABLE_MBC */
-
-
 
 /*
     RBW - 04/16/1999 - new version for GSFR --
@@ -1149,113 +1151,93 @@ static void BroadcastNewPacket(unsigned long event_type,
 	as a dummy to preserve alignment of the other fields in the
 	old packet: we should drop this before the next release.
 */
-#define CONFIGARGSNEW(_t) 28,\
-	    (unsigned long)(sizeof(unsigned long)),\
-	    &FW_W(*(_t)),\
-	    (unsigned long)(sizeof(unsigned long)),\
-	    &FW_W_FRAME(*(_t)),\
-	    (unsigned long)(sizeof(unsigned long)),\
-	    &(_t),\
-	    (unsigned long)(0),\
-	    &(*(_t))->frame_g.x,\
-	    (unsigned long)(0),\
-	    &(*(_t))->frame_g.y,\
-	    (unsigned long)(0),\
-	    &(*(_t))->frame_g.width,\
-	    (unsigned long)(0),\
-	    &(*(_t))->frame_g.height,\
-	    (unsigned long)(0),\
-	    &(*(_t))->Desk,\
-	    (unsigned long)(0),\
-	    &(*(_t))->layer,\
-	    (unsigned long)(sizeof(short)),\
-	    &(*(_t))->title_thickness,\
-	    (unsigned long)(sizeof(short)),\
-	    &(*(_t))->boundary_width,\
-	    (unsigned long)(0),\
-	    &(*(_t))->hints.base_width,\
-	    (unsigned long)(0),\
-	    &(*(_t))->hints.base_height,\
-	    (unsigned long)(0),\
-	    &(*(_t))->hints.width_inc,\
-	    (unsigned long)(0),\
-	    &(*(_t))->hints.height_inc,\
-	    (unsigned long)(0),\
-	    &(*(_t))->hints.min_width,\
-	    (unsigned long)(0),\
-	    &(*(_t))->hints.min_height,\
-	    (unsigned long)(0),\
-	    &(*(_t))->hints.max_width,\
-	    (unsigned long)(0),\
-	    &(*(_t))->hints.max_height,\
-	    (unsigned long)(0),\
-	    &FW_W_ICON_TITLE(*(_t)),\
-	    (unsigned long)(sizeof(unsigned long)),\
-	    &FW_W_ICON_PIXMAP(*(_t)),\
-	    (unsigned long)(0),\
-	    &(*(_t))->hints.win_gravity,\
-	    (unsigned long)(sizeof(unsigned long)),\
-	    &(*(_t))->colors.fore,\
-	    (unsigned long)(sizeof(unsigned long)),\
-	    &(*(_t))->colors.back,\
-	    (unsigned long)(0),\
-	    &(*(_t))->ewmh_hint_layer,\
-	    (unsigned long)(sizeof(unsigned long)),\
-	    &(*(_t))->ewmh_hint_desktop,\
-	    (unsigned long)(0),\
-	    &(*(_t))->ewmh_window_type,\
-	    (unsigned long)(sizeof((*(_t))->flags)),\
-	    &(*(_t))->flags
-
-
+#define CONFIGARGS(_fw) 31,\
+		(unsigned long)(-sizeof(Window)),	\
+		&FW_W(*(_fw)),				\
+		(unsigned long)(-sizeof(Window)),	\
+		&FW_W_FRAME(*(_fw)),			\
+		(unsigned long)(-sizeof(void *)),	\
+		&(_fw),					\
+		(unsigned long)(0),			\
+		&(*(_fw))->frame_g.x,			\
+		(unsigned long)(0),			\
+		&(*(_fw))->frame_g.y,			\
+		(unsigned long)(0),			\
+		&(*(_fw))->frame_g.width,		\
+		(unsigned long)(0),			\
+		&(*(_fw))->frame_g.height,		\
+		(unsigned long)(0),			\
+		&(*(_fw))->Desk,			\
+		(unsigned long)(0),			\
+		&(*(_fw))->layer,			\
+		(unsigned long)(0),			\
+		&(*(_fw))->hints.base_width,		\
+		(unsigned long)(0),			\
+		&(*(_fw))->hints.base_height,		\
+		(unsigned long)(0),			\
+		&(*(_fw))->hints.width_inc,		\
+		(unsigned long)(0),			\
+		&(*(_fw))->hints.height_inc,		\
+		(unsigned long)(0),			\
+		&(*(_fw))->hints.min_width,		\
+		(unsigned long)(0),			\
+		&(*(_fw))->hints.min_height,		\
+		(unsigned long)(0),			\
+		&(*(_fw))->hints.max_width,		\
+		(unsigned long)(0),			\
+		&(*(_fw))->hints.max_height,		\
+		(unsigned long)(-sizeof(Window)),	\
+		&FW_W_ICON_TITLE(*(_fw)),		\
+		(unsigned long)(-sizeof(Window)),	\
+		&FW_W_ICON_PIXMAP(*(_fw)),		\
+		(unsigned long)(0),			\
+		&(*(_fw))->hints.win_gravity,		\
+		(unsigned long)(-sizeof(Pixel)),	\
+		&(*(_fw))->colors.fore,			\
+		(unsigned long)(-sizeof(Pixel)),	\
+		&(*(_fw))->colors.back,			\
+		(unsigned long)(0),			\
+		&(*(_fw))->ewmh_hint_layer,		\
+		(unsigned long)(sizeof(unsigned long)),	\
+		&(*(_fw))->ewmh_hint_desktop,		\
+		(unsigned long)(0),			\
+		&(*(_fw))->ewmh_window_type,		\
+		(unsigned long)(sizeof(short)),		\
+		&(*(_fw))->title_thickness,		\
+		(unsigned long)(sizeof(short)),		\
+		&(*(_fw))->boundary_width,		\
+		(unsigned long)(sizeof(short)),		\
+		&dummy,					\
+		(unsigned long)(sizeof(short)),		\
+		&dummy,					\
+		(unsigned long)(sizeof((*(_fw))->flags)),	\
+		&(*(_fw))->flags,                       \
+		(unsigned long)(sizeof(action_flags)),  \
+		__get_allowed_actions((*(_fw)))
 
 void SendConfig(int module, unsigned long event_type, const FvwmWindow *t)
 {
 	const FvwmWindow **t1 = &t;
 
 	/*  RBW-  SendPacket(module, event_type, CONFIGARGS(t)); */
-	SendNewPacket(module, event_type, CONFIGARGSNEW(t1));
-#ifndef DISABLE_MBC
-	/* send out an old version of the packet to keep old mouldules happy */
-	/* fixme: should test to see if module wants this packet first */
-	{
-		long old_flags = 0;
-		SETOLDFLAGS
-			SendPacket(module, event_type == M_ADD_WINDOW ?
-				   M_OLD_ADD_WINDOW : M_OLD_CONFIGURE_WINDOW,
-				   OLDCONFIGARGS(t));
-	}
-#endif /* DISABLE_MBC */
+	SendNewPacket(module, event_type, CONFIGARGS(t1));
 
 	return;
 }
 
-
 void BroadcastConfig(unsigned long event_type, const FvwmWindow *t)
 {
 	const FvwmWindow **t1 = &t;
 
 	/*  RBW-  BroadcastPacket(event_type, CONFIGARGS(t)); */
-	BroadcastNewPacket(event_type, CONFIGARGSNEW(t1));
-#ifndef DISABLE_MBC
-	/* send out an old version of the packet to keep old mouldules happy */
-	{
-		long old_flags = 0;
-		SETOLDFLAGS
-			BroadcastPacket(
-				event_type == M_ADD_WINDOW ?
-				M_OLD_ADD_WINDOW : M_OLD_CONFIGURE_WINDOW,
-				OLDCONFIGARGS(t));
-	}
-#endif /* DISABLE_MBC */
+	BroadcastNewPacket(event_type, CONFIGARGS(t1));
 
 	return;
 }
 
-
 static unsigned long *
-make_named_packet(int *len, unsigned long event_type, const char *name,
-		  int num, ...)
+make_named_packet(
+	int *len, unsigned long event_type, const char *name, int num, ...)
 {
 	unsigned long *body;
 	va_list ap;
@@ -1282,12 +1264,6 @@ make_named_packet(int *len, unsigned long event_type, const char *name,
 		(*len - FvwmPacketHeaderSize - num)*sizeof(unsigned long) - 1);
 	body[2] = *len;
 
-#if 0
-	DB(("Packet (%lu): %lu %lu %lu `%s'", *len,
-	    body[FvwmPacketHeaderSize], body[FvwmPacketHeaderSize+1],
-	    body[FvwmPacketHeaderSize+2], name));
-#endif
-
 	return (body);
 }
 
@@ -1303,7 +1279,6 @@ SendName(int module, unsigned long event_type,
 	{
 		return;
 	}
-
 	body = make_named_packet(&l, event_type, name, 3, data1, data2, data3);
 	PositiveWrite(module, body, l*sizeof(unsigned long));
 	free(body);
@@ -1328,7 +1303,6 @@ BroadcastName(unsigned long event_type,
 	{
 		PositiveWrite(i, body, l*sizeof(unsigned long));
 	}
-
 	free(body);
 
 	return;
@@ -1366,7 +1340,7 @@ SendFvwmPicture(
 	char *name)
 {
 	unsigned long *body;
-	unsigned int
+	unsigned long
 		data4 = 0, data5 = 0, data6 = 0,
 		data7 = 0, data8 = 0, data9 = 0;
 	int l;
@@ -1392,7 +1366,6 @@ SendFvwmPicture(
 	body = make_named_packet(
 		&l, event_type, name, 9, data1, data2, data3, data4, data5,
 		data6, data7, data8, data9);
-
 	PositiveWrite(module, body, l*sizeof(unsigned long));
 	free(body);
 
@@ -1405,7 +1378,7 @@ BroadcastFvwmPicture(
 	unsigned long data3, FvwmPicture *picture, char *name)
 {
 	unsigned long *body;
-	unsigned int data4, data5, data6, data7, data8, data9;
+	unsigned long data4, data5, data6, data7, data8, data9;
 	int i, l;
 
 	if (!FMiniIconsSupported)
@@ -1433,12 +1406,10 @@ BroadcastFvwmPicture(
 	body = make_named_packet(
 		&l, event_type, name, 9, data1, data2, data3, data4, data5,
 		data6, data7, data8, data9);
-
 	for (i=0; i < npipes; i++)
 	{
 		PositiveWrite(i, body, l*sizeof(unsigned long));
 	}
-
 	free(body);
 
 	return;
@@ -1583,6 +1554,44 @@ void CMD_SendToModule(F_CMD_ARGS)
 	return;
 }
 
+/*
+** send an arbitrary string back to the calling module
+*/
+void CMD_Send_Reply(F_CMD_ARGS)
+{
+	unsigned long data0, data1, data2;
+	int module = exc->m.modnum;
+	FvwmWindow * const fw = exc->w.fw;
+
+	if (module < 0)
+	{
+		return;
+	}
+
+	if (!action)
+	{
+		return;
+	}
+
+	if (fw)
+	{
+		/* Modules may need to know which window this applies to */
+		data0 = FW_W(fw);
+		data1 = FW_W_FRAME(fw);
+		data2 = (unsigned long)fw;
+	}
+	else
+	{
+		data0 = 0;
+		data1 = 0;
+		data2 = 0;
+	}
+	SendName(module, MX_REPLY, data0, data1, data2, action);
+	FlushMessageQueue(module);
+
+	return;
+}
+
 /* This used to be marked "fvwm_inline".  I removed this
    when I added the lockonsend logic.  The routine seems too big to
    want to inline.  dje 9/4/98 */
@@ -1601,6 +1610,10 @@ void PositiveWrite(int module, unsigned long *ptr, int size)
 	{
 		return;
 	}
+	if (writePipes[module] == -1)
+	{
+		return;
+	}
 	if (!IS_MESSAGE_IN_MASK(&PipeMask[module], ptr[1]))
 	{
 		return;
@@ -1633,8 +1646,7 @@ void PositiveWrite(int module, unsigned long *ptr, int size)
 		c->done = 0;
 		c->data = (unsigned long *)(c + 1);
 		memcpy((void*)c->data, (const void*)ptr, size);
-
-		fqueue_add_at_end(&pipeQueue[module], c);
+			fqueue_add_at_end(&pipeQueue[module], c);
 	}
 
 	/* dje, from afterstep, for FvwmAnimate, allows modules to sync with
@@ -1642,10 +1654,11 @@ void PositiveWrite(int module, unsigned long *ptr, int size)
 	 * deadlocks happen. M_LOCKONSEND has been replaced by a separated
 	 * mask which defines on which messages the fvwm-to-module
 	 * communication need to be lock on send. olicha Nov 13, 1999 */
-	/* migo (19-Aug-2000): removed !myxgrabcount to sync M_DESTROY_WINDOW */
+	/* migo (19-Aug-2000): removed !myxgrabcount to sync M_DESTROY_WINDOW
+	 */
 	/* dv (06-Jul-2002): added the !myxgrabcount again.  Deadlocks *do*
-	 * happen without it.  There must be another way to fix M_DESTROY_WINDOW
-	 * handling in FvwmEvent. */
+	 * happen without it.  There must be another way to fix
+	 * M_DESTROY_WINDOW handling in FvwmEvent. */
 	/*if (IS_MESSAGE_IN_MASK(&SyncMask[module], ptr[1]))*/
 	if (IS_MESSAGE_IN_MASK(&SyncMask[module], ptr[1]) && !myxgrabcount)
 	{
@@ -1670,7 +1683,8 @@ void PositiveWrite(int module, unsigned long *ptr, int size)
 			 * to be KILLED!
 			 *
 			 * NOTE: rather than impose an arbitrary timeout on the
-			 * user, we will make this a configuration parameter. */
+			 * user, we will make this a configuration parameter.
+			 */
 			do
 			{
 				timeout.tv_sec = moduleTimeout;
@@ -1680,12 +1694,13 @@ void PositiveWrite(int module, unsigned long *ptr, int size)
 
 				/* Wait for input to arrive on just one
 				 * descriptor, with a timeout (fvwmSelect <= 0)
-				 * or read() returning wrong size is bad news */
+				 * or read() returning wrong size is bad news
+				 */
 				rc = fvwmSelect(
 					channel + 1, &readSet, NULL, NULL,
 					&timeout);
 				/* retry if select() failed with EINTR */
-			} while ((rc < 0) && !isTerminated && (errno == EINTR));
+			} while (rc < 0 && !isTerminated && (errno == EINTR));
 
 			if ( isTerminated )
 			{
@@ -1700,7 +1715,7 @@ void PositiveWrite(int module, unsigned long *ptr, int size)
 
 				name = get_pipe_name(module);
 				/* Doh! Something has gone wrong - get rid of
-				 * the offender!! */
+				 * the offender! */
 				fvwm_msg(ERR, "PositiveWrite",
 					 "Failed to read descriptor from"
 					 " '%s':\n"
@@ -1731,9 +1746,10 @@ void PositiveWrite(int module, unsigned long *ptr, int size)
 			 * when a window is de-iconified from the IconMan,
 			 * queueing make s this happen too late. */
 		}
-		while (!HandleModuleInput(
-			       targetWindow, module, ModuleUnlockResponse,
-			       False));
+		while (
+			!HandleModuleInput(
+				targetWindow, module, ModuleUnlockResponse,
+				False));
 	}
 
 	return;
@@ -1763,7 +1779,6 @@ void FlushMessageQueue(int module)
 	mqueue_object_type *obj;
 	char *dptr;
 	int a;
-	extern int errno;
 
 	if (pipeOn[module] <= 0)
 	{
@@ -1939,26 +1954,28 @@ void CMD_Send_WindowList(F_CMD_ARGS)
 	{
 		return;
 	}
-	SendPacket(mod, M_NEW_DESK, 1, Scr.CurrentDesk);
+	SendPacket(mod, M_NEW_DESK, 1, (long)Scr.CurrentDesk);
 	SendPacket(
-		mod, M_NEW_PAGE, 7, Scr.Vx, Scr.Vy, Scr.CurrentDesk,
-		Scr.MyDisplayWidth, Scr.MyDisplayHeight,
-		(int)(Scr.VxMax / Scr.MyDisplayWidth) + 1,
-		(int)(Scr.VyMax / Scr.MyDisplayHeight) + 1);
+		mod, M_NEW_PAGE, 7, (long)Scr.Vx, (long)Scr.Vy,
+		(long)Scr.CurrentDesk, (long)Scr.MyDisplayWidth,
+		(long)Scr.MyDisplayHeight,
+		(long)((Scr.VxMax / Scr.MyDisplayWidth) + 1),
+		(long)((Scr.VyMax / Scr.MyDisplayHeight) + 1));
 
 	if (Scr.Hilite != NULL)
 	{
 		SendPacket(
-			mod, M_FOCUS_CHANGE, 5, FW_W(Scr.Hilite),
-			FW_W_FRAME(Scr.Hilite), (unsigned long)True,
-			Scr.Hilite->hicolors.fore, Scr.Hilite->hicolors.back);
+			mod, M_FOCUS_CHANGE, 5, (long)FW_W(Scr.Hilite),
+			(long)FW_W_FRAME(Scr.Hilite), (unsigned long)True,
+			(long)Scr.Hilite->hicolors.fore,
+			(long)Scr.Hilite->hicolors.back);
 	}
 	else
 	{
 		SendPacket(
 			mod, M_FOCUS_CHANGE, 5, 0, 0, (unsigned long)True,
-			GetColor(DEFAULT_FORE_COLOR),
-			GetColor(DEFAULT_BACK_COLOR));
+			(long)GetColor(DEFAULT_FORE_COLOR),
+			(long)GetColor(DEFAULT_BACK_COLOR));
 	}
 	if (Scr.DefaultIcon != NULL)
 	{
@@ -2004,16 +2021,18 @@ void CMD_Send_WindowList(F_CMD_ARGS)
 			if (rc == True)
 			{
 				SendPacket(
-					mod, M_ICONIFY, 7, FW_W(t),
-					FW_W_FRAME(t), (unsigned long)t, r.x,
-					r.y, r.width, r.height);
+					mod, M_ICONIFY, 7, (long)FW_W(t),
+					(long)FW_W_FRAME(t), (unsigned long)t,
+					(long)r.x, (long)r.y,
+					(long)r.width, (long)r.height);
 			}
 		}
 		if ((IS_ICONIFIED(t))&&(IS_ICON_UNMAPPED(t)))
 		{
 			SendPacket(
-				mod, M_ICONIFY, 7, FW_W(t), FW_W_FRAME(t),
-				(unsigned long)t, 0, 0, 0, 0);
+				mod, M_ICONIFY, 7, (long)FW_W(t),
+				(long)FW_W_FRAME(t), (unsigned long)t,
+				(long)0, (long)0, (long)0, (long)0);
 		}
 		if (FMiniIconsSupported && t->mini_icon != NULL)
 		{
@@ -2027,16 +2046,18 @@ void CMD_Send_WindowList(F_CMD_ARGS)
 	if (Scr.Hilite == NULL)
 	{
 		BroadcastPacket(
-			M_FOCUS_CHANGE, 5, 0, 0, (unsigned long)True,
-			GetColor(DEFAULT_FORE_COLOR),
-			GetColor(DEFAULT_BACK_COLOR));
+			M_FOCUS_CHANGE, 5, (long)0, (long)0,
+			(unsigned long)True,
+			(long)GetColor(DEFAULT_FORE_COLOR),
+			(long)GetColor(DEFAULT_BACK_COLOR));
 	}
 	else
 	{
 		BroadcastPacket(
-			M_FOCUS_CHANGE, 5, FW_W(Scr.Hilite), FW_W(Scr.Hilite),
-			(unsigned long)True, Scr.Hilite->hicolors.fore,
-			Scr.Hilite->hicolors.back);
+			M_FOCUS_CHANGE, 5, (long)FW_W(Scr.Hilite),
+			(long)FW_W(Scr.Hilite), (unsigned long)True,
+			(long)Scr.Hilite->hicolors.fore,
+			(long)Scr.Hilite->hicolors.back);
 	}
 
 	SendPacket(mod, M_END_WINDOWLIST, 0);
