@@ -39,7 +39,6 @@
 
 #include <stdio.h>
 #include <unistd.h>
-#include <stdlib.h>
 
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
@@ -189,17 +188,17 @@ int FlocaleChar2bOneCharToUtf8(XChar2b c, char *buf)
 
 /* return number of bytes of character at current position
    (pointed to by str) */
-int FlocaleStringNumberOfBytes(FlocaleFont *flf, const unsigned char *str)
+int FlocaleStringNumberOfBytes(FlocaleFont *flf, const char *str)
 {
         int bytes = 0;
         if(FLC_ENCODING_TYPE_IS_UTF_8(flf->fc))
 	{
 	        /* handle UTF-8 */
-	        if(str[0] <= 0x7f)
+	        if ((str[0] & 0x80) == 0)
 	        {
 		        bytes = 1;
 	        }
-		else if(str[0] <= 0xdf)
+		else if((str[0] & ~0xdf) == 0)
 		{
 		        bytes = 2;
 		}
@@ -212,7 +211,7 @@ int FlocaleStringNumberOfBytes(FlocaleFont *flf, const unsigned char *str)
 	else if(flf->flags.is_mb)
 	{
 	        /* non-UTF-8 multibyte encoding */
-	        if(str[0] <= 0x7f)
+	        if ((str[0] & 0x80) == 0)
 		{
 			bytes = 1;
 		}
@@ -231,17 +230,17 @@ int FlocaleStringNumberOfBytes(FlocaleFont *flf, const unsigned char *str)
 
 /* given a string, font specifying its locale and a byte offset gives
    character offset */
-int FlocaleStringByteToCharOffset(FlocaleFont *flf, const unsigned char *str,
+int FlocaleStringByteToCharOffset(FlocaleFont *flf, const char *str,
 				  int offset)
 {
-	const unsigned char *curr_ptr = str;
+	const char *curr_ptr = str;
 	int i = 0;
 	int len = strlen(str);
 	int coffset = 0;
 	int curr_len;
 
-	for(i = 0 ; 
-	    i < offset && i < len ; 
+	for(i = 0 ;
+	    i < offset && i < len ;
 	    i += curr_len, curr_ptr += curr_len, coffset++)
 	{
 		curr_len = FlocaleStringNumberOfBytes(flf, curr_ptr);
@@ -249,19 +248,19 @@ int FlocaleStringByteToCharOffset(FlocaleFont *flf, const unsigned char *str,
 	return coffset;
 }
 
-/* like above but reversed, ie. return byte offset corresponding to given 
+/* like above but reversed, ie. return byte offset corresponding to given
    charater offset */
-int FlocaleStringCharToByteOffset(FlocaleFont *flf, const unsigned char *str,
+int FlocaleStringCharToByteOffset(FlocaleFont *flf, const char *str,
 				  int coffset)
 {
-	const unsigned char *curr_ptr = str;
+	const char *curr_ptr = str;
 	int i;
 	int len = strlen(str);
 	int offset = 0;
 	int curr_len;
 
-	for(i = 0 ; 
-	    i < coffset && i < len ; 
+	for(i = 0 ;
+	    i < coffset && i < len ;
 	    offset += curr_len, curr_ptr += curr_len, i++)
 	{
 		curr_len = FlocaleStringNumberOfBytes(flf, curr_ptr);
@@ -270,17 +269,17 @@ int FlocaleStringCharToByteOffset(FlocaleFont *flf, const unsigned char *str,
 }
 
 /* return length of string in characters */
-int FlocaleStringCharLength(FlocaleFont *flf, const unsigned char *str)
+int FlocaleStringCharLength(FlocaleFont *flf, const char *str)
 {
 	int i, len;
 	int str_len = strlen(str);
-	for(i = 0, len = 0 ; i < str_len ; 
+	for(i = 0, len = 0 ; i < str_len ;
 	    i += FlocaleStringNumberOfBytes(flf, str+i), len++);
 	return len;
 }
 
 static
-XChar2b *FlocaleUtf8ToUnicodeStr2b(unsigned char *str, int len, int *nl)
+XChar2b *FlocaleUtf8ToUnicodeStr2b(char *str, int len, int *nl)
 {
 	XChar2b *str2b = NULL;
 	int i = 0, j = 0, t;
@@ -288,12 +287,12 @@ XChar2b *FlocaleUtf8ToUnicodeStr2b(unsigned char *str, int len, int *nl)
 	str2b = (XChar2b *)safemalloc((len+1)*sizeof(XChar2b));
 	while (i < len && str[i] != 0)
 	{
-		if (str[i] <= 0x7f)
+		if ((str[i] & 0x80) == 0)
 		{
 			str2b[j].byte2 = str[i];
 			str2b[j].byte1 = 0;
 		}
-		else if (str[i] <= 0xdf && i+1 < len)
+		else if ((str[i] & ~0xdf) == 0 && i+1 < len)
 		{
 			t = ((str[i] & 0x1f) << 6) + (str[i+1] & 0x3f);
 			str2b[j].byte2 = (unsigned char)(t & 0xff);
@@ -323,7 +322,7 @@ XChar2b *FlocaleUtf8ToUnicodeStr2b(unsigned char *str, int len, int *nl)
  * big5hkscs-0, and cns-11643- */
 static
 XChar2b *FlocaleStringToString2b(
-	Display *dpy, FlocaleFont *flf, unsigned char *str, int len, int *nl)
+	Display *dpy, FlocaleFont *flf, char *str, int len, int *nl)
 {
 	XChar2b *str2b = NULL;
 	char *tmp = NULL;
@@ -353,7 +352,7 @@ XChar2b *FlocaleStringToString2b(
 	{
 		while (i < len && str[i] != 0)
 		{
-			if (str[i] <= 0x7f)
+			if ((str[i] & 0x80) == 0)
 			{
 				/* seems ok with KSC5601 and GB2312 as we get
 				 * almost the ascii. I do no try
@@ -382,7 +381,7 @@ XChar2b *FlocaleStringToString2b(
 	{
 		while (i < len && str[i] != 0)
 		{
-			if (str[i] <= 0x7f)
+			if ((str[i] & 0x80) == 0)
 			{
 				/* we should convert to ascii */
 #if 0
@@ -443,15 +442,14 @@ char *FlocaleEncodeString(
 
 	        /* first process combining characters */
 	        tmp_str = FiconvCharsetToUtf8(
-			dpy,
-			flf->str_fc,
-			(const char *)str,len);
+			dpy, flf->str_fc, (const char *)str,len);
 		/* if conversion to UTF-8 failed str1 will be NULL */
 		if(tmp_str != NULL)
 		{
 			/* do combining */
-			len = CombineChars(tmp_str,strlen(tmp_str),
-					    comb_chars, l_to_v);
+			len = CombineChars((unsigned char *)tmp_str,
+					   strlen(tmp_str), comb_chars,
+					   l_to_v);
 			/* returns the length of the resulting UTF-8 string */
 			/* convert back to current charset */
 			str1 = FiconvUtf8ToCharset(
@@ -507,7 +505,7 @@ char *FlocaleEncodeString(
  			(*comb_chars)[0].c.byte1 = 0;
  			(*comb_chars)[0].c.byte2 = 0;
 		}
-				
+
 		/* initialise logic to visual mapping here if that is demanded
 		   (this is default when no combining has been done (1-to-1))
 		*/
@@ -523,9 +521,9 @@ char *FlocaleEncodeString(
 	if (FlocaleGetBidiCharset(dpy, flf->str_fc) != NULL &&
 	    (bidi_charset = FlocaleGetBidiCharset(dpy, flf->fc)) != NULL)
 	{
-		str3 = FBidiConvert(str2, bidi_charset, len1, 
-				    is_rtl, &len2, 
-				    comb_chars != NULL ? *comb_chars : NULL, 
+		str3 = FBidiConvert(str2, bidi_charset, len1,
+				    is_rtl, &len2,
+				    comb_chars != NULL ? *comb_chars : NULL,
 				    l_to_v != NULL ? *l_to_v : NULL);
 		if (str3 != NULL && str3  != str2)
 		{
@@ -608,7 +606,7 @@ void FlocaleFontStructDrawString(
 	{
 		FlocaleInitGstpArgs(&gstp_args, flf, fws, x, y);
 		/* normal drawing */
-		if (flf->shadow_size != 0 && has_fg_pixels)
+		if (flf->shadow_size != 0 && has_fg_pixels == True)
 		{
 			XSetForeground(dpy, fws->gc, fgsh);
 			while (FlocaleGetShadowTextPosition(
@@ -618,6 +616,9 @@ void FlocaleFontStructDrawString(
 					is_string16, dpy, d, gc, xt, yt,
 					fws->e_str, fws->str2b, len);
 			}
+		}
+		if (has_fg_pixels == True)
+		{
 			XSetForeground(dpy, gc, fg);
 		}
 		xt = gstp_args.orig_x;
@@ -653,9 +654,13 @@ void FlocaleRotateDrawString(
 	char buf[4];
 
 	if (fws->str == NULL || len < 1)
+	{
 		return;
+	}
 	if (fws->flags.text_rotation == ROTATION_0)
+	{
 		return; /* should not happen */
+	}
 
 	if (my_gc == None)
 	{
@@ -713,12 +718,12 @@ void FlocaleRotateDrawString(
 	{
 		while(comb_chars[i].c.byte1 != 0 && comb_chars[i].c.byte2 != 0)
 		{
-			/* draw composing character on top of corresponding 
+			/* draw composing character on top of corresponding
 			   "real" character */
 			FlocaleWinString tmp_fws = *fws;
 			int offset = pixel_pos[comb_chars[i].position];
 			int curr_len = FlocaleChar2bOneCharToUtf8(
-							    comb_chars[i].c, 
+							    comb_chars[i].c,
 							    buf);
 			int out_len;
 			char *buf2 = FiconvUtf8ToCharset(
@@ -737,9 +742,9 @@ void FlocaleRotateDrawString(
 			tmp_fws.str2b = NULL;
 			if(flf->fontset != None)
 			{
-				XmbDrawString(dpy, canvas_pix, flf->fontset, 
+				XmbDrawString(dpy, canvas_pix, flf->fontset,
 					      fws->gc, offset,
-					      height - descent, buf2, 
+					      height - descent, buf2,
 					      strlen(buf2));
 			}
 			else if(flf->font != None)
@@ -755,7 +760,7 @@ void FlocaleRotateDrawString(
 				}
 				else if (flf->flags.is_mb)
 				{
-					tmp_fws.str2b = 
+					tmp_fws.str2b =
 					  FlocaleStringToString2b(
 						    dpy, flf, tmp_fws.e_str,
 						    curr_len, &out_len);
@@ -767,7 +772,7 @@ void FlocaleRotateDrawString(
 				XSetFont(dpy, font_gc, flf->font->fid);
 				FlocaleFontStructDrawString(
 					dpy, flf, canvas_pix, font_gc,
-					offset, height - descent, 
+					offset, height - descent,
 					fg, fgsh, has_fg_pixels, &tmp_fws,
 					out_len, True);
 			}
@@ -900,7 +905,7 @@ void FlocaleRotateDrawString(
 	XSetFillStyle(dpy, my_gc, FillStippled);
 	XSetStipple(dpy, my_gc, rotated_pix);
 	FlocaleInitGstpArgs(&gstp_args, flf, fws, xpfg, ypfg);
-	if (flf->shadow_size != 0 && has_fg_pixels)
+	if (flf->shadow_size != 0 && has_fg_pixels == True)
 	{
 		XSetForeground(dpy, my_gc, fgsh);
 		while (FlocaleGetShadowTextPosition(&xpsh, &ypsh, &gstp_args))
@@ -910,7 +915,6 @@ void FlocaleRotateDrawString(
 				dpy, fws->win, my_gc, xpsh, ypsh, rotated_w,
 				rotated_h);
 		}
-		XSetForeground(dpy, my_gc, fg);
 	}
 	xpsh = gstp_args.orig_x;
 	ypsh = gstp_args.orig_y;
@@ -1372,7 +1376,7 @@ FlocaleFont *FlocaleLoadFont(Display *dpy, char *fontname, char *module)
 	char *str, *opt_str, *encoding= NULL, *fn = NULL;
 	int shadow_size = 0;
 	int shadow_offset = 0;
-	int shadow_dir;
+	int shadow_dir = MULTI_DIR_SE;
 	int i;
 
 	/* removing quoting for modules */
@@ -1547,7 +1551,7 @@ FlocaleFont *FlocaleLoadFont(Display *dpy, char *fontname, char *module)
 		if (flf->fc == FlocaleCharsetGetUnknownCharset())
 		{
 			fprintf(stderr,"[%s][FlocaleLoadFont]: "
-				"WARNING -- Unkown charset for font\n\t'%s'\n",
+				"WARNING -- Unknown charset for font\n\t'%s'\n",
 				(module)? module: "FVWMlibs", flf->name);
 			flf->fc = FlocaleCharsetGetDefaultCharset(dpy, module);
 		}
@@ -1557,7 +1561,7 @@ FlocaleFont *FlocaleLoadFont(Display *dpy, char *fontname, char *module)
 			   flf->fftf.str_encoding != NULL)))
 		{
 			fprintf(stderr,"[%s][FlocaleLoadFont]: "
-				"WARNING -- Unkown string encoding for font\n"
+				"WARNING -- Unknown string encoding for font\n"
 				"\t'%s'\n",
 				(module)? module: "FVWMlibs", flf->name);
 		}
@@ -1828,8 +1832,8 @@ void FlocaleDrawString(
 	}
 
 	/* encode the string */
-	FlocaleEncodeWinString(dpy, flf, fws, &do_free, &len, &comb_chars,
-			       NULL);
+	FlocaleEncodeWinString(
+		dpy, flf, fws, &do_free, &len, &comb_chars, NULL);
 	curr_str = fws->e_str;
 	for(char_len = 0, i = 0 ;
 	    i < len && curr_str[i] != 0 ;
@@ -1839,15 +1843,16 @@ void FlocaleDrawString(
 	}
 
 	/* for superimposition calculate the character positions in pixels */
-	if(comb_chars != NULL)
+	if (comb_chars != NULL && (
+		   comb_chars[0].c.byte1 != 0 || comb_chars[0].c.byte2 != 0))
 	{
-		/* the second condition is actually redundant, 
-		   but there for clarity, 
+		/* the second condition is actually redundant,
+		   but there for clarity,
 		   ending at 0 is what's expected in a correct
 		   string */
-		pixel_pos = (int *)safemalloc((char_len != 0 ? char_len : 1)
-					      * sizeof(int));
-		
+		pixel_pos = (int *)safemalloc(
+			(char_len != 0 ? char_len : 1) * sizeof(int));
+
 		/* if there is 0 bytes in the encoded string, there might
 		   still be combining character to draw (at position 0) */
 		if(char_len == 0)
@@ -1855,19 +1860,20 @@ void FlocaleDrawString(
 			pixel_pos[0] = 0;
 		}
 
-		for(i = 0, curr_pixel_pos = 0 ; 
-		    i < char_len ; 
-		    i++, curr_str += curr_len)
+		for(
+			i = 0, curr_pixel_pos = 0 ;
+			i < char_len ;
+			i++, curr_str += curr_len)
 		{
 		        curr_len = FlocaleStringNumberOfBytes(flf, curr_str);
-			for(j = 0 ; j < curr_len ; j++)
+			for (j = 0 ; j < curr_len ; j++)
 			{
 			        buf[j] = curr_str[j];
 			}
 			buf[j] = 0;
 			pixel_pos[i] = curr_pixel_pos;
 			/* need to compensate for shadow width (if any) */
-			curr_pixel_pos += 
+			curr_pixel_pos +=
 				FlocaleTextWidth(flf, buf, curr_len) -
 				FLF_SHADOW_WIDTH(flf);
 		}
@@ -1896,7 +1902,7 @@ void FlocaleDrawString(
 		has_fg_pixels = True;
 	}
 
-	if(flf->font != None && 
+	if(flf->font != None &&
 	   (FLC_ENCODING_TYPE_IS_UTF_8(flf->fc) || flf->flags.is_mb))
 	{
 		/* in this case, length is number of 2-byte chars */
@@ -1932,6 +1938,9 @@ void FlocaleDrawString(
 					dpy, fws->win, flf->fontset, fws->gc,
 					xt, yt, fws->e_str, len);
 			}
+		}
+		if (has_fg_pixels == True)
+		{
 			XSetForeground(dpy, fws->gc, fg);
 		}
 		xt = gstp_args.orig_x;
@@ -1949,7 +1958,7 @@ void FlocaleDrawString(
 
 	/* here take care of superimposing chars */
 	i = 0;
-	if(comb_chars != NULL)
+	if (comb_chars != NULL)
 	{
 		while(comb_chars[i].c.byte1 != 0 && comb_chars[i].c.byte2 != 0)
 		{
@@ -1959,7 +1968,7 @@ void FlocaleDrawString(
 			int offset = pixel_pos[comb_chars[i].position];
 			char *buf2;
 			int out_len;
-		        curr_len = FlocaleChar2bOneCharToUtf8(comb_chars[i].c, 
+		        curr_len = FlocaleChar2bOneCharToUtf8(comb_chars[i].c,
 							      buf);
 			buf2 = FiconvUtf8ToCharset(
 				dpy, flf->str_fc, (const char *)buf, curr_len);
@@ -1985,8 +1994,9 @@ void FlocaleDrawString(
 			{
 			        int xt = fws->x;
 				int yt = fws->y;
-				FlocaleInitGstpArgs(&gstp_args, flf, fws, 
-						    fws->x, fws->y);
+
+				FlocaleInitGstpArgs(
+					&gstp_args, flf, fws, fws->x, fws->y);
 				if (flf->shadow_size != 0)
 				{
 					XSetForeground(dpy, fws->gc, fgsh);
@@ -1994,22 +2004,22 @@ void FlocaleDrawString(
 					       &xt, &yt, &gstp_args))
 					{
 						XmbDrawString(
-							      dpy, fws->win, 
-							      flf->fontset, 
+							      dpy, fws->win,
+							      flf->fontset,
 							      fws->gc,
-							      xt, yt, 
-							      buf2, 
+							      xt, yt,
+							      buf2,
 							      strlen(buf2));
 					}
-					XSetForeground(dpy, fws->gc, fg);
 				}
+				XSetForeground(dpy, fws->gc, fg);
 				xt = gstp_args.orig_x;
 				yt = gstp_args.orig_y;
 			        XmbDrawString(
 					dpy, fws->win, flf->fontset, fws->gc,
 					xt + offset, yt, buf2, strlen(buf2));
 			}
-			else if(flf->font != None)
+			else if (flf->font != None)
 			{
 				if (FLC_ENCODING_TYPE_IS_UTF_8(flf->fc))
 				{
@@ -2052,7 +2062,7 @@ void FlocaleDrawString(
 		}
 	}
 
-	if(do_free)
+	if (do_free)
 	{
 		if (fws->e_str != NULL)
 		{
@@ -2070,7 +2080,8 @@ void FlocaleDrawString(
 	if(comb_chars != NULL)
 	{
 	        free(comb_chars);
-		free(pixel_pos);
+		if(pixel_pos)
+			free(pixel_pos);
 	}
 
 	return;
@@ -2094,7 +2105,7 @@ void FlocaleDrawUnderline(
 	}
 
 	/* need to encode the string first to get BIDI and combining chars */
-	FlocaleEncodeWinString(dpy, flf, fws, &do_free, &len, &comb_chars, 
+	FlocaleEncodeWinString(dpy, flf, fws, &do_free, &len, &comb_chars,
 			       &l_to_v);
 	/* we don't need this, only interested in char mapping */
 	free(comb_chars);
@@ -2111,7 +2122,8 @@ void FlocaleDrawUnderline(
 	off1 = FlocaleTextWidth(flf, fws->e_str, voffset) +
 		((voffset == 0)?
 		 FLF_SHADOW_LEFT_SIZE(flf) : - FLF_SHADOW_RIGHT_SIZE(flf) );
-	off2 = FlocaleTextWidth(flf, fws->e_str + voffset, 1) -
+	off2 = FlocaleTextWidth(flf, fws->e_str + voffset,
+		      FlocaleStringNumberOfBytes(flf, fws->e_str + voffset)) -
 		FLF_SHADOW_WIDTH(flf) - 1 + off1;
 	y = fws->y + 2;
 	x_s = fws->x + off1;
@@ -2126,7 +2138,7 @@ void FlocaleDrawUnderline(
 		free(fws->e_str);
 		fws->e_str = NULL;
 	}
-	
+
 	if(fws->str2b != NULL)
 	{
 		free(fws->str2b);
@@ -2146,18 +2158,32 @@ int FlocaleTextWidth(FlocaleFont *flf, char *str, int sl)
 
 	if (!str || sl == 0)
 		return 0;
-	
+
 	if (sl < 0)
 	{
 		/* a vertical string: nothing to do! */
 		sl = -sl;
 	}
+
 	/* FIXME */
-	tmp_str = FlocaleEncodeString(
-		     Pdpy, flf, str, &do_free, sl, &new_l, NULL, &comb_chars, 
-		     NULL);
+	/* to avoid eccesive calls iconv (slow in Solaris 8)
+	   don't bother to encode if string is one byte
+	   when drawing a string this function is used to calculate
+	   position of each character (for superimposition) */
+	if(sl == 1)
+	{
+		tmp_str = str;
+		new_l = sl;
+		do_free = False;
+	}
+	else
+	{
+		tmp_str = FlocaleEncodeString(
+			   Pdpy, flf, str, &do_free, sl, &new_l, NULL,
+			   &comb_chars, NULL);
+	}
 	/* if we get zero-length, check to to see if there if there's any
-	   combining chars, if so use an imagninary space as a 
+	   combining chars, if so use an imagninary space as a
 	   "base character" */
 	if (strlen(tmp_str) == 0 && comb_chars &&
 	    (comb_chars[0].c.byte1 != 0 || comb_chars[0].c.byte2 != 0))
@@ -2212,7 +2238,7 @@ int FlocaleTextWidth(FlocaleFont *flf, char *str, int sl)
 	{
 		free(comb_chars);
 	}
-	
+
 	return result + ((result != 0)? FLF_SHADOW_WIDTH(flf):0);
 }
 
@@ -2272,11 +2298,11 @@ void FlocaleGetNameProperty(
 	int num;
 	XTextProperty text_prop;
 
+	list = NULL;
 	if (func(dpy, w, &text_prop) == 0)
 	{
 		return;
 	}
-
 	if (text_prop.encoding == XA_STRING)
 	{
 		/* STRING encoding, use this as it is */
