@@ -1,8 +1,7 @@
 /* -*-c-*- */
 /* Copyright (C) 1993, Robert Nation
- * Copyright (C) 2002  Olivier Chapuis
- *
- * This program is free software; you can redistribute it and/or modify
+ * Copyright (C) 2002  Olivier Chapuis */
+/* This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
@@ -17,12 +16,11 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-/* ---------------------------- included header files ----------------------- */
+/* ---------------------------- included header files ---------------------- */
 
 #include "config.h"
 
 #include <stdio.h>
-#include <stdlib.h>
 #include <signal.h>
 #include <ctype.h>
 #include <time.h>
@@ -36,10 +34,10 @@
 #include "PictureUtils.h"
 #include "PictureDitherMatrice.h"
 
-/* ---------------------------- local definitions and macro ------------------ */
-
-#define PICTURE_DEBUG_COLORS_ALLOC_FAILURE 0
+/* ---------------------------- local definitions and macro ----------------- */
 
+#if 0
+/* dv: unused */
 /* form alloc_in_cmap from the xpm lib */
 #define XPM_DIST(r1,g1,b1,r2,g2,b2) (long)\
                           (3*(abs((long)r1-(long)r2) + \
@@ -48,6 +46,7 @@
 			   abs((long)r1 + (long)g1 + (long)b1 - \
 			       ((long)r2 +  (long)g2 + (long)b2)))
 #define XPM_COLOR_CLOSENESS 40000
+#endif
 
 #define SQUARE(X) ((X)*(X))
 
@@ -80,16 +79,16 @@
  * do think that using gamma correction for the colors cubes is a good
  * idea */
 #define USE_GAMMA_CORECTION 0
-/* 2.2 is recommanded by the Poynon colors FAQ, some others suggest 1.5 and 2 
+/* 2.2 is recommanded by the Poynon colors FAQ, some others suggest 1.5 and 2
  * Use float constants!*/
 #define COLOR_GAMMA 1.5
 #define GREY_GAMMA  2.0
 
-/* ---------------------------- imports ------------------------------------- */
+/* ---------------------------- imports ------------------------------------ */
 
-/* ---------------------------- included code files ------------------------- */
+/* ---------------------------- included code files ------------------------ */
 
-/* ---------------------------- local types --------------------------------- */
+/* ---------------------------- local types -------------------------------- */
 
 typedef struct
 {
@@ -128,11 +127,16 @@ typedef struct
 	unsigned short *green_dither;
 	unsigned short *blue_dither;
 	/* colors allocation function */
-	int (*alloc_color)();
-	int (*alloc_color_no_limit)();
-	int (*alloc_color_dither)();
-	void (*free_colors)();
-	void (*free_colors_no_limit)();
+	int (*alloc_color)(Display *dpy, Colormap cmap, XColor *c);
+	int (*alloc_color_no_limit)(Display *dpy, Colormap cmap, XColor *c);
+	int (*alloc_color_dither)(
+		Display *dpy, Colormap cmap, XColor *c, int x, int y);
+	void (*free_colors)(
+		Display *dpy, Colormap cmap, Pixel *pixels, int n,
+		unsigned long planes);
+	void (*free_colors_no_limit)(
+		Display *dpy, Colormap cmap, Pixel *pixels, int n,
+		unsigned long planes);
 } PColorsInfo;
 
 typedef struct {
@@ -140,9 +144,9 @@ typedef struct {
 	long closeness;
 } CloseColor;
 
-/* ---------------------------- forward declarations ------------------------ */
+/* ---------------------------- forward declarations ----------------------- */
 
-/* ---------------------------- local variables ----------------------------- */
+/* ---------------------------- local variables ---------------------------- */
 
 static int PColorLimit = 0;
 static PColor *Pct = NULL;
@@ -155,13 +159,13 @@ static PColorsInfo Pcsi = {
 	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, NULL, NULL, NULL, NULL,
 	NULL, NULL, NULL, NULL};
 
-/* ---------------------------- exported variables (globals) ---------------- */
+/* ---------------------------- exported variables (globals) --------------- */
 
-/* ---------------------------- local functions ----------------------------- */
+/* ---------------------------- local functions ---------------------------- */
 
-/* ***************************************************************************
+/*
  * get shift and prec from a mask
- * ***************************************************************************/
+ */
 static
 void decompose_mask(
 	unsigned long mask, int *shift, int *prec)
@@ -182,10 +186,10 @@ void decompose_mask(
 	}
 }
 
-/* ***************************************************************************
+/*
  * color allocation in the colormap. strongly inspired by SetCloseColor from
  * the Xpm library (depth <= 8)
- * ***************************************************************************/
+ */
 
 static int
 closeness_cmp(const void *a, const void *b)
@@ -264,9 +268,9 @@ int alloc_color_in_cmap(XColor *c, Bool force)
 	return 0;
 }
 
-/* ***************************************************************************
+/*
  * dithering
- * ***************************************************************************/
+ */
 
 static
 int my_dither(int x, int y, XColor *c)
@@ -297,7 +301,7 @@ int my_dither(int x, int y, XColor *c)
 		/* color cube */
 		int dith, rs, gs, bs, gb, b;
 		int tr,tb,tg;
-		
+
 		rs = Pcsi.d_nr - 1;
 		gs = Pcsi.d_ng - 1;
 		bs = Pcsi.d_nb - 1;
@@ -430,9 +434,9 @@ int my_dither_depth_15_16_init(void)
 	return 1;
 }
 
-/* ***************************************************************************
+/*
  * Color allocation in the "palette"
- * ***************************************************************************/
+ */
 
 static
 int alloc_color_in_pct(XColor *c, int index)
@@ -566,9 +570,9 @@ int get_color_index(int r, int g, int b, int is_8)
 	return index;
 }
 
-/* ***************************************************************************
+/*
  * Main colors allocator
- * ***************************************************************************/
+ */
 static
 int alloc_color_proportion(Display *dpy, Colormap cmap, XColor *c)
 {
@@ -674,8 +678,8 @@ int alloc_color_x(
 
 static
 void free_colors_in_table(
-	Display *dpy, Colormap cmap, Pixel *pixels, int n, unsigned long planes,
-	Bool no_limit)
+	Display *dpy, Colormap cmap, Pixel *pixels, int n,
+	unsigned long planes)
 {
 	Pixel *p;
 	int i,j,do_free;
@@ -718,8 +722,8 @@ void free_colors_in_table(
 
 static
 void free_colors_x(
-	Display *dpy, Colormap cmap, Pixel *pixels, int n, unsigned long planes,
-	Bool no_limit)
+	Display *dpy, Colormap cmap, Pixel *pixels, int n,
+	unsigned long planes)
 {
 	XFreeColors(dpy, cmap, pixels, n, planes);
 	if (Pac != NULL)
@@ -737,10 +741,10 @@ void free_colors_x(
 	}
 }
 
-/* ***************************************************************************
+/*
  * local function for building pallet (dynamic colors, private DirectColor
  * cmap)
- * ***************************************************************************/
+ */
 static
 XColor *build_mapping_colors(int nr, int ng, int nb)
 {
@@ -768,7 +772,7 @@ XColor *build_mapping_colors(int nr, int ng, int nb)
 	return colors;
 }
 
-short *build_mapping_table(int nr, int ng, int nb, Bool use_named)
+static short *build_mapping_table(int nr, int ng, int nb, Bool use_named)
 {
 	int size = nr*ng*nb;
 	XColor *colors_map;
@@ -949,7 +953,7 @@ PColor *alloc_color_cube(
 			}
 		}
 	}
-	
+
 	if (ngrey > 0)
 	{
 		for (grey = start_grey; grey < end_grey; grey++)
@@ -1213,9 +1217,9 @@ void create_mapping_table(
 	}
 }
 
-void finish_ct_init(
-	int call_type, int ctt, int nr, int ng, int nb, int ngrey, int grey_bits,
-	Bool use_named)
+static void finish_ct_init(
+	int call_type, int ctt, int nr, int ng, int nb, int ngrey,
+	int grey_bits, Bool use_named)
 {
 	if (call_type == PICTURE_CALLED_BY_FVWM)
 	{
@@ -1241,9 +1245,10 @@ void finish_ct_init(
 		{
 			ctt++;
 		}
-		env = safemalloc(21+PICTURE_TABLETYPE_LENGHT+1);
-		sprintf(env,"FVWM_COLORTABLE_TYPE=%i",ctt);
-		putenv(env);
+		env = safemalloc(PICTURE_TABLETYPE_LENGHT + 1);
+		sprintf(env, "%i", ctt);
+		flib_putenv("FVWM_COLORTABLE_TYPE", env);
+		free(env);
 		if (Pdepth <= 8)
 		{
 			Pac = (PColor *)safecalloc(
@@ -1282,9 +1287,10 @@ int PictureAllocColorTable(
 	int strict_cl_set = False;
 	int alloc_table_set = False;
 	int color_limit;
-	int pa_type = (Pvisual->class != GrayScale)? PA_COLOR_CUBE:PA_GRAY_SCALE;
-	int fvwm_type = (Pvisual->class != GrayScale)?
-		FVWM_COLOR_CUBE:FVWM_GRAY_SCALE;
+	int pa_type = (Pvisual->class != GrayScale) ?
+		PA_COLOR_CUBE : PA_GRAY_SCALE;
+	int fvwm_type = (Pvisual->class != GrayScale) ?
+		FVWM_COLOR_CUBE : FVWM_GRAY_SCALE;
 	int cc[][6] =
 	{
 		/* {nr,ng,nb,ngrey,grey_bits,logic} */
@@ -1565,8 +1571,8 @@ int PictureAllocColorTable(
 	while(Pdepth <= 8 && !private_cmap && use_default &&
 	      i < cc_nbr && Pct == NULL && (Pvisual->class & 1))
 	{
-		size = cc[i][0]*cc[i][1]*cc[i][2] + cc[i][3] - 2*(cc[i][3] > 0) +
-			(1 << cc[i][4])*(cc[i][4] != 0);
+		size = cc[i][0]*cc[i][1]*cc[i][2] + cc[i][3] -
+			2*(cc[i][3] > 0) + (1 << cc[i][4])*(cc[i][4] != 0);
 		if (size > nbr_of_color || !(cc[i][5] & pa_type))
 		{
 			i++;
@@ -1575,8 +1581,8 @@ int PictureAllocColorTable(
 		if (free_colors <= nbr_of_color - size)
 		{
 			Pct = alloc_color_cube(
-				cc[i][0], cc[i][1], cc[i][2], cc[i][3], cc[i][4],
-				True);
+				cc[i][0], cc[i][1], cc[i][2], cc[i][3],
+				cc[i][4], True);
 		}
 		if (Pct != NULL)
 		{
@@ -1660,7 +1666,7 @@ int PictureAllocColorTable(
 				 * limit = 68 4x4x4 + 4
 				 * limit = 64 4x4x4 + 0 */
 			}
-			
+
 		}
 		else if (limit == 128 || limit == 64)
 		{
@@ -1767,9 +1773,9 @@ int PictureAllocColorTable(
 	return PColorLimit;
 }
 
-/* ***************************************************************************
+/*
  * Allocation of a private DirectColor cmap this is broken for depth > 16
- * ***************************************************************************/
+ */
 static
 Bool alloc_direct_colors(int *limit, Bool use_my_color_limit)
 {
@@ -1837,15 +1843,15 @@ Bool alloc_direct_colors(int *limit, Bool use_my_color_limit)
 	return 1;
 }
 
-/* ***************************************************************************
+/*
  * Init the table for Static Colors
- * ***************************************************************************/
+ */
 static
 void init_static_colors_table(void)
 {
 	XColor colors[256];
 	int i;
-	int nbr_of_colors = max(256, (1 << Pdepth));
+	int nbr_of_colors = min(256, (1 << Pdepth));
 
 	PColorLimit = nbr_of_colors;
 	Pct = (PColor *)safemalloc((nbr_of_colors+1) * sizeof(PColor));
@@ -1869,9 +1875,9 @@ void init_static_colors_table(void)
 }
 
 
-/* ***************************************************************************
+/*
  * misc local functions
- * ***************************************************************************/
+ */
 static
 void print_colormap(Colormap cmap)
 {
@@ -1892,7 +1898,7 @@ void print_colormap(Colormap cmap)
 	}
 }
 
-/* ---------------------------- interface functions ------------------------- */
+/* ---------------------------- interface functions ------------------------ */
 
 int PictureAllocColor(Display *dpy, Colormap cmap, XColor *c, int no_limit)
 {
@@ -1995,8 +2001,8 @@ PictureImageColorAllocator *PictureOpenImageColorAllocator(
 }
 
 void PictureCloseImageColorAllocator(
-	Display *dpy, PictureImageColorAllocator *pica, int *nalloc_pixels,
-	Pixel **alloc_pixels, Bool *no_limit)
+	Display *dpy, PictureImageColorAllocator *pica,
+	unsigned int *nalloc_pixels, Pixel **alloc_pixels, Bool *no_limit)
 {
 	if (nalloc_pixels)
 	{
@@ -2013,7 +2019,8 @@ void PictureCloseImageColorAllocator(
 	if (pica->pixels_table)
 	{
 		int i,j;
-		int k = 0, l = 0, np = 0;
+		int k = 0, l = 0;
+		unsigned int np = 0;
 		int free_num = 0;
 		Pixel *free_pixels = NULL;
 		Pixel *save_pixels = NULL;
@@ -2045,7 +2052,7 @@ void PictureCloseImageColorAllocator(
 				}
 				for(j=1; j < pica->pixels_table[i]; j++)
 				{
-					free_pixels[l++] = i; 
+					free_pixels[l++] = i;
 				}
 			}
 		}
@@ -2076,8 +2083,8 @@ void PictureCloseImageColorAllocator(
 }
 
 void PictureFreeColors(
-	Display *dpy, Colormap cmap, Pixel *pixels, int n, unsigned long planes,
-	Bool no_limit)
+	Display *dpy, Colormap cmap, Pixel *pixels, int n,
+	unsigned long planes, Bool no_limit)
 {
 	if (no_limit)
 	{
@@ -2241,7 +2248,7 @@ int PictureInitColors(
 		{
 			Pcsi.alloc_color_dither = NULL;
 		}
-		break;	
+		break;
 	case StaticColor:
 		if (0 && Pvisual->red_mask != 0 && Pvisual->green_mask != 0 &&
 		    Pvisual->blue_mask != 0)
@@ -2322,7 +2329,7 @@ int PictureInitColors(
 		if (call_type == PICTURE_CALLED_BY_FVWM &&
 		    getenv("FVWM_COLORTABLE_TYPE") != NULL)
 		{
-			putenv("FVWM_COLORTABLE_TYPE=");
+			flib_putenv("FVWM_COLORTABLE_TYPE", "");
 		}
 		return PColorLimit;
 	}
@@ -2337,7 +2344,7 @@ int PictureInitColors(
 		if (call_type == PICTURE_CALLED_BY_FVWM &&
 		    getenv("FVWM_COLORTABLE_TYPE") != NULL)
 		{
-			putenv("FVWM_COLORTABLE_TYPE=");
+			flib_putenv("FVWM_COLORTABLE_TYPE", "");
 		}
 		return 0;
 	}
