@@ -1,3 +1,4 @@
+/* -*-c-*- */
 /* This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -13,12 +14,12 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/***************************************************************************
+/*
  * FvwmBanner
  *
  *  Show Fvwm Banner
  *
- ***************************************************************************/
+ */
 
 #include "config.h"
 
@@ -59,14 +60,14 @@
 typedef struct _PImageIcon {
 	Pixmap pixmap;
 	Pixmap mask;
-	int width;
-	int height;
+	unsigned int width;
+	unsigned int height;
 	int depth;
 }        PImageIcon;
 
-/**************************************************************************
+/*
  * A few function prototypes
- **************************************************************************/
+ */
 void RedrawWindow(void);
 void GetXPMData(char **);
 void GetXBMData(void);
@@ -96,11 +97,11 @@ Bool no_wm = False;
 
 #define MW_EVENTS   (ExposureMask | ButtonReleaseMask)
 
-/****************************************************************************
+/*
  *
  * Creates an icon window as needed
  *
- ****************************************************************************/
+ */
 int main(int argc, char **argv)
 {
   char *display_name = NULL, *string = NULL;
@@ -111,6 +112,8 @@ int main(int argc, char **argv)
   struct timeval value;
   int fd[2];
   XSetWindowAttributes attr;
+  int wx;
+  int wy;
 
   /* Save our program  name - for error messages */
   string = strrchr (argv[0], '/');
@@ -155,6 +158,8 @@ int main(int argc, char **argv)
   FScreenInit(dpy);
   FShapeInit(dpy);
   FRenderInit(dpy);
+  /* SetMessageMask(fd, M_CONFIG_INFO | M_END_CONFIG_INFO  | M_SENDCONFIG); */
+  SetMessageMask(fd, M_CONFIG_INFO | M_END_CONFIG_INFO);
   parseOptions(fd);
 
   /* chick in the neck situation:
@@ -211,8 +216,8 @@ int main(int argc, char **argv)
   mysizehints.win_gravity = NorthWestGravity;
 
   FScreenCenterOnScreen(
-    NULL, FSCREEN_PRIMARY, &mysizehints.x, &mysizehints.y,
-    view.width, view.height);
+    NULL, FSCREEN_PRIMARY, &wx, &wy, view.width, view.height);
+  FScreenMangleScreenIntoUSPosHints(FSCREEN_PRIMARY, &mysizehints);
 
   wm_del_win = XInternAtom(dpy,"WM_DELETE_WINDOW",False);
   XSetWMProtocols(dpy,win,&wm_del_win,1);
@@ -220,8 +225,7 @@ int main(int argc, char **argv)
   XSetWMNormalHints(dpy,win,&mysizehints);
   change_window_name("FvwmBanner");
 
-  XMoveResizeWindow(dpy, win, mysizehints.x, mysizehints.y, mysizehints.width,
-		    mysizehints.height);
+  XMoveResizeWindow(dpy, win, wx, wy, mysizehints.width, mysizehints.height);
 
   if (FHaveShapeExtension && view.mask != None)
     FShapeCombineMask(dpy, win, FShapeBounding, 0, 0, view.mask, FShapeSet);
@@ -277,11 +281,11 @@ int main(int argc, char **argv)
   return 0;
 }
 
-/****************************************************************************
+/*
  *
  * Looks for a color XPM icon file
  *
- ****************************************************************************/
+ */
 void GetXBMData(void)
 {
   if ((view.pixmap =
@@ -324,7 +328,8 @@ void GetImageFile(char *file, char *path)
 	{
 		if(PImageLoadPixmapFromFile(
 			dpy, Root, full_file, &view.pixmap, &view.mask, None,
-			&view.width, &view.height, &view.depth, 0, NULL, fpa))
+			&view.width, &view.height, &view.depth, 0, NULL, 0,
+			fpa))
 		{
 			return;
 		}
@@ -404,9 +409,9 @@ static void parseOptions(int fd[2])
   return;
 }
 
-/**************************************************************************
+/*
  *  Change the window name displayed in the title bar.
- **************************************************************************/
+ */
 void change_window_name(char *str)
 {
   XTextProperty name;
@@ -422,15 +427,16 @@ void change_window_name(char *str)
 }
 
 
-/***********************************************************************
+/*
  *
  *  Procedure:
  *      SIGPIPE handler - SIGPIPE means fvwm is dying
  *
- ***********************************************************************/
+ */
 
 /*ARGSUSED*/
-void DeadPipe (int nonsense)
+RETSIGTYPE DeadPipe (int nonsense)
 {
   exit (0);
+  SIGNAL_RETURN;
 }
