@@ -1,3 +1,4 @@
+/* -*-c-*- */
 /*
  * FvwmButtons, copyright 1996, Jarl Totland
  *
@@ -31,7 +32,6 @@
 
 #include <unistd.h>
 #include <ctype.h>
-#include <stdlib.h>
 #include <stdio.h>
 #include <X11/Intrinsic.h>
 #include "FvwmButtons.h"
@@ -40,7 +40,7 @@
 extern char *MyName;
 
 
-int buttonXPos(button_info *b, int i)
+int buttonXPos(const button_info *b, int i)
 {
   int column = i % b->parent->c->num_columns;
 
@@ -48,7 +48,7 @@ int buttonXPos(button_info *b, int i)
     (b->parent->c->width * column / b->parent->c->num_columns);
 }
 
-int buttonYPos(button_info *b, int i)
+int buttonYPos(const button_info *b, int i)
 {
   int row = i / b->parent->c->num_columns;
 
@@ -56,7 +56,7 @@ int buttonYPos(button_info *b, int i)
     (b->parent->c->height * row / b->parent->c->num_rows);
 }
 
-int buttonWidth(button_info *b)
+int buttonWidth(const button_info *b)
 {
   int column = b->n % b->parent->c->num_columns;
   int column2 = column + b->BWidth;
@@ -65,7 +65,7 @@ int buttonWidth(button_info *b)
     (b->parent->c->width * column / b->parent->c->num_columns);
 }
 
-int buttonHeight(button_info *b)
+int buttonHeight(const button_info *b)
 {
   int row = b->n / b->parent->c->num_columns;
   int row2 = row + b->BHeight;
@@ -76,7 +76,7 @@ int buttonHeight(button_info *b)
 
 int buttonSwallowCount(button_info *b)
 {
-  return (b->flags & (b_Swallow | b_Panel)) ? (b->swallow & b_Count) : 0;
+  return (b->flags.b_Swallow || b->flags.b_Panel) ? (b->swallow & b_Count) : 0;
 }
 
 
@@ -84,39 +84,39 @@ int buttonSwallowCount(button_info *b)
 *** buttonInfo()
 *** Give lots of info for this button: XPos, YPos, XPad, YPad, Frame(signed)
 **/
-void buttonInfo(button_info *b,int *x,int *y,int *px,int *py,int *f)
+void buttonInfo(const button_info *b, int *x, int *y, int *px, int *py, int *f)
 {
-  ushort w=b_Padding|b_Frame;
+  ushort bPadding = (b->flags.b_Padding ? 0 : 1);
+  ushort bFrame = (b->flags.b_Frame ? 0 : 1);
   *x=buttonXPos(b,b->n);
   *y=buttonYPos(b,b->n);
   *px=b->xpad;
   *py=b->ypad;
   *f=b->framew;
-  w&=~(b->flags&(b_Frame|b_Padding));
 
-  if(b->flags&b_Container && w&b_Frame)
+  if(b->flags.b_Container && bFrame)
   {
     *f=0;
-    w&=~b_Frame;
+    bFrame = 0;
   }
-  if((b->flags&b_Container || b->flags&b_Swallow) && w&b_Padding)
+  if((b->flags.b_Container || b->flags.b_Swallow) && bPadding)
   {
     *px=*py=0;
-    w&=~b_Padding;
+    bPadding = 0;
   }
 
-  while(w && (b=b->parent))
+  while((bPadding || bFrame) && (b=b->parent))
   {
-    if(w&b_Frame && b->c->flags&b_Frame)
+    if(bFrame && b->c->flags.b_Frame)
     {
       *f=b->c->framew;
-      w&=~b_Frame;
+      bFrame = 0;
     }
-    if(w&b_Padding && b->c->flags&b_Padding)
+    if(bPadding && b->c->flags.b_Padding)
     {
       *px=b->c->xpad;
       *py=b->c->ypad;
-      w&=~b_Padding;
+      bPadding = 0;
     }
   }
 }
@@ -151,12 +151,12 @@ void GetInternalSize(button_info *b,int *x,int *y,int *w,int *h)
 **/
 int buttonFrameSigned(button_info *b)
 {
-  if(b->flags&b_Frame)
+  if(b->flags.b_Frame)
     return b->framew;
-  if(b->flags&b_Container)              /* Containers usually gets 0 relief  */
+  if(b->flags.b_Container)	/* Containers usually gets 0 relief  */
     return 0;
   while((b=b->parent))
-    if(b->c->flags&b_Frame)
+    if(b->c->flags.b_Frame)
       return b->c->framew;
 #ifdef DEBUG
   fprintf(stderr,"%s: BUG: No relief width definition?\n",MyName);
@@ -170,12 +170,12 @@ int buttonFrameSigned(button_info *b)
 **/
 int buttonXPad(button_info *b)
 {
-  if(b->flags&b_Padding)
+  if(b->flags.b_Padding)
     return b->xpad;
-  if(b->flags&(b_Container|b_Swallow))  /* Normally no padding for these     */
+  if(b->flags.b_Container || b->flags.b_Swallow) /* Normally no padding for these     */
     return 0;
   while((b=b->parent))
-    if(b->c->flags&b_Padding)
+    if(b->c->flags.b_Padding)
       return b->c->xpad;
 #ifdef DEBUG
   fprintf(stderr,"%s: BUG: No padding definition?\n",MyName);
@@ -189,12 +189,12 @@ int buttonXPad(button_info *b)
 **/
 int buttonYPad(button_info *b)
 {
-  if(b->flags&b_Padding)
+  if(b->flags.b_Padding)
     return b->ypad;
-  if(b->flags&(b_Container|b_Swallow))  /* Normally no padding for these     */
+  if(b->flags.b_Container || b->flags.b_Swallow) /* Normally no padding for these     */
     return 0;
   while((b=b->parent))
-    if(b->c->flags&b_Padding)
+    if(b->c->flags.b_Padding)
       return b->c->ypad;
 #ifdef DEBUG
   fprintf(stderr,"%s: BUG: No padding definition?\n",MyName);
@@ -206,49 +206,30 @@ int buttonYPad(button_info *b)
 *** buttonFont()
 *** Give the font pointer for this button
 **/
-XFontStruct *buttonFont(button_info *b)
+FlocaleFont *buttonFont(button_info *b)
 {
-  if(b->flags&b_Font)
-    return b->font;
+  if(b->flags.b_Font)
+    return b->Ffont;
   while((b=b->parent))
-    if(b->c->flags&b_Font)
-      return b->c->font;
+    if(b->c->flags.b_Font)
+      return b->c->Ffont;
 #ifdef DEBUG
   fprintf(stderr,"%s: BUG: No font definition?\n",MyName);
 #endif
   return None;
 }
 
-#ifdef I18N_MB
-/**
-*** buttonFontSet()
-*** Give the font pointer for this button
-**/
-XFontSet buttonFontSet(button_info *b)
-{
-  if(b->flags&b_Font)
-    return b->fontset;
-  while((b=b->parent))
-    if(b->c->flags&b_Font)
-      return b->c->fontset;
-#ifdef DEBUG
-  fprintf(stderr,"%s: BUG: No fontset definition?\n",MyName);
-#endif
-  return None;
-}
-#endif
-
 /**
 *** buttonFore()
 *** Give the foreground pixel of this button
 **/
-Pixel buttonFore(button_info *b)
+Pixel buttonFore(const button_info *b)
 {
-  if(b->flags&b_Fore)
+  if(b->flags.b_Fore)
     return b->fc;
   while ((b=b->parent))
   {
-    if(b->c->flags&b_Fore)
+    if(b->c->flags.b_Fore)
       return b->c->fc;
   }
 #ifdef DEBUG
@@ -261,13 +242,13 @@ Pixel buttonFore(button_info *b)
 *** buttonBack()
 *** Give the background pixel of this button
 **/
-Pixel buttonBack(button_info *b)
+Pixel buttonBack(const button_info *b)
 {
-  if(b->flags&b_Back)
+  if(b->flags.b_Back)
     return b->bc;
   while((b=b->parent))
   {
-    if(b->c->flags&b_Back)
+    if(b->c->flags.b_Back)
       return b->c->bc;
   }
 #ifdef DEBUG
@@ -282,11 +263,11 @@ Pixel buttonBack(button_info *b)
 **/
 Pixel buttonHilite(button_info *b)
 {
-  if(b->flags&b_Back)
+  if(b->flags.b_Back)
     return b->hc;
   while((b=b->parent))
   {
-    if(b->c->flags&b_Back)
+    if(b->c->flags.b_Back)
       return b->c->hc;
   }
 #ifdef DEBUG
@@ -301,11 +282,11 @@ Pixel buttonHilite(button_info *b)
 **/
 Pixel buttonShadow(button_info *b)
 {
-  if(b->flags&b_Back)
+  if(b->flags.b_Back)
     return b->sc;
   while((b=b->parent))
   {
-    if(b->c->flags&b_Back)
+    if(b->c->flags.b_Back)
       return b->c->sc;
   }
 #ifdef DEBUG
@@ -316,16 +297,135 @@ Pixel buttonShadow(button_info *b)
 
 int buttonColorset(button_info *b)
 {
-  if (b->flags & b_Colorset)
+  if (b->flags.b_Hangon)
+  {
+	if (b->flags.b_PressColorset)
+		return b->pressColorset;
+	if (UberButton->c->flags.b_PressColorset)
+		return UberButton->c->pressColorset;
+  }
+  else if (b == ActiveButton && b->flags.b_ActiveColorset)
+    return b->activeColorset;
+  else if (b == ActiveButton && UberButton->c->flags.b_ActiveColorset)
+    return UberButton->c->activeColorset;
+  else if (b == CurrentButton && b->flags.b_PressColorset)
+    return b->pressColorset;
+  else if (b == CurrentButton && UberButton->c->flags.b_PressColorset)
+    return UberButton->c->pressColorset;
+
+  if (b->flags.b_Colorset)
     return b->colorset;
+  else if (b->flags.b_Container && b->c->flags.b_Colorset)
+    return b->c->colorset;
   while ((b = b->parent))
   {
-    if (b->c->flags & b_Colorset)
+    if (b->c->flags.b_Colorset)
       return b->c->colorset;
   }
   return -1;
 }
 
+char *buttonTitle (button_info *b)
+{
+	if (b->flags.b_Hangon)
+	{
+	    if (b->flags.b_PressTitle)
+		return b->pressTitle;
+	}
+	/* If this is the current active button but no explicit ActiveTitle was
+	   specified, use the Title (if there is one).
+	   Similarly for PressTitle. */
+	else if (b == ActiveButton && b->flags.b_ActiveTitle)
+		return b->activeTitle;
+	else if (b == CurrentButton && b->flags.b_PressTitle)
+		return b->pressTitle;
+
+	if (b->flags.b_Title)
+		return b->title;
+
+	return NULL;
+}
+
+FvwmPicture *buttonIcon (button_info *b)
+{
+	if (b->flags.b_Hangon)
+        {
+		if (b->flags.b_PressIcon)
+			return b->pressicon;
+	}
+	else if (b == ActiveButton && b->flags.b_ActiveIcon)
+		return b->activeicon;
+	else if (b == CurrentButton && b->flags.b_PressIcon)
+		return b->pressicon;
+
+	/* b->icon == None if no icon specified */
+	return b->icon;
+}
+
+unsigned short iconFlagSet (button_info *b)
+{
+	if (b->flags.b_Hangon)
+        {
+                if (b->flags.b_PressIcon)
+                        return 1;
+        }
+        else if (b == ActiveButton && b->flags.b_ActiveIcon)
+                return 1;
+        else if (b == CurrentButton && b->flags.b_PressIcon)
+                return 1;
+
+        return (b->flags.b_Icon ? 1 : 0);
+}
+
+int buttonBackgroundButton(button_info *b, button_info **r_b)
+{
+	button_info *tmpb, *pb;
+	Bool done = False;
+
+	tmpb = pb = b;
+	while(tmpb->parent != NULL && !done)
+	{
+		pb = tmpb;
+		if (tmpb->flags.b_Container)
+		{
+			if(tmpb->c->flags.b_Colorset)
+			{
+				done = True;
+			}
+			else if (!(tmpb->c->flags.b_IconParent) &&
+				 !(tmpb->c->flags.b_ColorsetParent))
+			{
+				done = True;
+			}
+
+		}
+		else if (tmpb->flags.b_Swallow && buttonSwallowCount(b) == 3 &&
+			 tmpb->flags.b_Colorset)
+		{
+			done = True;
+		}
+		else if (tmpb->flags.b_Colorset)
+		{
+			done = True;
+		}
+		else if (!tmpb->flags.b_IconBack && !tmpb->flags.b_IconParent
+			 && !tmpb->flags.b_Swallow &&
+			 !tmpb->flags.b_ColorsetParent)
+		{
+			done = True;
+		}
+		tmpb = tmpb->parent;
+	}
+
+	if (done)
+	{
+		if (r_b != NULL)
+			*r_b = pb;
+		return True;
+	}
+	return False;
+}
+
 
 /**
 *** buttonSwallow()
@@ -334,14 +434,14 @@ int buttonColorset(button_info *b)
 byte buttonSwallow(button_info *b)
 {
   byte s=0,t=0;
-  if(b->flags & (b_Swallow | b_Panel))
+  if(b->flags.b_Swallow || b->flags.b_Panel)
   {
     s=b->swallow;
     t=b->swallow_mask;
   }
   while((b=b->parent))
   {
-    if(b->c->flags & (b_Swallow | b_Panel))
+    if(b->c->flags.b_Swallow || b->c->flags.b_Panel)
     {
       s&=~(b->c->swallow_mask&~t);
       s|=(b->c->swallow&b->c->swallow_mask&~t);
@@ -358,14 +458,14 @@ byte buttonSwallow(button_info *b)
 byte buttonJustify(button_info *b)
 {
   byte j=1,i=0;
-  if(b->flags&b_Justify)
+  if(b->flags.b_Justify)
   {
     i=b->justify_mask;
     j=b->justify;
   }
   while((b=b->parent))
   {
-    if(b->c->flags&b_Justify)
+    if(b->c->flags.b_Justify)
     {
       j&=~(b->c->justify_mask&~i);
       j|=(b->c->justify&b->c->justify_mask&~i);
@@ -430,7 +530,6 @@ button_info *alloc_button(button_info *ub,int num)
   ub->c->buttons[num]=b;
 
   memset((void *)b, 0, sizeof(*b));
-  b->flags = 0;
   b->swallow = 0;
   b->BWidth = b->BHeight = 1;
   b->BPosX = b->BPosY = 0;
@@ -446,11 +545,13 @@ button_info *alloc_button(button_info *ub,int num)
   b->h=1;
   b->bw=1;
 
+#if 0
   if(b->parent != NULL)
   {
     if (b->parent->c->flags & b_Colorset)
       b->flags |= b_ColorsetParent;
   }
+#endif
 
   return(b);
 }
@@ -463,17 +564,22 @@ void MakeContainer(button_info *b)
 {
   b->c=(container_info*)mymalloc(sizeof(container_info));
   memset((void *)b->c, 0, sizeof(container_info));
-  b->flags|=b_Container;
+  b->flags.b_Container = 1;
   if(b->parent != NULL)
   {
-    if (b->parent->c->flags & b_IconBack || b->parent->c->flags & b_IconParent)
-      b->c->flags |= b_IconParent;
-    if (b->parent->c->flags & b_Colorset)
-      b->c->flags |= b_ColorsetParent;
+    if (b->parent->c->flags.b_IconBack || b->parent->c->flags.b_IconParent)
+      b->c->flags.b_IconParent = 1;
+    if (b->parent->c->flags.b_Colorset ||
+	b->parent->c->flags.b_ColorsetParent)
+      b->c->flags.b_ColorsetParent = 1;
   }
   else /* This applies to the UberButton */
   {
-    b->c->flags=b_Font|b_Padding|b_Frame|b_Back|b_Fore;
+    b->c->flags.b_Font = 1;
+    b->c->flags.b_Padding = 1;
+    b->c->flags.b_Frame = 1;
+    b->c->flags.b_Back = 1;
+    b->c->flags.b_Fore = 1;
     b->c->font_string=safestrdup("fixed");
     b->c->xpad=2;
     b->c->ypad=4;
@@ -497,7 +603,7 @@ void NumberButtons(button_info *b)
     if(b->c->buttons[i])
     {
       b->c->buttons[i]->n=i;
-      if(b->c->buttons[i]->flags&b_Container)
+      if(b->c->buttons[i]->flags.b_Container)
 	NumberButtons(b->c->buttons[i]);
     }
 }
@@ -521,7 +627,7 @@ char PlaceAndExpandButton(int x, int y, button_info *b, button_info *ub)
   }
   if (y>=c->num_rows || y<0)
   {
-    if (b->flags&b_PosFixed || !(ub->c->flags&b_SizeSmart) || y<0)
+    if (b->flags.b_PosFixed || !ub->c->flags.b_SizeSmart || y<0)
     {
       fprintf(stderr,"%s: Button out of vertical range. Quitting.\n",
 	      MyName);
@@ -542,7 +648,7 @@ char PlaceAndExpandButton(int x, int y, button_info *b, button_info *ub)
   }
   if(y+b->BHeight>c->num_rows)
   {
-    if (c->flags&b_SizeSmart)
+    if (c->flags.b_SizeSmart)
     {
       c->num_rows=y+b->BHeight;
       c->num_buttons=c->num_columns*c->num_rows;
@@ -631,7 +737,7 @@ void ShuffleButtons(button_info *ub)
   for(i=0;i<num_items;i++)
     actual_buttons_used+=local_buttons[i]->BWidth*local_buttons[i]->BHeight;
 
-  if (!(c->flags&b_SizeFixed)||!(c->num_rows)||!(c->num_columns))
+  if (!c->flags.b_SizeFixed||!(c->num_rows)||!(c->num_columns))
   {
     /* Size and create the window */
     if(c->num_rows==0 && c->num_columns==0)
@@ -642,7 +748,7 @@ void ShuffleButtons(button_info *ub)
       c->num_rows=1+(actual_buttons_used-1)/c->num_columns;
     while(c->num_rows * c->num_columns < actual_buttons_used)
       c->num_columns++;
-    if (!(c->flags&b_SizeFixed))
+    if (!c->flags.b_SizeFixed)
     {
       while(c->num_rows*c->num_columns >=
 	    actual_buttons_used + c->num_columns)
@@ -650,7 +756,7 @@ void ShuffleButtons(button_info *ub)
     }
   }
 
-  if (c->flags&b_SizeSmart)
+  if (c->flags.b_SizeSmart)
   {
     /* Set rows/columns to at least the height/width of largest button */
     for(i=0;i<num_items;i++)
@@ -660,13 +766,13 @@ void ShuffleButtons(button_info *ub)
 	c->num_rows=b->BHeight;
       if (c->num_columns<b->BWidth)
 	c->num_columns=b->BWidth;
-      if (b->flags&b_PosFixed && c->num_columns<b->BWidth+b->BPosX)
+      if (b->flags.b_PosFixed && c->num_columns<b->BWidth+b->BPosX)
 	c->num_columns=b->BWidth+b->BPosX;
-      if (b->flags&b_PosFixed && c->num_columns<b->BWidth-b->BPosX)
+      if (b->flags.b_PosFixed && c->num_columns<b->BWidth-b->BPosX)
 	c->num_columns=b->BWidth-b->BPosX;
-      if (b->flags&b_PosFixed && c->num_rows<b->BHeight+b->BPosY)
+      if (b->flags.b_PosFixed && c->num_rows<b->BHeight+b->BPosY)
 	c->num_rows=b->BHeight+b->BPosY;
-      if (b->flags&b_PosFixed && c->num_rows<b->BHeight-b->BPosY)
+      if (b->flags.b_PosFixed && c->num_rows<b->BHeight-b->BPosY)
 	c->num_rows=b->BHeight-b->BPosY;
     }
   }
@@ -680,7 +786,7 @@ void ShuffleButtons(button_info *ub)
   {
     b=local_buttons[i];
     /* Shuffle subcontainers recursively */
-    if(b && b->flags&b_Container)
+    if(b && b->flags.b_Container)
       ShuffleButtons(b);
   }
 
@@ -688,7 +794,7 @@ void ShuffleButtons(button_info *ub)
   for(i=0;i<num_items;i++)
   {
     b=local_buttons[i];
-    if (!(b->flags&b_PosFixed)) continue;
+    if (!b->flags.b_PosFixed) continue;
     /* recalculate position for negative offsets */
     if (b->BPosX<0) b->BPosX=b->BPosX+c->num_columns-b->BWidth+1;
     if (b->BPosY<0) b->BPosY=b->BPosY+c->num_rows-b->BHeight+1;
@@ -706,7 +812,7 @@ void ShuffleButtons(button_info *ub)
   for(i=0;i<num_items;i++)
   {
     b=local_buttons[i];
-    if (b->flags&b_PosFixed) continue;
+    if (b->flags.b_PosFixed) continue;
 
     if (next_button_x+b->BWidth>c->num_columns)
     {
@@ -776,7 +882,7 @@ button_info *NextButton(button_info **ub,button_info **b,int *i,int all)
   *b=(*ub)->c->buttons[*i];
 
   /* Found new container */
-  if((*b)->flags & b_Container)
+  if((*b)->flags.b_Container)
   {
     *i=-1;
     *ub=*b;
@@ -792,7 +898,7 @@ button_info *NextButton(button_info **ub,button_info **b,int *i,int all)
 /**
 *** button_belongs_to()
 *** Function that finds out which button a given position belongs to.
-*** Returns -1 is not part of any, button if a proper button.
+*** Returns -1 if not part of any, button if a proper button.
 **/
 int button_belongs_to(button_info *ub,int button)
 {
@@ -818,6 +924,28 @@ int button_belongs_to(button_info *ub,int button)
   return -1;
 }
 
+/**
+*** get_xy_button()
+*** Function that returns the button that covers the given row/column in the
+*** button matrix. Returns NULL if none.
+**/
+button_info *get_xy_button(button_info *ub, int row, int column)
+{
+	int i;
+
+	if (!ub->flags.b_Container)
+	{
+		return NULL;
+	}
+	i = button_belongs_to(ub, column + row * ub->c->num_columns);
+	if (i == -1)
+	{
+		return NULL;
+	}
+
+	return ub->c->buttons[i];
+}
+
 /**
 *** select_button()
 *** Given (x,y) and uberbutton, returns pointer to referred button, or NULL
@@ -829,7 +957,7 @@ button_info *select_button(button_info *ub,int x,int y)
   int column;
   button_info *b;
 
-  if (!(ub->flags&b_Container))
+  if (!ub->flags.b_Container)
     return ub;
 
   x -= buttonXPad(ub) + buttonFrame(ub);
@@ -849,3 +977,19 @@ button_info *select_button(button_info *ub,int x,int y)
       b, x + ub->c->xpos - buttonXPos(b, i),
       y + ub->c->ypos - buttonYPos(b, i));
 }
+
+void get_button_root_geometry(rectangle *r, const button_info *b)
+{
+	int x;
+	int y;
+	int f;
+	Window win;
+
+	r->width = buttonWidth(b);
+	r->height = buttonHeight(b);
+	buttonInfo(b, &r->x, &r->y, &x, &y, &f);
+	XTranslateCoordinates(
+		Dpy, MyWindow, Root, r->x, r->y, &r->x, &r->y, &win);
+
+	return;
+}
