@@ -1,3 +1,4 @@
+/* -*-c-*- */
 /* FvwmTaskBar Module for Fvwm.
  *
  *  Copyright 1994,  Mike Finger (mfinger@mermaid.micro.umn.edu or
@@ -31,7 +32,6 @@
 
 #include "config.h"
 
-#include <stdlib.h>
 #include <stdio.h>
 #include <X11/Xlib.h>
 
@@ -54,6 +54,7 @@ extern GC shadow, hilite, graph, whitegc, blackgc, checkered, icongraph,
 extern GC iconbackgraph;
 extern GC focusbackgraph;
 extern FlocaleWinString *FwinString;
+extern int pad;
 extern int button_width;
 extern int iconcolorset;
 extern int focuscolorset;
@@ -69,11 +70,9 @@ extern int NRows, RowHeight;
 
 #define MIN_BUTTON_SIZE 32 /*....*/
 
-/*************************************************************************
- *                                                                       *
- *  Button handling functions and procedures                             *
- *                                                                       *
- *************************************************************************/
+/*
+ * Button handling functions and procedures
+ */
 
 /* Draws a 3D rectangle */
 void Draw3dRect(
@@ -262,6 +261,10 @@ void ButtonDraw(Button *button, int x, int y, int w, int h, XEvent *evp)
 	XRectangle rect,inter;
 	Region t_region;
 
+	if (w <= 0)
+	{
+		w = 1;
+	}
 	if (button == NULL)
 	{
 		return;
@@ -360,7 +363,7 @@ void ButtonDraw(Button *button, int x, int y, int w, int h, XEvent *evp)
 	{
 		return;
 	}
-	
+
 	t_region = XCreateRegion();
 	XUnionRectWithRegion (&rect, t_region, t_region);
 	/* see if we have the place for "...", if not for "..", if not for "."
@@ -374,7 +377,10 @@ void ButtonDraw(Button *button, int x, int y, int w, int h, XEvent *evp)
 			t3p = ".\0";
 			w3p = FlocaleTextWidth(Ffont, t3p, 1);
 			if ((newx + w3p + 2) >= w)
+			{
+				XDestroyRegion(t_region);
 				return;
+			}
 		}
 	}
 
@@ -465,11 +471,9 @@ char *ButtonName(Button *button)
 }
 
 
-/*************************************************************************
- *                                                                       *
- *  ButtonArray handling functions and procedures                        *
- *                                                                       *
- *************************************************************************/
+/*
+ *  ButtonArray handling functions and procedures
+ */
 
 /* -------------------------------------------------------------------------
    InitArray - Initialize the arrary of buttons
@@ -698,7 +702,7 @@ void DrawButtonArray(ButtonArray *array, int all, XEvent *evp)
 		}
 		if (temp->needsupdate || all)
 			ButtonDraw(
-				temp, x + array->x, y, array->tw-3, array->h,
+				temp, x + array->x, y, array->tw-pad, array->h,
 				(temp->needsupdate)? NULL:evp);
 		x += array->tw;
 	}
@@ -724,7 +728,7 @@ void RadioButton(ButtonArray *array, int butnum, int state)
   }
 }
 /* -------------------------------------------------------------------------
-   
+
    ------------------------------------------------------------------------- */
 Bool CheckRootTransparentButtons(ButtonArray *array)
 {
@@ -823,7 +827,8 @@ void ButtonCoordinates(ButtonArray *array, int numbut, int *xc, int *yc)
 	*yc = y;
 }
 
-void ButtonDimensions(ButtonArray *array, int *width, int *height)
+void ButtonDimensions(ButtonArray *array, unsigned int *width,
+		      unsigned int *height)
 {
 	*width = array->tw;
 	*height = RowHeight+2;
