@@ -1,3 +1,4 @@
+/* -*-c-*- */
 /* FvwmTaskBar Module for Fvwm.
  *
  * (Much reworked version of FvwmWinList)
@@ -62,8 +63,6 @@
 #include <sys/bsdtypes.h> /* Saul */
 #endif /* Saul */
 
-#include <stdlib.h>
-
 #include "libs/Module.h"
 #include "libs/fvwmlib.h"  /* for pixmaps routines */
 #include "libs/FScreen.h"
@@ -130,6 +129,7 @@ GC      focushilite = None;
 GC      blackgc = None;
 GC      whitegc = None;
 GC      checkered = None;
+int     pad = 3;
 int     colorset = -1;
 int     iconcolorset = -1;
 int     focuscolorset = -1;
@@ -141,26 +141,25 @@ Atom MwmAtom = None;
 static Bool is_dead_pipe = False;
 
 /* Module related information */
-char *Module;
+ModuleArgs *module;
 int  win_width    = 5,
-     win_height   = 5,
-     win_grav,
-     win_x,
-     win_y,
-     win_border = 4,
-     win_has_title = 0,
-     win_title_dir = 0,
-     win_title_height = 0,
-     win_is_shaded = 0,
-     button_width = DEFAULT_BTN_WIDTH,
-     Clength,
-     ButPressed   = -1,
-     ButReleased  = -1,
-     Checked      = 0,
-     WindowState  = -2, /* -2 unmaped, 1 not hidden, -1 hidden,
-			 *  0 hidden -> not hidden (for the events loop) */
-     FocusInWin = 0;    /* 1 if the Taskbar has the focus */
-
+	win_height   = 5,
+	win_grav,
+	win_x,
+	win_y,
+	win_border = 4,
+	win_has_title = 0,
+	win_title_dir = 0,
+	win_title_height = 0,
+	win_is_shaded = 0,
+	button_width = DEFAULT_BTN_WIDTH,
+	ButPressed   = -1,
+	ButReleased  = -1,
+	Checked      = 0,
+	WindowState  = -2, /* -2 unmaped, 1 not hidden, -1 hidden,
+			    *  0 hidden -> not hidden (for the events loop) */
+	FocusInWin = 0;    /* 1 if the Taskbar has the focus */
+direction_t win_shade_direction = 0;
 static int auto_stick_y = 0;
 
 static volatile sig_atomic_t AlarmSet = NOT_SET;
@@ -172,12 +171,12 @@ static volatile sig_atomic_t stick_taskbar_alarm = False;
 int UpdateInterval = 30;
 
 int whichButton = -1;
-Bool startButtonPressed = FALSE;
+Bool startButtonPressed = False;
 
 ButtonArray buttons;
 List windows;
 
-char *ClickAction[max(NUMBER_OF_MOUSE_BUTTONS, 3)] =
+char *ClickAction[max(NUMBER_OF_EXTENDED_MOUSE_BUTTONS, 3)] =
 {
   DEFAULT_CLICK1,
   DEFAULT_CLICK2,
@@ -239,7 +238,6 @@ extern int WindowButtonsRightMargin;
 char *ImagePath = NULL;
 char *XineramaConfig = NULL;
 static int fscreen = 0;
-int ColorLimit = 0;
 
 static void ParseConfig( void );
 static void ParseConfigLine(char *tline);
@@ -254,58 +252,36 @@ static Bool change_colorset(int cset, Bool force);
 static void WarpTaskBar(int y);
 int IsItemIndexIconSuppressed(List *list, int i);
 
-/******************************************************************************
+/*
   Main - Setup the XConnection,request the window list and loop forever
     Based on main() from FvwmIdent:
       Copyright 1994, Robert Nation and Nobutaka Suzuki.
-******************************************************************************/
+*/
 int main(int argc, char **argv)
 {
-  const char *temp;
-  char *s;
   int i;
 
   FlocaleInit(LC_CTYPE, "", "", "FvwmTaskBar");
   setlocale(LC_TIME, "");
   FGettextInit("FvwmTaskBar", LOCALEDIR, "FvwmTaskBar");
 
-  for (i = 3; i < NUMBER_OF_MOUSE_BUTTONS; i++)
+  for (i = 3; i < NUMBER_OF_EXTENDED_MOUSE_BUTTONS; i++)
   {
     ClickAction[i] = DEFAULT_CLICK_N;
   }
 
-  /* Save the program name for error messages and config parsing */
-  temp = argv[0];
-  s=strrchr(argv[0], '/');
-  if (s != NULL)
-    temp = s + 1;
-
-
-  if((argc != 6)&&(argc != 7)) {
-    fprintf(stderr,"%s Version %s should only be executed by fvwm!\n",temp,
+  module = ParseModuleArgs(argc,argv,1); /* use alias if provided */
+  if (module==NULL)
+  {
+    fprintf(stderr,
+            "FvwmTaskBar Version %s should only be executed by fvwm!\n",
 	    VERSION);
     exit(1);
   }
 
-  /* alise support */
-  if (argc == 7 && argv[6] != NULL)
-  {
-    Module = safemalloc(strlen(argv[6])+2);
-    strcpy(Module,"*");
-    strcat(Module, argv[6]);
-    Clength = strlen(Module);
-  }
-  else
-  {
-    Module = safemalloc(strlen(temp)+2);
-    strcpy(Module,"*");
-    strcat(Module, temp);
-    Clength = strlen(Module);
-  }
-
   /* setup fvwm pipes */
-  Fvwm_fd[0] = atoi(argv[1]);
-  Fvwm_fd[1] = atoi(argv[2]);
+  Fvwm_fd[0] = module->to_fvwm;
+  Fvwm_fd[1] = module->from_fvwm;
 
 #ifdef HAVE_SIGACTION
   {
@@ -389,16 +365,16 @@ int main(int argc, char **argv)
 #ifdef FVWM_DEBUG_MSGS
   if ( isTerminated )
   {
-    fprintf(stderr, "%s: Received signal: exiting...\n", Module);
+    fprintf(stderr, "%s: Received signal: exiting...\n", module->name);
   }
 #endif
   return 0;
 }
 
 
-/******************************************************************************
+/*
   EndLessLoop -  Read and redraw until we get killed, blocking when can't read
-******************************************************************************/
+*/
 void EndLessLoop(void)
 {
   fd_set readset;
@@ -438,11 +414,11 @@ void EndLessLoop(void)
 }
 
 
-/******************************************************************************
+/*
   ReadFvwmPipe - Read a single message from the pipe from Fvwm
     Originally Loop() from FvwmIdent:
       Copyright 1994, Robert Nation and Nobutaka Suzuki.
-******************************************************************************/
+*/
 void ReadFvwmPipe(void)
 {
     FvwmPacket* packet = ReadFvwmPacket(Fvwm_fd[1]);
@@ -452,11 +428,11 @@ void ReadFvwmPipe(void)
 	ProcessMessage( packet->type, packet->body );
 }
 
-/******************************************************************************
+/*
   ProcessMessage - Process the message coming from Fvwm
     Skeleton based on processmessage() from FvwmIdent:
       Copyright 1994, Robert Nation and Nobutaka Suzuki.
-******************************************************************************/
+*/
 void ProcessMessage(unsigned long type,unsigned long *body)
 {
   int redraw=-1;
@@ -502,20 +478,39 @@ void ProcessMessage(unsigned long type,unsigned long *body)
        2. be carful with dynamic style change */
     if (cfgpacket->w == win)
     {
-      if (win_border != (short)cfgpacket->border_width ||
-	  win_title_height != (int)cfgpacket->title_height ||
-	  win_has_title != HAS_TITLE(cfgpacket) ||
-	  win_title_dir != GET_TITLE_DIR(cfgpacket))
+      int nb;
+      int nh;
+      int nw;
+      int ntd;
+      int nht;
+      int nth;
+
+      nb = (short)cfgpacket->border_width;
+      nw = screen_g.width - (win_border<<1);
+      nh = (int)cfgpacket->frame_height - (win_border<<1);
+      nht = HAS_TITLE(cfgpacket);
+      ntd = GET_TITLE_DIR(cfgpacket);
+      nth = (int)cfgpacket->title_height;
+      if (ntd == DIR_S || ntd == DIR_N)
+      {
+	nh -= nth;
+      }
+      else
+      {
+	nw -= nth;
+      }
+      if (nb != win_border || nw != win_width || nh != win_height ||
+	  nht != win_has_title || ntd != win_title_dir ||
+	  nth != win_title_height)
       {
 	XSizeHints hints;
 	long dumy;
 
-	win_border = (short)cfgpacket->border_width;
-	win_width = screen_g.width-(win_border<<1);
-	win_title_height = (int)cfgpacket->title_height;
-	win_has_title = HAS_TITLE(cfgpacket);
-	win_title_dir = GET_TITLE_DIR(cfgpacket);
-
+	win_border = nb;
+	win_width = nw;
+	win_has_title = nht;
+	win_title_dir = ntd;
+	win_title_height = nth;
 	if (AutoStick)
 	{
 	  win_x = screen_g.x + win_border;
@@ -558,13 +553,18 @@ void ProcessMessage(unsigned long type,unsigned long *body)
 	hints.base_width  = win_width;
 	hints.max_width   = win_width;
 
+	/* hack to prevent mapping on wrong screen with StartsOnScreen */
+	FScreenMangleScreenIntoUSPosHints(fscreen, &hints);
 	XSetWMNormalHints(dpy,win,&hints);
 
 	if (AutoStick)
+	{
 	  XMoveResizeWindow(dpy, win, win_x, win_y, win_width, win_height);
-	else
+	}
+	else if (!IS_SHADED(cfgpacket))
+	{
 	  XResizeWindow(dpy, win, win_width, win_height);
-
+	}
 	UpdateArray(&buttons, -1, -1,
 		    win_width - stwin_width - StartAndLaunchButtonsWidth
 		    - WindowButtonsLeftMargin - WindowButtonsRightMargin,
@@ -608,6 +608,8 @@ void ProcessMessage(unsigned long type,unsigned long *body)
 	win_y += screen_g.y;
 	XMoveWindow(dpy, win, win_x, win_y);
       }
+      win_is_shaded = IS_SHADED(cfgpacket);
+      win_shade_direction = SHADED_DIR(cfgpacket);
       break;
     }
 
@@ -799,14 +801,22 @@ void ProcessMessage(unsigned long type,unsigned long *body)
       Button *temp = find_n(&buttons, i);
       if (temp)
       {
+	Bool do_reshade = False;
+	direction_t shade_dir = win_shade_direction;
+	char buff[MAX_MODULE_INPUT_TEXT_LEN];
+
 	if (AnimCommand && (AnimCommand[0] != 0)
 	    && IsItemIndexIconSuppressed(&windows,i))
 	{
-	  char buff[MAX_MODULE_INPUT_TEXT_LEN];
 	  Window child;
 	  int x, y;
 	  int abs_x, abs_y;
 
+	  if (win_is_shaded)
+	  {
+		  SendText(Fvwm_fd, "WindowShade Off", win);
+		  do_reshade = True;
+	  }
 	  ButtonCoordinates(&buttons, i, &x, &y);
 	  XTranslateCoordinates(dpy, win, Root, x, y,
 				&abs_x, &abs_y, &child);
@@ -825,6 +835,13 @@ void ProcessMessage(unsigned long type,unsigned long *body)
 	temp->needsupdate = 1;
 	temp->iconified = iconified;
 	DrawButtonArray(&buttons, 0, NULL);
+	if (do_reshade)
+	{
+		sprintf(
+			buff,"WindowShade %s",
+			gravity_dir_to_string(shade_dir,""));
+		SendText(Fvwm_fd, buff, win);
+	}
       }
       if (AnimCommand && AnimCommand[0] != 0)
 	SendUnlockNotification(Fvwm_fd);
@@ -927,33 +944,35 @@ void redraw_buttons()
 }
 
 
-/***********************************************************************
+/*
   Detected a broken pipe - time to exit
     Based on DeadPipe() from FvwmIdent:
       Copyright 1994, Robert Nation and Nobutaka Suzuki.
- **********************************************************************/
-void DeadPipe(int nonsense)
+*/
+RETSIGTYPE DeadPipe(int nonsense)
 {
   is_dead_pipe = True;
   exit(1);
+  SIGNAL_RETURN;
 }
 
-/**********************************************************************
+/*
   Reentrant signal handler that will tell the application to quit.
     It is not safe to just call "exit()" here; instead we will
     just set a flag that will break the event-loop when the
     handler returns.
- **********************************************************************/
+*/
 static RETSIGTYPE
 TerminateHandler(int sig)
 {
   fvwmSetTerminate(sig);
+  SIGNAL_RETURN;
 }
 
 #if 0
-/******************************************************************************
+/*
   WaitForExpose - Used to wait for expose event so we don't draw too early
-******************************************************************************/
+*/
 void WaitForExpose(void)
 {
   XEvent Event;
@@ -967,9 +986,9 @@ void WaitForExpose(void)
 }
 #endif
 
-/******************************************************************************
+/*
   RedrawWindow - Update the needed lines and erase any old ones
-******************************************************************************/
+*/
 void RedrawWindow(int force, XEvent *evp)
 {
 	if (Tip.open)
@@ -1011,7 +1030,6 @@ static char *configopts[] =
   "imagepath",
   "colorset",
   XINERAMA_CONFIG_STRING,
-  "ColorLimit",
   NULL
 };
 
@@ -1046,6 +1064,7 @@ static char *moduleopts[] =
   "Rows",
   "PageOnly",
   "ScreenOnly",
+  "Pad",
   NULL
 };
 
@@ -1053,7 +1072,7 @@ void ParseConfig(void)
 {
   char *buf;
 
-  InitGetConfigLine(Fvwm_fd,Module);
+  InitGetConfigLine(Fvwm_fd,CatString3("*",module->name,0));
   while (GetConfigLine(Fvwm_fd,&buf), buf != NULL)
   {
     ParseConfigLine(buf);
@@ -1068,9 +1087,9 @@ static void ParseConfigLine(char *tline)
   while (isspace((unsigned char)*tline))
     tline++;
 
-  if (strncasecmp(tline, Module, Clength) != 0)
+  if (strncasecmp(tline, CatString3("*",module->name,0), module->namelen) != 0)
   {
-    /* Non module spcific option */
+    /* Non module specific options */
     index = GetTokenIndex(tline, configopts, -1, &rest);
     while (*rest && *rest != '\n' && isspace(*rest))
       rest++;
@@ -1093,9 +1112,6 @@ static void ParseConfigLine(char *tline)
       }
       CopyString(&XineramaConfig, rest);
       break;
-    case 3: /* ColorLimit */
-      sscanf(rest, "%d", &ColorLimit);
-      break;
     default:
       /* unknown option */
       break;
@@ -1103,8 +1119,8 @@ static void ParseConfigLine(char *tline)
   } /* if options */
   else
   {
-    /* option beginning with '*ModuleName' */
-    rest = tline + Clength;
+    /* options beginning with '*ModuleName' */
+    rest = tline + module->namelen+1;
     index = GetTokenIndex(rest, moduleopts, -1, &rest);
     while (*rest && *rest != '\n' && isspace(*rest))
       rest++;
@@ -1119,7 +1135,7 @@ static void ParseConfigLine(char *tline)
     case 2: /* Geometry */
       while (isspace((unsigned char)*rest) && *rest != '\n' && *rest != 0)
 	rest++;
-      UpdateString(&geometry, rest);
+      UpdateString(&geometry, PeekToken(rest, NULL));
       break;
     case 3: /* Fore */
       CopyString(&ForeColor, rest);
@@ -1130,8 +1146,11 @@ static void ParseConfigLine(char *tline)
       colorset = -1;
       break;
     case 5: /* Colorset */
-      colorset = -1;
       colorset = atoi(rest);
+      if (colorset < 0)
+      {
+	      colorset = -1;
+      }
       AllocColorset(colorset);
       break;
     case 6: /* IconFore */
@@ -1143,8 +1162,11 @@ static void ParseConfigLine(char *tline)
       iconcolorset = -1;
       break;
     case 8: /* IconColorset */
-      iconcolorset = -1;
       iconcolorset = atoi(rest);
+      if (iconcolorset < 0)
+      {
+	      iconcolorset = -1;
+      }
       AllocColorset(iconcolorset);
       break;
     case 9: /* Action */
@@ -1163,7 +1185,10 @@ static void ParseConfigLine(char *tline)
       AutoHide=True;
       AutoStick=True;
       VisiblePixels=atoi(rest);
-      if (VisiblePixels < 1) VisiblePixels = DEFAULT_VISIBLE_PIXELS;
+      if (VisiblePixels < 1)
+      {
+	      VisiblePixels = DEFAULT_VISIBLE_PIXELS;
+      }
       break;
     case 14: /* UseIconNames */
       UseIconNames=True;
@@ -1176,12 +1201,20 @@ static void ParseConfigLine(char *tline)
       break;
     case 17: /* UpdateInterval */
       UpdateInterval = atoi(rest);
+      if (UpdateInterval < 0)
+      {
+	      UpdateInterval = 30;
+      }
       break;
     case 18: /* HighlightFocus */
       HighlightFocus=True;
       break;
     case 19: /* ButtonWidth */
       button_width = atoi(rest);
+      if (button_width < 0)
+      {
+	      button_width = DEFAULT_BTN_WIDTH;
+      }
       break;
     case 20: /* NoIconAction */
       CopyString(&AnimCommand, rest);
@@ -1200,8 +1233,11 @@ static void ParseConfigLine(char *tline)
       focuscolorset = -1;
       break;
     case 24: /* FocusColorset */
-      focuscolorset = -1;
       focuscolorset = atoi(rest);
+      if (focuscolorset < 0)
+      {
+	      focuscolorset = -1;
+      }
       AllocColorset(focuscolorset);
       break;
     case 25: /* 3DFvwm */
@@ -1209,8 +1245,9 @@ static void ParseConfigLine(char *tline)
       break;
     case 26: /* Rows */
       RowsNumber = atoi(rest);
-      if (!(1 <= RowsNumber && RowsNumber <= 8)) {
-	RowsNumber = 1;
+      if (!(1 <= RowsNumber && RowsNumber <= 8))
+      {
+	      RowsNumber = 1;
       }
       break;
     case 27: /* PageOnly */
@@ -1219,20 +1256,28 @@ static void ParseConfigLine(char *tline)
     case 28: /* ScreenOnly */
       ScreenOnly=True;
       break;
+    case 29: /* Pad */
+      pad = atoi(rest);
+      if (pad < 0)
+      {
+	      pad = 3;
+      }
+      break;
     default:
       if (!GoodiesParseConfig(tline) &&
 	  !StartButtonParseConfig(tline))
       {
-	fprintf(stderr,"%s: unknown configuration option %s", Module, tline);
+	fprintf(stderr,"%s: unknown configuration option %s",
+                module->name, tline);
       }
       break;
     } /* switch */
   } /* else options */
 }
 
-/******************************************************************************
+/*
   Alarm - Handle a SIGALRM - used to implement timeout events
-******************************************************************************/
+*/
 static RETSIGTYPE
 Alarm(int nonsense)
 {
@@ -1253,11 +1298,12 @@ Alarm(int nonsense)
 #if !defined(HAVE_SIGACTION) && !defined(USE_BSD_SIGNALS)
   signal (SIGALRM, Alarm);
 #endif
+  SIGNAL_RETURN;
 }
 
-/******************************************************************************
+/*
   DoAlarmAction -
-******************************************************************************/
+*/
 void
 DoAlarmAction(void)
 {
@@ -1304,9 +1350,9 @@ DoAlarmAction(void)
   }
 }
 
-/******************************************************************************
+/*
   CheckForTip - determine when to popup the tip window
-******************************************************************************/
+*/
 void CheckForTip(int x, int y)
 {
   int  num, bx, by, trunc;
@@ -1347,13 +1393,16 @@ void CheckForTip(int x, int y)
   }
   else {
     ClearAlarm(SHOW_TIP);
-    if (Tip.open) ShowTipWindow(0);
+    if (Tip.open)
+    {
+      ShowTipWindow(0);
+    }
   }
 }
 
-/******************************************************************************
+/*
    HandleButtonRelease - Process a ButtonRelease X event
-******************************************************************************/
+*/
 void HandleButtonRelease(
 	XEvent *evp, Time *NewTimestamp, int *redraw)
 {
@@ -1421,7 +1470,7 @@ void HandleButtonRelease(
 	{
 		ButReleased = ButPressed; /* Avoid race fvwm pipe */
 		if (evp->xbutton.button >= 1 &&
-		    evp->xbutton.button <= NUMBER_OF_MOUSE_BUTTONS)
+		    evp->xbutton.button <= NUMBER_OF_EXTENDED_MOUSE_BUTTONS)
 		{
 			rectangle r;
 			Window tmpw;
@@ -1468,9 +1517,9 @@ void HandleButtonRelease(
 	*redraw = 0;
 }
 
-/******************************************************************************
+/*
   HandleEvents - Process an X event
-******************************************************************************/
+*/
 void HandleEvents(
 	XEvent *evp, Time *NewTimestamp, int *redraw, int *expose_redraw)
 {
@@ -1601,14 +1650,16 @@ void HandleEvents(
 					ItemID(&windows, num));
 		}
 
-		CheckForTip(evp->xmotion.x, evp->xmotion.y);
+		CheckForTip(evp->xcrossing.x, evp->xcrossing.y);
 		break;
 
 	case LeaveNotify:
 		*NewTimestamp = evp->xcrossing.time;
 		ClearAlarm(SHOW_TIP);
 		if (Tip.open)
+		{
 			ShowTipWindow(0);
+		}
 
 		if (AutoHide)
 			SetAlarm(HIDE_TASK_BAR);
@@ -1701,10 +1752,13 @@ void HandleEvents(
 		{
 			AdjustWindow(
 				evp->xconfigure.width, evp->xconfigure.height);
-			SetWindowBackground(
-				dpy, win, win_width, win_height,
-				&Colorset[colorset],
-				Pdepth, graph, True);
+			if (colorset >= 0)
+			{
+				SetWindowBackground(
+					dpy, win, win_width, win_height,
+					&Colorset[colorset], Pdepth, graph,
+					True);
+			}
 			*redraw = 1;
 		}
 		/* useful because of dynamic style change */
@@ -1744,9 +1798,9 @@ void HandleEvents(
 	XSync(dpy,0); /* needed (tips) */
 }
 
-/******************************************************************************
+/*
   LoopOnEvents - Process all the X events we get
-******************************************************************************/
+*/
 void LoopOnEvents(void)
 {
 	XEvent Event;
@@ -1776,21 +1830,25 @@ void LoopOnEvents(void)
 	return;
 }
 
-/***********************************
+/*
   AdjustWindow - Resize the window
-  **********************************/
+*/
 void AdjustWindow(int width, int height)
 {
   NRows = (height+2)/RowHeight;
+  if (NRows <= 0)
+  {
+    NRows = 1;
+  }
   win_height = height;
   win_width = width;
   ArrangeButtonArray(&buttons);
   change_colorset(0, False);
 }
 
-/******************************************************************************
+/*
   LinkAction - Link an response to a users action
-******************************************************************************/
+*/
 void LinkAction(const char *string)
 {
   const char *temp=string;
@@ -1803,7 +1861,7 @@ void LinkAction(const char *string)
     int i;
 
     i = sscanf(temp + 5, "%d%n", &b, &n);
-    if (i > 0 && b >=1 && b <= NUMBER_OF_MOUSE_BUTTONS)
+    if (i > 0 && b >=1 && b <= NUMBER_OF_EXTENDED_MOUSE_BUTTONS)
     {
       CopyString(&ClickAction[b - 1], temp + 5 + n);
     }
@@ -2033,9 +2091,9 @@ static Bool change_colorset(int cset, Bool force)
 }
 
 
-/******************************************************************************
+/*
   StartMeUp - Do X initialization things
-******************************************************************************/
+*/
 void StartMeUp(void)
 {
    XSizeHints hints;
@@ -2043,9 +2101,11 @@ void StartMeUp(void)
    int ret;
    int i;
    XSetWindowAttributes attr;
+   int wx;
+   int wy;
 
    if (!(dpy = XOpenDisplay(""))) {
-     fprintf(stderr,"%s: can't open display %s", Module,
+     fprintf(stderr,"%s: can't open display %s", module->name,
 	     XDisplayName(""));
      exit (1);
    }
@@ -2068,7 +2128,7 @@ void StartMeUp(void)
      UpdateString(&geometry, "+0-0");
    /* evaluate further down */
    ret = FScreenParseGeometryWithScreen(
-     geometry, &hints.x, &hints.y, (unsigned int *)&hints.width,
+     geometry, &wx, &wy, (unsigned int *)&hints.width,
      (unsigned int *)&hints.height, &fscreen);
    FScreenGetScrRect(
      NULL, fscreen,
@@ -2082,14 +2142,15 @@ void StartMeUp(void)
    if (selfont_string == NULL)
      selfont_string = font_string;
 
-   if ((FButtonFont = FlocaleLoadFont(dpy, font_string, Module)) == NULL)
+   if ((FButtonFont = FlocaleLoadFont(dpy, font_string,module->name)) == NULL)
    {
-     fprintf(stderr, "%s: Couldn't load font. Exiting!\n",Module);
+     fprintf(stderr, "%s: Couldn't load font. Exiting!\n",module->name);
      exit(1);
    }
-   if ((FSelButtonFont = FlocaleLoadFont(dpy, selfont_string, Module)) == NULL)
+   if ((FSelButtonFont = FlocaleLoadFont(dpy, selfont_string,module->name))
+                                                                   == NULL)
    {
-     fprintf(stderr, "%s: Couldn't load font. Exiting!\n",Module);
+     fprintf(stderr, "%s: Couldn't load font. Exiting!\n",module->name);
      exit(1);
    }
    LoadGoodiesFont();
@@ -2111,24 +2172,24 @@ void StartMeUp(void)
    {
      if (AutoStick)
      {
-       if (-hints.y < Midline)
-	 hints.y = screen_g.height - (AutoHide ? VISIBLE_PIXELS() : win_height);
+       if (-wy < Midline)
+	 wy = screen_g.height - (AutoHide ? VISIBLE_PIXELS() : win_height);
        else
-	 hints.y = AutoHide ? VISIBLE_PIXELS() - win_height : 0;
-       hints.y += screen_g.y;
+	 wy = AutoHide ? VISIBLE_PIXELS() - win_height : 0;
+       wy += screen_g.y;
      }
      else
      {
-       hints.y += screen_g.height - win_height;
+       wy += screen_g.height - win_height;
      }
    }
    else if (AutoStick)
    {
-     if (hints.y < Midline)
-       hints.y = AutoHide ? VISIBLE_PIXELS() - win_height : 0;
+     if (wy < Midline)
+       wy = AutoHide ? VISIBLE_PIXELS() - win_height : 0;
      else
-       hints.y = screen_g.height - (AutoHide ? VISIBLE_PIXELS() : win_height);
-     hints.y += screen_g.y;
+       wy = screen_g.height - (AutoHide ? VISIBLE_PIXELS() : win_height);
+     wy += screen_g.y;
    }
 
    if (ret & XNegative)
@@ -2148,7 +2209,7 @@ void StartMeUp(void)
 
    hints.flags=USPosition|PPosition|USSize|PSize|PResizeInc|
      PWinGravity|PMinSize|PMaxSize|PBaseSize;
-   hints.x           = screen_g.x;
+   wx           = screen_g.x;
    hints.width       = win_width;
    hints.height      = win_height;
    hints.width_inc   = 1;
@@ -2160,8 +2221,8 @@ void StartMeUp(void)
    hints.base_width  = win_width;
    hints.base_height = RowHeight;
 
-   win_x = hints.x + win_border;
-   win_y = hints.y;
+   win_x = wx + win_border;
+   win_y = wy;
 
    if(Pdepth < 2) {
      back = PictureWhitePixel();
@@ -2189,7 +2250,7 @@ void StartMeUp(void)
      (colorset >= 0) ? Colorset[colorset].bg : back;
    attr.border_pixel = 0;
    attr.colormap = Pcmap;
-   win=XCreateWindow(dpy,Root,hints.x,hints.y,hints.width,hints.height,0,Pdepth,
+   win=XCreateWindow(dpy,Root,wx,wy,hints.width,hints.height,0,Pdepth,
 		     InputOutput,Pvisual,CWBackPixel|CWBorderPixel|CWColormap,
 		     &attr);
 
@@ -2199,22 +2260,24 @@ void StartMeUp(void)
   {
     XTextProperty nametext;
     char *list[]={NULL,NULL};
-    list[0] = Module+1;
+    list[0] = module->name;
 
-    classhints.res_name=safestrdup(Module+1);
-    classhints.res_class=safestrdup("FvwmTaskBar");
+    classhints.res_name= module->name;
+    classhints.res_class= "FvwmTaskBar";
 
     if(!XStringListToTextProperty(list,1,&nametext))
     {
-      fprintf(stderr,"%s: Failed to convert name to XText\n",Module);
+      fprintf(stderr,"%s: Failed to convert name to XText\n",module->name);
       exit(1);
     }
+    /* hack to prevent mapping on wrong screen with StartsOnScreen */
+    FScreenMangleScreenIntoUSPosHints(fscreen, &hints);
     XSetWMProperties(dpy,win,&nametext,&nametext,
 		     NULL,0,&hints,NULL,&classhints);
     XFree(nametext.value);
   }
 
-   for (i = 1; i <= NUMBER_OF_MOUSE_BUTTONS; i++)
+   for (i = 1; i <= NUMBER_OF_EXTENDED_MOUSE_BUTTONS; i++)
    {
      XGrabButton(dpy,i,AnyModifier,win,True,GRAB_EVENTS,GrabModeAsync,
 		 GrabModeAsync,None,None);
@@ -2241,9 +2304,9 @@ void StartMeUp(void)
    atexit(ShutMeDown);
 }
 
-/******************************************************************************
+/*
   ShutMeDown - Do X client cleanup
-******************************************************************************/
+*/
 static void
 ShutMeDown(void)
 {
@@ -2260,16 +2323,16 @@ ShutMeDown(void)
 }
 
 
-/******************************************************************************
+/*
   ChangeWindowName - Self explanitory
     Original work from FvwmIdent:
       Copyright 1994, Robert Nation and Nobutaka Suzuki.
-******************************************************************************/
+*/
 void ChangeWindowName(char *str)
 {
   XTextProperty name;
   if (XStringListToTextProperty(&str,1,&name) == 0) {
-    fprintf(stderr,"%s: cannot allocate window name.\n",Module);
+    fprintf(stderr,"%s: cannot allocate window name.\n",module->name);
     return;
   }
   XSetWMName(dpy,win,&name);
@@ -2277,11 +2340,11 @@ void ChangeWindowName(char *str)
   XFree(name.value);
 }
 
-/**************************************************************************
+/*
  *
  * Sets mwm hints
  *
- *************************************************************************/
+ */
 /*
  *  Now, if we (hopefully) have MWW - compatible window manager ,
  *  say, mwm, ncdwm, or else, we will set useful decoration style.
@@ -2290,33 +2353,33 @@ void ChangeWindowName(char *str)
 
 void SetMwmHints(unsigned int value, unsigned int funcs, unsigned int input)
 {
-PropMwmHints prop;
+	PropMwmHints prop;
 
-  if (MwmAtom==None)
-    {
-      MwmAtom=XInternAtom(dpy,"_MOTIF_WM_HINTS",False);
-    }
-  if (MwmAtom!=None)
-    {
-      /* sh->mwm.decorations contains OR of the MWM_DECOR_XXXXX */
-      prop.decorations= value;
-      prop.functions = funcs;
-      prop.inputMode = input;
-      prop.flags =
-	MWM_HINTS_DECORATIONS | MWM_HINTS_FUNCTIONS | MWM_HINTS_INPUT_MODE;
+	if (MwmAtom==None)
+	{
+		MwmAtom=XInternAtom(dpy,"_MOTIF_WM_HINTS",False);
+	}
+	if (MwmAtom!=None)
+	{
+		/* sh->mwm.decorations contains OR of the MWM_DECOR_XXXXX */
+		prop.props[0] =
+			MWM_HINTS_DECORATIONS | MWM_HINTS_FUNCTIONS |
+			MWM_HINTS_INPUT_MODE;
+		prop.props[1] = funcs;
+		prop.props[2]= value;
+		prop.props[3] = input;
 
-      /* HOP - LA! */
-      XChangeProperty (dpy,win,
-		       MwmAtom, MwmAtom,
-		       32, PropModeReplace,
-		       (unsigned char *)&prop,
-		       PROP_MWM_HINTS_ELEMENTS);
-    }
+		/* HOP - LA! */
+		XChangeProperty(
+			dpy, win, MwmAtom, MwmAtom, 32, PropModeReplace,
+			(unsigned char *)&(prop.props[0]),
+			PROP_MWM_HINTS_ELEMENTS);
+	}
 }
 
-/***********************************************************************
+/*
  WarpTaskBar -- Enforce AutoStick feature
- ***********************************************************************/
+*/
 static void WarpTaskBar(int y)
 {
   /* The tests on y are really useful ! */
@@ -2342,10 +2405,10 @@ static void WarpTaskBar(int y)
   ClearAlarm(STICK_TASK_BAR);
 }
 
-/***********************************************************************
+/*
  SleepALittle -- do a small delay in a fairly portable way
  Replace "#if 1" with "#if 0" to turn delay off
- ***********************************************************************/
+*/
 static void SleepALittle(void)
 {
   struct timeval tv;
@@ -2359,9 +2422,9 @@ static void SleepALittle(void)
 #endif
 }
 
-/***********************************************************************
+/*
  RevealTaskBar -- Make taskbar fully visible
- ***********************************************************************/
+*/
 void RevealTaskBar()
 {
   int new_win_y;
@@ -2398,9 +2461,9 @@ void RevealTaskBar()
   WindowState = 0;
 }
 
-/***********************************************************************
+/*
  HideTaskbar -- Make taskbar partially visible
- ***********************************************************************/
+*/
 void HideTaskBar()
 {
   int new_win_y;
@@ -2459,9 +2522,9 @@ void HideTaskBar()
   WindowState = -1;
 }
 
-/***********************************************************************
+/*
  SetAlarm -- Schedule a timeout event
- ************************************************************************/
+*/
 static void
 SetAlarm(tb_alarm_t event)
 {
@@ -2470,9 +2533,9 @@ SetAlarm(tb_alarm_t event)
   alarm(1);
 }
 
-/***********************************************************************
+/*
  ClearAlarm -- Disable timeout events
- ************************************************************************/
+*/
 static void
 ClearAlarm(tb_alarm_t event)
 {
@@ -2483,9 +2546,9 @@ ClearAlarm(tb_alarm_t event)
   }
 }
 
-/************************************************************************
+/*
   X Error Handler
-************************************************************************/
+*/
 static int
 ErrorHandler(Display *d, XErrorEvent *event)
 {
@@ -2497,6 +2560,6 @@ ErrorHandler(Display *d, XErrorEvent *event)
   if (FRenderGetErrorCodeBase() + FRenderBadPicture == event->error_code)
     return 0;
 
-  PrintXErrorAndCoredump(d, event, Module);
+  PrintXErrorAndCoredump(d, event, module->name);
   return 0;
 }
