@@ -1,3 +1,4 @@
+/* -*-c-*- */
 /* icons.c. by Bo Yang.
  * Modifications: Copyright 1995 by Bo Yang.
  *
@@ -27,11 +28,11 @@
  * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  */
 
-/***********************************************************************
+/*
  *
  * Derived from fvwm icon code
  *
- ***********************************************************************/
+ */
 
 #include "config.h"
 #include "libs/fvwmlib.h"
@@ -42,7 +43,6 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <signal.h>
-#include <stdlib.h>
 
 #include <X11/Xlib.h>
 #include <X11/Xutil.h>
@@ -58,11 +58,11 @@
 #include "stepgfx.h"
 
 
-/****************************************************************************
+/*
  *
  * Loads an icon file into a pixmap
  *
- ****************************************************************************/
+ */
 Bool LoadIconFile(int button, int ico)
 {
 #ifndef NO_ICONS
@@ -79,9 +79,11 @@ Bool LoadIconFile(int button, int ico)
 	if (!PImageLoadPixmapFromFile(
 		dpy, main_win, path, &Buttons[button].icons[ico].icon,
 		&Buttons[button].icons[ico].mask,
-		&Buttons[button].icons[ico].alpha, &Buttons[button].icons[ico].w,
-		&Buttons[button].icons[ico].h, &Buttons[button].icons[ico].depth,
-		0, NULL, fpa))
+		&Buttons[button].icons[ico].alpha,
+		&Buttons[button].icons[ico].w,
+		&Buttons[button].icons[ico].h,
+		&Buttons[button].icons[ico].depth,
+		0, NULL, 0, fpa))
 	{
 		fprintf(stderr, "[FvwmWharf] cannot load pixmap from "
 			"file '%s'\n",path);
@@ -126,11 +128,11 @@ Bool LoadIconFile(int button, int ico)
 #endif
 }
 
-/****************************************************************************
+/*
  *
  * Creates an Icon Window
  *
- ****************************************************************************/
+ */
 void CreateIconWindow(int button, Window *win)
 {
 #ifndef NO_ICONS
@@ -155,11 +157,11 @@ void CreateIconWindow(int button, Window *win)
 #endif
 }
 
-/****************************************************************************
+/*
  *
  * Combines icon shape masks after a resize
  *
- ****************************************************************************/
+ */
 void ConfigureIconWindow(int button, int row, int column, int colorset)
 {
 #ifndef NO_ICONS
@@ -228,11 +230,11 @@ void ConfigureIconWindow(int button, int row, int column, int colorset)
 #endif
 }
 
-/*******************************************************************
+/*
  *
  * Make a gradient pixmap
  *
- *******************************************************************/
+ */
 
 int GetXPMGradient(int button, int from[3], int to[3], int maxcols,
 		   int type)
@@ -279,11 +281,11 @@ int GetXPMGradient(int button, int from[3], int to[3], int maxcols,
     return 1;
 }
 
-/*******************************************************************
+/*
  *
  * Make a colorset pixmap
  *
- *******************************************************************/
+ */
 
 void GetXPMColorset(int button, int colorset)
 {
@@ -310,11 +312,11 @@ void GetXPMColorset(int button, int colorset)
   return;
 }
 
-/*******************************************************************
+/*
  *
  * Make a solid color pixmap
  *
- *******************************************************************/
+ */
 
 int GetSolidXPM(int button, Pixel pixel)
 {
